/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

/*
 *  CApp overrides FXApp in order to intercept key events as early as
 *  possible and route them to the form painter rather than the intended
 *  destination.
 *
 *  CApp remembers important key state for use by the application.
 *
 *  This file is the only file in the application that contains platform
 *  specific code.
 */

#ifndef WIN32
#include <sys/time.h>    // allenb
#endif

#include "xincs.h"
#include "fxver.h"
#include "fxdefs.h"
#include "fxkeys.h"
#include "FXStream.h"
#include "FXString.h"
#include "FXObject.h"
#include "FXDict.h"
#include "FXRegistry.h"
#include "FXAccelTable.h"
#include "FXApp.h"
#include "FXId.h"
#include "FXDC.h"
#include "FXDCWindow.h"
#include "FXVisual.h"
#include "FXCursor.h"
#include "FXFont.h"
#include "FXDrawable.h"
#include "FXBitmap.h"
#include "FXImage.h"
#include "FXIcon.h"
#include "FXGIFIcon.h"
#include "FXWindow.h"
#include "FXFrame.h"
#include "FXString.h"
#include "FXLabel.h"
#include "FXButton.h"
#include "FXComposite.h"
#include "FXRootWindow.h"
#include "FXShell.h"
#include "FXTopWindow.h"
#include "FXDialogBox.h"
#include "FXMessageBox.h"
#include "FXProfiler.h"

#include "CApp.h"

extern FXAPI unsigned int fxmodifierkeys();

FXIMPLEMENT(CApp, FXApp, 0, 0) 

CApp::CApp(const FXchar* name, const FXchar* vendor) 
: FXApp(name, vendor)
{
	m_bAllowDelete = TRUE;
	m_bCtrlPressed = FALSE;
} 

CApp::~CApp()
{
	delete getMainWindow();
}

#ifndef WIN32
void CApp::dispatchEvent(FXRawEvent& ev)
{
	FXWindow *window;
	FXuint    ks1,ks2,state,code;

	int x,y;

	m_bCtrlPressed = (ev.xkey.state&CONTROLMASK) ? TRUE : FALSE;
	FXbool bShiftPressed = (ev.xkey.state&SHIFTMASK) ? TRUE : FALSE;

	// allenb - wcontext is private, added helper to FXApp
	// if(XFindContext((Display*)display,ev.xany.window,wcontext,(XPointer*)&window)==0)
	
//	if(XFindContext((Display*)display,ev.xany.window,getWContext(),(XPointer*)&window)==0)
//	if(XFindContext((Display*)display,ev.xany.window,wcontext,(XPointer*)&window)==0)	
//	if(XFindContext((Display*)display,ev.xany.window,findWindowWithId(ev.xany.window),(XPointer*)&window)==0)
	window = findWindowWithId(ev.xany.window);

	if ( window )
	{
		FXASSERT(window);

		if ( !isValidWidget(window) )
		{
			if (ev.xany.type == ButtonPress)
			{
				doResetAction();
			}

			FXApp::dispatchEvent(ev); 
			return; 
		} 
 
		state=ev.xkey.state;
		ks1=XLookupKeysym(&ev.xkey,0);
		ks2=XLookupKeysym(&ev.xkey,1);

		if((state&NUMLOCKMASK) && (KEY_KP_Space<=ks2) && (ks2<=KEY_KP_Equal))
		{
		  if(state&SHIFTMASK) code=ks1; else code=ks2;
		}
		else if((KEY_a<=ks1) && (ks1<=KEY_z))
		{            // Alpha shifted and/or caplocked
		  if(((state&CAPSLOCKMASK)!=0) ^ ((state&SHIFTMASK)!=0)) code=ks2; else code=ks1;
		}
		else if((state&SHIFTMASK) && (ks2!=0))
		{     // Simple shifted
		  code=ks2;
		}
		else
		{                                             // Unshifted
		 code=ks1;
		}

		if	(
			( m_bCtrlPressed && bShiftPressed ) ||
			( (code == KEY_Delete || code == KEY_KP_Delete) && !m_bAllowDelete )
			)
		{
			FXApp::dispatchEvent(ev); 
			return; 
		}

		// Translate it
		switch(ev.xany.type)
		{
		  // Motion
		case MotionNotify:
			x = ev.xmotion.x;
			y = ev.xmotion.y;
			if ( doMouseMove(window, x, y) )
			{
				return;
			}
			break;
    
		  // Button
		case ButtonPress:
			x = ev.xbutton.x;
			y = ev.xbutton.y;

			if ( doLButtonDown(window, x, y, m_bCtrlPressed) )
			{
				return;
			}
			break;

		case ButtonRelease:
			x = ev.xbutton.x;
			y = ev.xbutton.y;
			if ( doLButtonUp(window, x, y) )
			{
				return;
			}
			break;

		case KeyPress:
			state=ev.xkey.state;
			ks1=XLookupKeysym(&ev.xkey,0);
			ks2=XLookupKeysym(&ev.xkey,1);
			if((state&NUMLOCKMASK) && (KEY_KP_Space<=ks2) && (ks2<=KEY_KP_Equal))
			{
			  if(state&SHIFTMASK) code=ks1; else code=ks2;
			}
			else if((KEY_a<=ks1) && (ks1<=KEY_z))
			{            // Alpha shifted and/or caplocked
			  if(((state&CAPSLOCKMASK)!=0) ^ ((state&SHIFTMASK)!=0)) code=ks2; else code=ks1;
			}
			else if((state&SHIFTMASK) && (ks2!=0))
			{     // Simple shifted
			  code=ks2;
			}
			else
			{                                             // Unshifted
			 code=ks1;
			}

			doKeyDown(window, code, state);
			break;
		}
	}

	FXApp::dispatchEvent(ev);

	/*
	 *	TrackManager will use the KEYUP event to update properties
	 *	which needs to occur *after* dispatchevent so that update
	 *	will work on the new value not the old value.
	 */
	if ( ev.xany.type == KeyRelease )
	{
		state=ev.xkey.state;
		ks1=XLookupKeysym(&ev.xkey,0);
		ks2=XLookupKeysym(&ev.xkey,1);
		if((state&NUMLOCKMASK) && (KEY_KP_Space<=ks2) && (ks2<=KEY_KP_Equal))
		{
		  if(state&SHIFTMASK) code=ks1; else code=ks2;
		}
		else if((KEY_a<=ks1) && (ks1<=KEY_z))
		{// Alpha shifted and/or caplocked
		  if(((state&CAPSLOCKMASK)!=0) ^ ((state&SHIFTMASK)!=0)) code=ks2; else code=ks1;
		}
		else if((state&SHIFTMASK) && (ks2!=0))
		{// Simple shifted
		  code=ks2;
		}
		else
		{// Unshifted
		 code=ks1;
		}

		doKeyUp(window, code, state);
	}

	if ( ev.xany.type == Expose || ev.xany.type == GraphicsExpose )
	{
		doPaint(window); 
	}
}
#else

static FXuint winkey2foxkey(FXuint state,WPARAM wParam){
  FXuint i;
  
  // Patch from Alexander V. Voinov <avv@quasar.ipa.nw.ru>
  if(wParam >= 48 && wParam <= 255){ 
    HKL kl=GetKeyboardLayout(0);
    BYTE kstate[256];
    GetKeyboardState(kstate);
    unsigned short winchar=0;
    int ret=ToAsciiEx(wParam,0,kstate,&winchar,0,kl);
    if(ret>0){
      if((state&(CONTROLMASK|ALTMASK))==0){
        return winchar;
        }
      else{
        
        // Numbers
        static FXuint numbersmap[]={
          KEY_parenright,KEY_exclam,KEY_at,KEY_numbersign,KEY_dollar,KEY_percent,
          KEY_asciicircum,KEY_ampersand,KEY_asterisk,KEY_parenleft
          };

        // Letters
        if(wParam>=48&&wParam<=57){
          if(state&SHIFTMASK)
            return numbersmap[wParam-48];
          else
            return KEY_0+(wParam-48);
          }

        if(((state&CAPSLOCKMASK)!=0)^((state&SHIFTMASK)!=0))
          return wParam+(KEY_A-65);
        else
          return wParam+(KEY_a-KEY_A);
        }
      }
    }

//   // Letters
//   if(wParam>=65&&wParam<=90){
//     if(((state&CAPSLOCKMASK)!=0) ^ ((state&SHIFTMASK)!=0))
//       return wParam+(KEY_A-65);
//     else
//       return wParam+(KEY_a-KEY_A);
//     }
// 
//   // Numbers
//   FXuint numbersmap[]={
//     KEY_parenright,KEY_exclam,KEY_at,KEY_numbersign,KEY_dollar,KEY_percent,
//     KEY_asciicircum,KEY_ampersand,KEY_asterisk,KEY_parenleft
//     };
// 
//   if(wParam>=48&&wParam<=57){
//     if(state&SHIFTMASK)
//       return numbersmap[wParam-48];
//     else
//       return KEY_0+(wParam-48);
//     }

  // Keypad numbers
  if(wParam>=VK_NUMPAD0&&wParam<=VK_NUMPAD9){
    return KEY_KP_0+(wParam-VK_NUMPAD0);
    }

  // Undocumented keycodes; how portable???
  if(state&SHIFTMASK){
    FXuint codesmap1[]={
      0xC0, KEY_asciitilde, 0xBD, KEY_underscore, 0xBB, KEY_plus, 0xDB, KEY_braceleft,
      0xDD, KEY_braceright, 0xDC, KEY_bar, 0xBA, KEY_colon, 0xDE, KEY_quotedbl,
      0xBC, KEY_less, 0xBE, KEY_greater, 0xBF, KEY_question
      };
    for(i=0; i<ARRAYNUMBER(codesmap1)/2; i++){
      if(codesmap1[2*i]==wParam) return codesmap1[2*i+1];
      }
    }
  else{
    FXuint codesmap2[]={
      0xC0, KEY_grave, 0xBD, KEY_minus, 0xBB, KEY_equal, 0xDB, KEY_bracketleft,
      0xDD, KEY_bracketright, 0xDC, KEY_backslash, 0xBA, KEY_semicolon, 0xDE, KEY_apostrophe,
      0xBC, KEY_comma, 0xBE, KEY_period, 0xBF, KEY_slash
      };
    for(i=0; i<ARRAYNUMBER(codesmap2)/2; i++){
      if(codesmap2[2*i]==wParam) return codesmap2[2*i+1];
      }
    }

  // Function keys
  if(wParam>=VK_F1&&wParam<=VK_F16){
    return KEY_F1+(wParam-VK_F1);
    }

  // Left or Right Shift Key
  if(wParam==VK_SHIFT){
    return (GetKeyState(VK_LSHIFT)<0) ? KEY_Shift_L : KEY_Shift_R;
    }

  // Left or Right Alt key
  if(wParam==VK_MENU){
    return (GetKeyState(VK_RMENU)<0) ? KEY_Alt_L : KEY_Alt_R;
    }

  // Left or Right Ctrl key
  if(wParam==VK_CONTROL){
    return (GetKeyState(VK_LCONTROL)<0) ? KEY_Control_L : KEY_Control_R;
    }

  // Pause/Break key
  if(wParam==VK_PAUSE){
    return (GetKeyState(VK_SHIFT)<0) ? KEY_Pause : KEY_Break;
    }

  // Miscellaneous
  FXuint miscmap[]={
    VK_SPACE,     KEY_space,
    VK_BACK,      KEY_BackSpace,
    VK_TAB,       KEY_Tab,
    VK_RETURN,    KEY_Return,
    VK_SCROLL,    KEY_Scroll_Lock,
    VK_ESCAPE,    KEY_Escape,
    VK_INSERT,    KEY_Insert,
    VK_DELETE,    KEY_Delete,
    VK_HOME,      KEY_Home,
    VK_LEFT,      KEY_Left,
    VK_UP,        KEY_Up,
    VK_RIGHT,     KEY_Right,
    VK_DOWN,      KEY_Down,
    VK_PRIOR,     KEY_Prior,
    VK_NEXT,      KEY_Next,
    VK_END,       KEY_End,
    VK_CAPITAL,   KEY_Caps_Lock,
    VK_NUMLOCK,   KEY_Num_Lock,
    VK_MULTIPLY,  KEY_KP_Multiply,
    VK_ADD,       KEY_KP_Add,
    VK_SEPARATOR, KEY_KP_Separator,
    VK_SUBTRACT,  KEY_KP_Subtract,
    VK_DECIMAL,   KEY_KP_Decimal,
    VK_DIVIDE,    KEY_KP_Divide,
    VK_SNAPSHOT,  KEY_Print,
    };

  for(i=0;i<ARRAYNUMBER(miscmap)/2;i++){
    if(miscmap[2*i]==wParam) return miscmap[2*i+1];
    }

  return KEY_VoidSymbol;
  }


long CApp::dispatchEvent(FXID hwnd,unsigned int iMsg,unsigned int wParam,long lParam)
{
	FXWindow* window = 0;
	FXWindow* parent = 0;
	FXDCWindow* pDC  = 0;

	if(!IsWindow((HWND)hwnd))
		return DefWindowProc((HWND)hwnd,iMsg,wParam,lParam);

	window=(FXWindow*)GetWindowLong((HWND)hwnd,0);
	if(window==0 && iMsg!=WM_CREATE)
		return DefWindowProc((HWND)hwnd,iMsg,wParam,lParam);

	if ( iMsg != WM_KEYDOWN && iMsg != WM_KEYUP )
	{
		if ( !isValidWidget(window) )
		{
			if ( iMsg == WM_LBUTTONDOWN )
			{
				doResetAction();
			}

			return FXApp::dispatchEvent(hwnd, iMsg, wParam, lParam);
		}
	}

	FXuint code = fxmodifierkeys();
	FXuint key = winkey2foxkey(code,wParam);
	
	FXbool bCtrlReleased = FALSE;
	if ( m_bCtrlPressed && !(code&CONTROLMASK) && (iMsg == WM_KEYUP || iMsg == WM_SYSKEYUP) )
	{
		bCtrlReleased = TRUE;
	}

	m_bCtrlPressed = (code&CONTROLMASK) ? TRUE : FALSE;
	FXbool bShiftPressed = (code&SHIFTMASK) ? TRUE : FALSE;
	
	int x,y;

	if ( window != NULL )
	{
		if	(
			( m_bCtrlPressed && bShiftPressed ) ||
			( (key == KEY_Delete || key == KEY_KP_Delete) && !m_bAllowDelete ) 
			)
		{
			return FXApp::dispatchEvent(hwnd, iMsg, wParam, lParam);
		}

		switch(iMsg)
		{
			case WM_KEYDOWN:
			case WM_SYSKEYDOWN:
				if ( !isValidWidget(window) )
				{
					if ( doKeyDown(window, key, code) )
					{
						return 0;
					}
				}
				break;

			case WM_LBUTTONDOWN:
				x = (int)((short)LOWORD(lParam));
				y = (int)((short)HIWORD(lParam));
				
				if ( doLButtonDown(window, x, y, m_bCtrlPressed) )
				{
					return 1;
				}
				break;

			case WM_MOUSEMOVE:
				x = LOWORD(lParam);
				y = HIWORD(lParam);
				if ( doMouseMove(window, x, y) )
				{
	//				return 1;
				}
				break;

			case WM_LBUTTONUP:
				x = LOWORD(lParam); 
				y = HIWORD(lParam); 
				if ( doLButtonUp(window, x, y) )
				{
					return 1;
				}
				break;
		};
	}

	LRESULT r = FXApp::dispatchEvent(hwnd, iMsg, wParam, lParam);

	/*
	 *	TrackManager will use the KEYUP event to update properties
	 *	which needs to occur *after* dispatchevent so that update
	 *	will work on the new value not the old value.
	 */
	if ( bCtrlReleased )
		doKeyUp(window, key, code);

	if ( iMsg == WM_PAINT )
		doPaint(window);

	return r;
}
#endif


