/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

void CProperty::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	FXuint nOption  = getIntFromString(get(pWindow));
	OPTION* pOption = option();

	if ( pOption )
	{
		for (OPTION* p; p->pszName; p++)
		{
			if ( (nOption & p->value) == p->value )
			{
				if ( strOptions.length() > 0 )
				{
					strOptions += "|";
				}
				strOptions += p->pszName;
			}
		}
	}
}

FXbool CProperty::getProperty(const FXchar* pszName, IProperty** ppProperty)
{
	*ppProperty = &m_pDispenser->getProperty(pszName);
	
	if ( *ppProperty == 0 )
	{
		return FALSE;
	}

	return TRUE;
}

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyDummy : public CProperty
{
public:
	CPropertyDummy(IDispenser* pDispenser) : CProperty(pDispenser) {}

	virtual const char* name(void) {return "";}
	virtual ePropertyType type(void) {return PROPERTY_NONE;}
	virtual const char* description(void) {return "";}
	virtual void set(FXWindow* pWindow, FXString strValue) {}
	virtual FXString get(FXWindow* pWindow) {return "";}
};

IProperty& CDispenser::getProperty(const FXchar* pszName)
{
	FXint nCount = getPropertyCount();
	IProperty* p = 0;
	static CPropertyDummy d(this);
	
	for (FXint i = 0; i < nCount; i++)
	{
		p = getProperty(i);

		if ( !strcmp(pszName, p->name()) )
		{
			return *p;
		}
	}

	return d;
}

void toggleAttribute(FXint& nOldValue, FXint& nNewValue, FXint attributeA, FXint attributeB)
{
	// Has user turned off attributeA?
	if ( (nOldValue & attributeA) && !(nNewValue & attributeA))
	{
		// Yes. So turn on attributeB
		nNewValue |= attributeB;
	}

	// Has user turned on attributeA?
	if ( !(nOldValue & attributeA) && (nNewValue & attributeA))
	{
		// Yes. So turn off attributeB
		nNewValue &= ~attributeB;
	}

	// Has user turned off attributeB?
	if ( (nOldValue & attributeB) && !(nNewValue & attributeB))
	{
		// Yes. So turn on attributeA
		nNewValue |= attributeA;
	}

	// Has user turned on attributeB?
	if ( !(nOldValue & attributeB) && (nNewValue & attributeB))
	{
		// Yes. So turn off attributeA
		nNewValue &= ~attributeA;
	}
}

