/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>

#include <CFile.h>

CFile::CFile()
{
	m_pFile = NULL;
	m_bOpen = FALSE;

	m_nMaxPos = 0;
	m_nBufferLen = 1024;
	m_nPos = 0;
	m_nScanPos = 0;
}

CFile::~CFile()
{
	close();
}

FXbool CFile::openWrite(FXString& strFile, FXbool bEnd)
{
	close();

	memset(&m_szName[0], 0, sizeof(m_szName));
	memcpy(&m_szName[0], strFile.text(), strFile.length());

	m_bOpen = FALSE;
	m_pFile = fopen( strFile.text(), "wt" );

	if ( m_pFile != NULL )
	{
		if ( bEnd == TRUE )
		{
			fseek( m_pFile, 0L, SEEK_END );
		}

		m_bOpen = TRUE;
	}

	return m_bOpen;
}

FXbool CFile::openRead(FXString& strFile, FXbool bEnd)
{
	m_nMaxPos = 0;
	m_nBufferLen = 1024;
	m_nPos = 0;
	m_nScanPos = 0;

	close();

	memset(&m_szName[0], 0, sizeof(m_szName));
	memcpy(&m_szName[0], strFile.text(), strFile.length());

	m_bOpen = FALSE;
	m_pFile = fopen( strFile.text(), "rt" );

	if ( m_pFile != NULL )
	{
		if ( bEnd == TRUE )
		{
			fseek( m_pFile, 0L, SEEK_END );
		}

		m_bOpen = TRUE;
	}

	return m_bOpen;
}

void CFile::close(void)
{
	if ( m_bOpen == TRUE )
	{
		fclose( m_pFile );
		m_bOpen = FALSE;
	}

	m_szName[0] = '\0';
}

FXbool CFile::lineInput(FXString& strLine)
{
	if ( m_bOpen == FALSE )
	{
		strLine = "";
		return FALSE;
	}
	
	if ( m_nPos < m_nBufferLen )
	{
		size_t nBytes = fread(&m_buffer[m_nPos], 1,(m_nBufferLen-m_nPos), m_pFile );

		m_nPos += nBytes;
		m_buffer[m_nPos] = '\000';

		m_nMaxPos = m_nPos;
	}

	while	(
			m_nScanPos < m_nMaxPos			&&
			m_buffer[m_nScanPos]			&&		// NUL will act as '\n'
			( m_buffer[m_nScanPos] != '\n' && m_buffer[m_nScanPos] != '\r' )
			)
	{
		char c = m_buffer[m_nScanPos];
		m_nScanPos++;
	}

	if (m_nScanPos < m_nPos)
	{
		// Terminate line
		if ( m_buffer[m_nScanPos - 1] == 0x0D )
		{
			m_buffer[m_nScanPos - 1] = '\000';
		}
		else
		{
			m_buffer[m_nScanPos] = '\000';
		}

		strLine = m_buffer;

		if ((m_nPos - m_nScanPos) > 1)
		{
			// Move data down in buffer
			memmove(&m_buffer[0], &m_buffer[0] + m_nScanPos + 1, m_nPos - (m_nScanPos + 1));
		}
		m_nPos    -= (m_nScanPos+1);
		m_nScanPos = 0;

		return TRUE;
	}

	return FALSE;
}

FXbool CFile::write(FXString& strLine)
{
	return (fwrite(strLine.text(), 1, strLine.length(), m_pFile) == (unsigned)strLine.size());
}

ITextStream& CFile::operator<<(const FXString s)
{
	fwrite(s.text(), 1, s.length(), m_pFile);
	return *this;
}

ITextStream& CFile::operator<<(const char c)
{
	fwrite(&c, 1, 1, m_pFile);
	return *this;
}

ITextStream& CFile::operator<<(const FXint i)
{
	char sz[5+1] = "";
	sprintf(sz, "%i", i);
	fwrite(sz, 1, strlen(sz), m_pFile);
	return *this;
}

