/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include "CAppPane.h"
#include "CDialogPane.h"

FXDEFMAP(CDialogPane) CDialogPaneMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CDialogPane::onPaint)
};

FXIMPLEMENT(CDialogPane, CAppPane, CDialogPaneMap, ARRAYNUMBER(CDialogPaneMap)) 

CDialogPane::CDialogPane()
{
}

CDialogPane::CDialogPane(FXComposite* p,const FXString& name,FXIcon* ic,FXMenuPane* mn,FXuint opts,FXint x,FXint y,FXint w,FXint h):
  CAppPane(p,name,ic,mn,opts,x,y,w,h)
{
}

CDialogPane::~CDialogPane()
{
}

//
// layout code stolen from FXTopWindow which supports LAYOUT_FIX_xxxxx
//
#define LAYOUT_SIDE_MASK (LAYOUT_SIDE_LEFT|LAYOUT_SIDE_RIGHT|LAYOUT_SIDE_TOP|LAYOUT_SIDE_BOTTOM)
// Layout modes
#define LAYOUT_MASK (LAYOUT_SIDE_MASK|LAYOUT_RIGHT|LAYOUT_CENTER_X|LAYOUT_BOTTOM|LAYOUT_CENTER_Y|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|LAYOUT_FILL_X|LAYOUT_FILL_Y)
const FXuint hspacing = 0;
const FXuint vspacing = 0;

/*
   This is FXTopWindow::layout with

	  if (child==getFirst())
		  continue;

   commented out.
*/
void CDialogPane::layout(){
  FXint left,right,top,bottom;
  FXint mw=0,mh=0;
  FXWindow* child;
  FXint x,y,w,h;
  FXint extra_space,total_space;
  FXuint hints,side;

  // Placement rectangle; right/bottom non-inclusive
  left=0;
  right=width;
  top=0;
  bottom=height;
  
  // Get maximum child size
  if(options&PACK_UNIFORM_WIDTH) mw=maxChildWidth();
  if(options&PACK_UNIFORM_HEIGHT) mh=maxChildHeight();
  
  // Pack them in the cavity
  for(child=getFirst(); child; child=child->getNext()){
//	  if (child==getFirst())
//		  continue;

    if(child->shown()){
      hints=child->getLayoutHints();
      side=hints&LAYOUT_SIDE_MASK;
      x=child->getX();
      y=child->getY();
      if(hints&LAYOUT_FIX_WIDTH) w=child->getWidth(); 
      else if(options&PACK_UNIFORM_WIDTH) w=mw;
      else w=child->getDefaultWidth();
      if(hints&LAYOUT_FIX_HEIGHT) h=child->getHeight(); 
      else if(options&PACK_UNIFORM_HEIGHT) h=mh;
      else h=child->getDefaultHeight();
      if(side==LAYOUT_SIDE_LEFT){           // Left side
        if(!(hints&LAYOUT_FIX_Y)){
          extra_space=0;
          if(hints&LAYOUT_FILL_Y){
            h=bottom-top;
            if(h<0) h=0;
            }
          else if(hints&LAYOUT_CENTER_Y){
            if(h<(bottom-top)) extra_space=(bottom-top-h)/2;
            }
          if(hints&LAYOUT_BOTTOM)
            y=bottom-extra_space-h;
          else 
            y=top+extra_space;
          }
        if(!(hints&LAYOUT_FIX_X)){
          extra_space=0;
          total_space=0;
          if(hints&LAYOUT_FILL_X){
            w=right-left;
            if(w<0) w=0;
            }
          else if(hints&LAYOUT_CENTER_X){
            if(w<(right-left)){
              total_space=right-left-w;
              extra_space=total_space/2;
              }
            }
          x=left+extra_space;
          left+=(w+hspacing+total_space);
          }
        }
      else if(side==LAYOUT_SIDE_RIGHT){     // Right side
        if(!(hints&LAYOUT_FIX_Y)){
          extra_space=0;
          if(hints&LAYOUT_FILL_Y){
            h=bottom-top;
            if(h<0) h=0;
            }
          else if(hints&LAYOUT_CENTER_Y){
            if(h<(bottom-top)) extra_space=(bottom-top-h)/2;
            }
          if(hints&LAYOUT_BOTTOM)
            y=bottom-extra_space-h;
          else 
            y=top+extra_space;
          }
        if(!(hints&LAYOUT_FIX_X)){
          extra_space=0;
          total_space=0;
          if(hints&LAYOUT_FILL_X){
            w=right-left;
            if(w<0) w=0;
            }
          else if(hints&LAYOUT_CENTER_X){
            if(w<(right-left)){
              total_space=right-left-w;
              extra_space=total_space/2;
              }
            }
          x=right-w-extra_space;
          right-=(w+hspacing+total_space);
          }
        }
      else if(side==LAYOUT_SIDE_BOTTOM){    // Bottom side
        if(!(hints&LAYOUT_FIX_X)){
          extra_space=0;
          if(hints&LAYOUT_FILL_X){
            w=right-left;
            if(w<0) w=0;
            }
          else if(hints&LAYOUT_CENTER_X){
            if(w<(right-left)) extra_space=(right-left-w)/2;
            }
          if(hints&LAYOUT_RIGHT)
            x=right-extra_space-w;
          else 
            x=left+extra_space;
          }
        if(!(hints&LAYOUT_FIX_Y)){
          extra_space=0;
          total_space=0;
          if(hints&LAYOUT_FILL_Y){
            h=bottom-top;
            if(h<0) h=0;
            }
          else if(hints&LAYOUT_CENTER_Y){
            if(h<(bottom-top)){
              total_space=bottom-top-h;
              extra_space=total_space/2;
              }
            }
          y=bottom-h-extra_space;
          bottom-=(h+vspacing+total_space);
          }
        }
      else if(side==LAYOUT_SIDE_TOP){       // Top side
        if(!(hints&LAYOUT_FIX_X)){
          extra_space=0;
          if(hints&LAYOUT_FILL_X){
            w=right-left;
            if(w<0) w=0;
            }
          else if(hints&LAYOUT_CENTER_X){
            if(w<(right-left)) extra_space=(right-left-w)/2;
            }
          if(hints&LAYOUT_RIGHT)
            x=right-extra_space-w;
          else 
            x=left+extra_space;
          }
        if(!(hints&LAYOUT_FIX_Y)){
          extra_space=0;
          total_space=0;
          if(hints&LAYOUT_FILL_Y){
            h=bottom-top;
            if(h<0) h=0;
            }
          else if(hints&LAYOUT_CENTER_Y){
            if(h<(bottom-top)){
              total_space=bottom-top-h;
              extra_space=total_space/2;
              }
            }
          y=top+extra_space;
          top+=(h+vspacing+total_space);
          }
        }
      child->position(x,y,w,h);
      }
    }
  
  // No more dirty
  flags&=~FLAG_DIRTY;
}

long CDialogPane::onPaint(FXObject*,FXSelector,void* ptr)
{
	FXEvent *ev=(FXEvent*)ptr;
	FXDCWindow dc(this,ev);
	dc.setForeground(getParent()->getBackColor());
	dc.fillRectangle(ev->rect.x,ev->rect.y,ev->rect.w,ev->rect.h);
	return 1;
}

