/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CFrame_H__
#define __CFrame_H__

class FXAPI CFrame : public FXComposite
{
	FXDECLARE(CFrame)

// Services
public:
	CFrame(FXComposite* p,const FXString& name,FXIcon* ic=NULL,FXMenuPane* mn=NULL,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

protected:
	CFrame();
	CFrame(const CFrame&){}
	virtual ~CFrame();

// Properties
public:
	void setTitle(const FXString& name);
	FXString getTitle() const;

	void setHiliteColor(FXColor clr);
	FXColor getHiliteColor() const { return hiliteColor; }

	void setShadowColor(FXColor clr);
	FXColor getShadowColor() const { return shadowColor; }

	void setBaseColor(FXColor clr);
	FXColor getBaseColor() const { return baseColor; }

	void setBorderColor(FXColor clr);
	FXColor getBorderColor() const { return borderColor; }

	void setFont(FXFont *fnt);
	FXFont* getFont() const { return font; }

	void setMinimizeButton(FXbool bShow);
	FXbool getMinimizeButton(void);

	void setMaximizeButton(FXbool bShow);
	FXbool getMaximizeButton(void);

	void setMenuButton(FXbool bShow);
	FXbool getMenuButton(void);

// Operations
public:
	virtual void create();
	virtual void detach();

protected:
	virtual void layout();

// Events
public:
	long onPaint(FXObject*,FXSelector,void*);

// Attributes
protected:
	FXchar        title[1024];
	FXMenuButton *windowbtn;
	FXButton     *minimizebtn;
	FXButton     *restorebtn;
	FXButton     *maximizebtn;
	FXButton     *deletebtn;
	FXFont       *font;
	FXColor       baseColor;
	FXColor       hiliteColor;
	FXColor       shadowColor;
	FXColor       borderColor;
	FXColor       titleColor;
	FXColor       titleBackColor;
};

#endif // __CFrame_H__