/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include <CConnector.h>

#include <CApp.h>
#include <CGarbage.h>

#include "CTracker.h"
#include "CTrackManager.h"
#include "CAppDialog.h"

#include <CComboBox.h>
#include <CNameComboBox.h>

#include "CAppPane.h"
#include "CDialogPane.h" 
#include <CPropertyList.h>
#include <COutlookBar.h>
#include "CMainFrame.h"

#include <CDispenserButton.h>
#include <CDispenserCheckButton.h>
#include <CDispenserComboBox.h>
#include <CDispenserDialogFrame.h>
#include <CDispenserGroupBox.h>
#include <CDispenserFrame.h>
#include <CDispenserLabel.h>
#include <CDispenserList.h>
#include <CDispenserMatrix.h>
#include <CDispenserRadioButton.h>
#include <CDispenserScrollbar.h>
#include <CDispenserShutter.h>
#include <CDispenserSplitter.h>
#include <CDispenserTab.h>
#include <CDispenserTextField.h>

#include <CDispenserArrowButton.h>
#include <CDispenserDial.h>
#include <CDispenserDirList.h>
#include <CDispenserFileList.h>
#include <CDispenserProgressBar.h>
#include <CDispenserSlider.h>
#include <CDispenserSpinner.h>
#include <CDispenserStatusbar.h>
#include <CDispenserToggleButton.h>
#include <CDispenserToolBar.h>
#include <CDispenserTreeList.h>
#include <CDispenserTreeListBox.h>

/*
 *   The dialog box position is set in a confusing number of places:
 *
 *     CMainFrame::layout()
 *     CMainFrame::createDialog()
 *     CDispenserDialogFrame::createWidget()
 *
 *   All these functions set the x,y,w,h to 8, 8, 160, 200
 */

class CVerticalFrame : public FXVerticalFrame
{
public:
	CVerticalFrame(FXComposite *p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_SPACING,FXint pr=DEFAULT_SPACING,FXint pt=DEFAULT_SPACING,FXint pb=DEFAULT_SPACING,FXint hs=DEFAULT_SPACING,FXint vs=DEFAULT_SPACING)
		: FXVerticalFrame(p,opts,x,y,0,h,pl,pr,pt,pb,hs,vs)
	{
		m_DefaultWidth = w;
	}

	virtual FXint getDefaultWidth()
	{
		return m_DefaultWidth;
	}

private:
	FXint m_DefaultWidth;
};

struct SHUTTER
{
	FXString       strName;
	FXShutterItem* pItem;
};

struct DISPENSER
{
	IDispenser* pDispenser;
};

DISPENSER DispenserMap[] = 
{
	{new CDispenserTextField()},
	{new CDispenserButton()},
	{new CDispenserComboBox()},
	{new CDispenserList()},
	{new CDispenserCheckButton()},
	{new CDispenserRadioButton()},
	{new CDispenserGroupBox()},
	{new CDispenserLabel()},
	{new CDispenserVScrollbar()},
	{new CDispenserHScrollbar()},

	{new CDispenserArrowButton()},
	{new CDispenserDial()},
	{new CDispenserDirList()},
	{new CDispenserFileList()},
	{new CDispenserProgressBar()},
	{new CDispenserSlider()},
	{new CDispenserSpinner()},
	{new CDispenserStatusbar()},
	{new CDispenserToggleButton()},
//	{new CDispenserToolBar()},
	{new CDispenserTreeList()},
//	{new CDispenserTreeListBox()},

	{new CDispenserShutter()},
	{new CDispenserShutterItem()},
	{new CDispenserTabBook()},
	{new CDispenserTabItem()},
	{new CDispenserMatrix()},
	{new CDispenserSplitter()},
	{new CDispenserHorizontalFrame()},
	{new CDispenserVerticalFrame()},

	{0}
};

FXDEFMAP(CMainFrame) CMainFrameMap[]=
{
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_ALIGNLEFT,CMainFrame::onAlignLeft),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_ALIGNRIGHT,CMainFrame::onAlignRight),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_ALIGNTOP,CMainFrame::onAlignTop),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_ALIGNBOTTOM,CMainFrame::onAlignBottom),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_SIZEWIDTH,CMainFrame::onSizeWidth),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_SIZEHEIGHT,CMainFrame::onSizeHeight),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_SIZEBOTH,CMainFrame::onSizeBoth),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_CENTREHORIZONTAL,CMainFrame::onCentreHorizontal),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_CENTREVERTICAL,CMainFrame::onCentreVertical),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_SNAPTOGRID,CMainFrame::onSnapToGrid),
	FXMAPFUNC(SEL_UPDATE ,CMainFrame::IDM_SNAPTOGRID,CMainFrame::onUpdateSnapToGrid),
	FXMAPFUNC(SEL_COMMAND,CMainFrame::IDM_DRAGOUTLINE,CMainFrame::onDragOutline),
	FXMAPFUNC(SEL_UPDATE ,CMainFrame::IDM_DRAGOUTLINE,CMainFrame::onUpdateDragOutline),
	FXMAPFUNCS(SEL_UPDATE,CMainFrame::IDM_ALIGNLEFT,CMainFrame::IDM_CENTREVERTICAL,CMainFrame::onUpdateAlignLeft)
};

FXIMPLEMENT(CMainFrame, FXMainWindow, CMainFrameMap, ARRAYNUMBER(CMainFrameMap)) 

long CMainFrame::onAlignLeft(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->alignLeft();
	return 1;
}

long CMainFrame::onUpdateAlignLeft(FXObject* o,FXSelector,void* ptr)
{
 	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
 	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onAlignRight(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->alignRight();
	return 1;
}

long CMainFrame::onUpdateAlignRight(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onAlignTop(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->alignTop();
	return 1;
}

long CMainFrame::onUpdateAlignTop(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onAlignBottom(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->alignBottom();
	return 1;
}

long CMainFrame::onUpdateAlignBottom(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onSizeWidth(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->sizeWidth();
	return 1;
}

long CMainFrame::onUpdateSizeWidth(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onSizeHeight(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->sizeHeight();
	return 1;
}

long CMainFrame::onUpdateSizeHeight(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onSizeBoth(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->sizeBoth();
	return 1;
}

long CMainFrame::onUpdateSizeBoth(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onCentreHorizontal(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->centreHorizontal();
	return 1;
}

long CMainFrame::onUpdateCentreHorizontal(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onCentreVertical(FXObject* o,FXSelector,void* ptr)
{
	application->getTrackManager()->centreVertical();
	return 1;
}

long CMainFrame::onUpdateCentreVertical(FXObject* o,FXSelector,void* ptr)
{
	FXuint msg = application->getTrackManager()->isMultipleSelected() ? FXWindow::ID_ENABLE : FXWindow::ID_DISABLE;
	o->handle(this,MKUINT(msg,SEL_COMMAND),ptr);
	return 1;
}

long CMainFrame::onSnapToGrid(FXObject* o,FXSelector s,void*)
{
	if ( application->getTrackManager()->SnapToGrid() )
	{
		application->getTrackManager()->SnapToGrid(false);
		o->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
	}
	else
	{
		application->getTrackManager()->SnapToGrid(true);
		o->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
	}
	application->getTrackManager()->clearGrabbers();

	return 1;
}

long CMainFrame::onUpdateSnapToGrid(FXObject* o,FXSelector s,void*)
{
	if ( application->getTrackManager()->SnapToGrid() )
	{
		o->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
	}
	else
	{
		o->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
	}
	return 1;
}

long CMainFrame::onDragOutline(FXObject* o,FXSelector s,void*)
{
	if ( application->getTrackManager()->DragOutline() )
	{
		application->getTrackManager()->DragOutline(false);
		o->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
	}
	else
	{
		application->getTrackManager()->DragOutline(true);
		o->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
	}

	return 1;
}

long CMainFrame::onUpdateDragOutline(FXObject* o,FXSelector s,void*)
{
	if ( application->getTrackManager()->DragOutline() )
	{
		o->handle(this,MKUINT(ID_CHECK,SEL_COMMAND),NULL);
	}
	else
	{
		o->handle(this,MKUINT(ID_UNCHECK,SEL_COMMAND),NULL);
	}
	return 1;
}

// Initialise frame to fit on an 800x600 display
CMainFrame::CMainFrame(FXApp* a):FXMainWindow(a,"FOX Dialog Editor",NULL,NULL,DECOR_ALL,20,20,750,510)
{
	m_Garbage = new FXTooltip(getApp(),0);

	createIcons();
	createMenu();
	createToolbar();

	m_pSplitter1  = new FXSplitter(this,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|/*SPLITTER_REVERSED|*/SPLITTER_TRACKING);
 	  CVerticalFrame* frame1 = new CVerticalFrame(m_pSplitter1,LAYOUT_FILL_X|LAYOUT_FIX_Y, 0, 0, 150);
 	  CVerticalFrame* frame2 = new CVerticalFrame(m_pSplitter1,/*FRAME_THICK|FRAME_RAISED|*/LAYOUT_FILL_X|LAYOUT_FIX_Y);
	CShutter* shutterFrame = new CShutter(frame1,NULL,0,FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y, 0, 0, 120);

	m_pSplitter2  = new FXSplitter(frame2,LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_REVERSED|SPLITTER_TRACKING);
 	   m_pFrame = new CVerticalFrame(m_pSplitter2,/*FRAME_THICK|FRAME_RAISED|*/LAYOUT_FILL_X|LAYOUT_FIX_Y);

	CVerticalFrame* listframe = new CVerticalFrame(m_pSplitter2,FRAME_THICK|FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FIX_Y, 0, 0, 185);

//////////////////// properties //////////////////

	CConnector connector;
	m_pNames = new CNameComboBox(connector, listframe, 50, 4, 0, 0,
					COMBOBOX_STATIC|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT, 0,0,0,16);

	m_pProperties  = new CPropertyList(listframe,LIST_EXTENDEDSELECT|LAYOUT_FILL_X|LAYOUT_FILL_Y);

	m_pDescription = new FXLabel(listframe, "", 0, 
						FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FIX_HEIGHT|JUSTIFY_LEFT|JUSTIFY_TOP, 0,0,0,50);

	FXFont* pFont = 0;
	m_Garbage = pFont = new FXFont(application,"helvetica",8,FONTWEIGHT_NORMAL, FONTSLANT_REGULAR, FONTENCODING_DEFAULT, FONTSETWIDTH_CONDENSED);

	m_pDescription->setFont(pFont);

//////////////////// properties //////////////////

//////////////////// widgets    //////////////////

	SHUTTER Shutter;
	FXArray<SHUTTER> Shutters;

	Shutter.pItem   = new CShutterItem(shutterFrame,"Standard",NULL,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,0,0,1,1,1,1);
	Shutter.strName = "Standard";

	Shutters.append(Shutter);

	if ( application )
	{
		CConnector connector = createDialog();

		for (DISPENSER* p = DispenserMap; p->pDispenser; p++)
		{
			FXbool bFound = FALSE;
			for ( FXint i = 0; i < Shutters.no(); i++ )
			{
				if ( strcmp( p->pDispenser->getWidgetCategory(), Shutters[i].strName.text() ) == 0 )
				{
					new CButton(Shutters[i].pItem->getContent(), connector, p->pDispenser, application->getTrackManager(), CTrackManager::ID_ADDWIDGET);
					application->addDispenser(p->pDispenser);
					bFound = TRUE;
					break;
				}
			}

			if ( !bFound)
			{
				Shutter.pItem   = new CShutterItem(shutterFrame,p->pDispenser->getWidgetCategory(),NULL,LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,0,0,0,0,0,0,1,1,1,1);
				Shutter.strName = p->pDispenser->getWidgetCategory();
				Shutters.append(Shutter);
				new CButton(Shutter.pItem->getContent(), connector, p->pDispenser, application->getTrackManager(), CTrackManager::ID_ADDWIDGET);
				application->addDispenser(p->pDispenser);
			}
		}
  	}

//////////////////// widgets    //////////////////

	m_bInitialSize = true;
}

CConnector CMainFrame::createDialog(FXbool bShow)
{
	CConnector connector;

	if ( application )
	{
		// Create the application pane -- the grey parent of the 'dialog box'
		CAppPane* pAppPane = new CAppPane(m_pFrame, "A Test", NULL, NULL, 
						FRAME_NONE|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_Y,0,0,0,0);

		pAppPane->setBackColor(FXRGB(128, 128, 128));
		
		// Next, create a CConnector and provide all the necessary details -- pointer to all the important
		// windows such as pDescription and pProperties. Note that pWidget is set to m_pAppPane as CApp needs
		// to know that this is a valid widget.
		//
		// Note also that a CDispenserDialogFrame has been added. This CDispenserFrame will be shared between the pAppPane,
		// the pDialog and the pDialogPane as a user may click on any of these but they'll all bring up the
		// properties for the dialog.

		connector.pTrackManager = application->getTrackManager();
		connector.pDispenser    = new CDispenserDialogFrame();
		connector.pProperties   = m_pProperties;
		connector.pWidget       = pAppPane;
		connector.pDescription  = m_pDescription;
		connector.pNames        = m_pNames;

		// Ok. Add this to the CApp's list of valid widgets. This widget isn't 'visible' to the user
		// so it's important that we call application->addWidget which just adds it to a list rather 
		// than connector.addWidget which makes it very visible to the user!

		application->addWidget(connector);

		// Now add the 'dialog' -- it's really a CFrame! Note that we've used connector.addWidget which
		// will use the CDispenserDialogFrame to create a new Frame, populate the property list and put the Frame's
		// name and class name in the Name Combo Box. 

		m_pDialog = (FXComposite*)connector.addWidget(pAppPane, FALSE);

		// We need to create the dialog pane -- the client area of the dialog. connector.pWidget
		// was set in the previous call to connector.addWidget which is the dialog pane's parent.

		CDialogPane* pDialogPane = new CDialogPane(m_pDialog, "A Test", NULL, NULL, 
						DECOR_TITLE|DECOR_BORDER|LAYOUT_FILL_X|LAYOUT_TOP|LAYOUT_LEFT|LAYOUT_FILL_Y);

		// We must add this to CApp's list of valid widgets. 

		connector.pWidget = pDialogPane;
		application->addWidget(connector);

		// And finally, inform CApp of these three special widgets.

		application->getTrackManager()->setFrames(pAppPane, m_pDialog, pDialogPane);

		connector.pWidget = 0;
	}

	return connector;
}

CMainFrame::~CMainFrame()
{
}

void CMainFrame::create()
{
	FXMainWindow::create();

	show();
}

void CMainFrame::layout()
{
	FXTopWindow::layout();

	if ( m_bInitialSize )
	{
		m_bInitialSize = false;

		m_pDialog->position(8, 8, 280, 200);
		m_pDialog->enable();
		m_pDialog->show();
	}
}


