/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserArrowButton.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyJustify.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyArrowStyle : public CProperty
{
public:
	CPropertyArrowStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	virtual void set(FXWindow* pWindow, FXString strValue);
	virtual FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector) {}

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

///////////////////////////////////////////////////////////////////////////////
//

// From FXArrowButton.h
template <class T>
OPTION CPropertyArrowStyle<T>::Style[] = 
{
	{"ARROW_UP"      , ARROW_UP, ""},
	{"ARROW_DOWN"    , ARROW_DOWN, ""},
	{"ARROW_LEFT"    , ARROW_LEFT , ""},
	{"ARROW_RIGHT"   , ARROW_RIGHT, ""},
	{"ARROW_REPEAT"  , ARROW_REPEAT , ""},
	{"ARROW_AUTOGRAY", ARROW_AUTOGRAY  , ""},
	{"ARROW_AUTOHIDE", ARROW_AUTOHIDE  , ""},
	{"ARROW_TOOLBAR" , ARROW_TOOLBAR, ""},
	{0, 0, 0}
};

template <class T>
const char* CPropertyArrowStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyArrowStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyArrowStyle<T>::description(void)
{
	return "The arrow style of this object";
}

template <class T>
const char* CPropertyArrowStyle<T>::tooltip(void)
{
	return "getArrowStyle()\nsetArrowStyle()";
}

template <class T>
void CPropertyArrowStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setArrowStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyArrowStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getArrowStyle() );
}

template <class T>
void CPropertyArrowStyle<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getArrowStyle();
	FXint nNewValue = getIntFromString(strNewValue);

	// Has user turned on ARROW_UP?
	if ( !(nOldValue & ARROW_UP) && (nNewValue & ARROW_UP))
	{
		// Yes. So turn off ARROW_DOWN|ARROW_LEFT|ARROW_RIGHT
		nNewValue &= ~(ARROW_DOWN|ARROW_LEFT|ARROW_RIGHT);
	}

	// Has user turned on ARROW_DOWN?
	if ( !(nOldValue & ARROW_DOWN) && (nNewValue & ARROW_DOWN) )
	{
		// Yes. So turn off ARROW_UP|ARROW_LEFT|ARROW_RIGHT
		nNewValue &= ~(ARROW_UP|ARROW_LEFT|ARROW_RIGHT);
	}

	// Has user turned on ARROW_LEFT?
	if ( !(nOldValue & ARROW_LEFT) && (nNewValue & ARROW_LEFT))
	{
		// Yes. So turn off ARROW_UP|ARROW_DOWN|ARROW_RIGHT
		nNewValue &= ~(ARROW_UP|ARROW_DOWN|ARROW_RIGHT);
	}

	// Has user turned on ARROW_RIGHT?
	if ( !(nOldValue & ARROW_RIGHT) && (nNewValue & ARROW_RIGHT) )
	{
		// Yes. So turn off ARROW_UP|ARROW_DOWN|ARROW_LEFT
		nNewValue &= ~(ARROW_UP|ARROW_DOWN|ARROW_LEFT);
	}

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
OPTION* CPropertyArrowStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyArrowStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getArrowStyle() == 0 )
		return;
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertyArrowColor : public CProperty
{
public:
	CPropertyArrowColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyArrowColor<T, C>::name(void)
{
	return "ArrowColor";
}

template <class T, FXColor C>
ePropertyType CPropertyArrowColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyArrowColor<T, C>::description(void)
{
	return "The arrow color of this object";
}

template <class T, FXColor C>
const char* CPropertyArrowColor<T, C>::tooltip(void)
{
	return "getArrowColor()\nsetArrowColor()";
}

template <class T, FXColor C>
void CPropertyArrowColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setArrowColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyArrowColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getArrowColor() );
}

template <class T, FXColor C>
FXString CPropertyArrowColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getArrowColor() != C )
	{
		FXString str = getFXRGBString(p->getArrowColor());
		return "\t" + widget + "->setArrowColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Arrow[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x04,0x04,0x04,0x0c,0x0c,
  0x0c,0x11,0x11,0x11,0x16,0x16,0x16,0x1c,0x1c,0x1c,0x22,0x22,0x22,0x55,0x55,0x55,
  0x4d,0x4d,0x4d,0x42,0x42,0x42,0x39,0x39,0x39,0xff,0x7c,0x80,0xff,0x50,0x50,0xd6,
  0x00,0x93,0xcc,0xec,0xff,0xef,0xd6,0xc6,0xe7,0xe7,0xd6,0xad,0xa9,0x90,0x33,0x00,
  0x00,0x66,0x00,0x00,0x99,0x00,0x00,0x00,0x33,0x00,0x33,0x33,0x00,0x66,0x33,0x00,
  0x99,0x33,0x00,0xcc,0x33,0x00,0xff,0x33,0x00,0x00,0x66,0x00,0x33,0x66,0x00,0x66,
  0x66,0x00,0x99,0x66,0x00,0xcc,0x66,0x00,0xff,0x66,0x00,0x00,0x99,0x00,0x33,0x99,
  0x00,0x66,0x99,0x00,0x99,0x99,0x00,0xcc,0x99,0x00,0xff,0x99,0x00,0x00,0xcc,0x00,
  0x33,0xcc,0x00,0x66,0xcc,0x00,0x99,0xcc,0x00,0xcc,0xcc,0x00,0xff,0xcc,0x00,0x66,
  0xff,0x00,0x99,0xff,0x00,0xcc,0xff,0x00,0x00,0x00,0x33,0x33,0x00,0x33,0x66,0x00,
  0x33,0x99,0x00,0x33,0xcc,0x00,0x33,0xff,0x00,0x33,0x00,0x33,0x33,0x33,0x33,0x33,
  0x66,0x33,0x33,0x99,0x33,0x33,0xcc,0x33,0x33,0xff,0x33,0x33,0x00,0x66,0x33,0x33,
  0x66,0x33,0x66,0x66,0x33,0x99,0x66,0x33,0xcc,0x66,0x33,0xff,0x66,0x33,0x00,0x99,
  0x33,0x33,0x99,0x33,0x66,0x99,0x33,0x99,0x99,0x33,0xcc,0x99,0x33,0xff,0x99,0x33,
  0x00,0xcc,0x33,0x33,0xcc,0x33,0x66,0xcc,0x33,0x99,0xcc,0x33,0xcc,0xcc,0x33,0xff,
  0xcc,0x33,0x33,0xff,0x33,0x66,0xff,0x33,0x99,0xff,0x33,0xcc,0xff,0x33,0xff,0xff,
  0x33,0x00,0x00,0x66,0x33,0x00,0x66,0x66,0x00,0x66,0x99,0x00,0x66,0xcc,0x00,0x66,
  0xff,0x00,0x66,0x00,0x33,0x66,0x33,0x33,0x66,0x66,0x33,0x66,0x99,0x33,0x66,0xcc,
  0x33,0x66,0xff,0x33,0x66,0x00,0x66,0x66,0x33,0x66,0x66,0x66,0x66,0x66,0x99,0x66,
  0x66,0xcc,0x66,0x66,0x00,0x99,0x66,0x33,0x99,0x66,0x66,0x99,0x66,0x99,0x99,0x66,
  0xcc,0x99,0x66,0xff,0x99,0x66,0x00,0xcc,0x66,0x33,0xcc,0x66,0x99,0xcc,0x66,0xcc,
  0xcc,0x66,0xff,0xcc,0x66,0x00,0xff,0x66,0x33,0xff,0x66,0x99,0xff,0x66,0xcc,0xff,
  0x66,0xff,0x00,0xcc,0xcc,0x00,0xff,0x00,0x99,0x99,0x99,0x33,0x99,0x99,0x00,0x99,
  0xcc,0x00,0x99,0x00,0x00,0x99,0x33,0x33,0x99,0x66,0x00,0x99,0xcc,0x33,0x99,0xff,
  0x00,0x99,0x00,0x66,0x99,0x33,0x66,0x99,0x66,0x33,0x99,0x99,0x66,0x99,0xcc,0x66,
  0x99,0xff,0x33,0x99,0x33,0x99,0x99,0x66,0x99,0x99,0x99,0x99,0x99,0xcc,0x99,0x99,
  0xff,0x99,0x99,0x00,0xcc,0x99,0x33,0xcc,0x99,0x66,0xcc,0x66,0x99,0xcc,0x99,0xcc,
  0xcc,0x99,0xff,0xcc,0x99,0x00,0xff,0x99,0x33,0xff,0x99,0x66,0xcc,0x99,0x99,0xff,
  0x99,0xcc,0xff,0x99,0xff,0xff,0x99,0x00,0x00,0xcc,0x33,0x00,0x99,0x66,0x00,0xcc,
  0x99,0x00,0xcc,0xcc,0x00,0xcc,0x00,0x33,0x99,0x33,0x33,0xcc,0x66,0x33,0xcc,0x99,
  0x33,0xcc,0xcc,0x33,0xcc,0xff,0x33,0xcc,0x00,0x66,0xcc,0x33,0x66,0xcc,0x66,0x66,
  0x99,0x99,0x66,0xcc,0xcc,0x66,0xcc,0xff,0x66,0x99,0x00,0x99,0xcc,0x33,0x99,0xcc,
  0x66,0x99,0xcc,0x99,0x99,0xcc,0xcc,0x99,0xcc,0xff,0x99,0xcc,0x00,0xcc,0xcc,0x33,
  0xcc,0xcc,0x66,0xcc,0xcc,0x99,0xcc,0xcc,0xcc,0xcc,0xcc,0xff,0xcc,0xcc,0x00,0xff,
  0xcc,0x33,0xff,0xcc,0x66,0xff,0x99,0x99,0xff,0xcc,0xcc,0xff,0xcc,0xff,0xff,0xcc,
  0x33,0x00,0xcc,0x66,0x00,0xff,0x99,0x00,0xff,0x00,0x33,0xcc,0x33,0x33,0xff,0x66,
  0x33,0xff,0x99,0x33,0xff,0xcc,0x33,0xff,0xff,0x33,0xff,0x00,0x66,0xff,0x33,0x66,
  0xff,0x66,0x66,0xcc,0x99,0x66,0xff,0xcc,0x66,0xff,0xff,0x66,0xcc,0x00,0x99,0xff,
  0x33,0x99,0xff,0x66,0x99,0xff,0x99,0x99,0xff,0xcc,0x99,0xff,0xff,0x99,0xff,0x00,
  0xcc,0xff,0x33,0xcc,0xff,0x66,0xcc,0xff,0x08,0x08,0x08,0x99,0xcc,0xff,0xcc,0xcc,
  0xff,0xff,0xcc,0xff,0x33,0xff,0xff,0x66,0xff,0xcc,0x29,0x29,0x29,0x99,0xff,0xff,
  0xcc,0xff,0xff,0xff,0x66,0x66,0x66,0xff,0x66,0xff,0xff,0x66,0x66,0x66,0xff,0xff,
  0x66,0xff,0x66,0xff,0xff,0xa5,0x00,0x21,0x5f,0x5f,0x5f,0x77,0x77,0x77,0x86,0x86,
  0x86,0x96,0x96,0x96,0xcb,0xcb,0xcb,0xb2,0xb2,0xb2,0xd7,0xd7,0xd7,0xdd,0xdd,0xdd,
  0xe3,0xe3,0xe3,0xea,0xea,0xea,0xf1,0xf1,0xf1,0xf8,0xf8,0xf8,0xcc,0x00,0x00,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xd7,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0xfe,0xdb,0xc8,0xb1,0xa3,0xc7,0x8f,0x00,0x14,0x7e,
  0x1c,0x49,0x32,0x64,0xc2,0x7f,0x07,0x52,0xaa,0x5c,0xc9,0x72,0xe5,0x3a,0x93,0x08,
  0x51,0xb6,0x9c,0xc9,0xf2,0xa5,0x48,0x9a,0x38,0x53,0xda,0x3c,0x99,0x13,0xe7,0xce,
  0x98,0x3d,0x69,0xfe,0x3c,0x28,0x73,0x25,0x80,0xa0,0x43,0x0d,0x16,0x0d,0xea,0x12,
  0x00,0xbe,0xa7,0x50,0xa3,0x2e,0x4d,0x09,0xe0,0x68,0xce,0xa4,0x05,0xa7,0x32,0x3d,
  0xf0,0x32,0xaa,0x57,0x7c,0x5a,0xab,0x5a,0x15,0x0a,0x93,0x28,0xce,0xb1,0x2d,0xbb,
  0x7e,0x85,0x1a,0x56,0xac,0xcf,0xb2,0x4a,0x69,0xba,0x9d,0xa9,0x76,0x2d,0xd8,0x96,
  0x62,0xab,0x92,0xbd,0x89,0x37,0x2f,0x5d,0xa7,0x76,0xef,0xb2,0xcc,0xab,0x37,0x2d,
  0xdc,0xac,0x7d,0x09,0x1b,0x0e,0x2c,0xd8,0x28,0x61,0xb4,0x3a,0x0f,0x13,0x9c,0xfa,
  0x78,0xae,0xca,0xba,0x6b,0xdb,0x3e,0xae,0x29,0x79,0xe0,0xd2,0xca,0x7e,0x2f,0x03,
  0xb6,0xab,0x39,0x74,0x53,0xbe,0x07,0x40,0x2b,0x8e,0xcc,0xb8,0x74,0x61,0xce,0x7c,
  0x55,0x6f,0x3e,0x80,0x6f,0x74,0xe6,0xc4,0xaf,0x61,0xf3,0xdc,0xba,0xb2,0x76,0x6b,
  0xdc,0x6f,0x51,0xf3,0xa6,0x6d,0xfb,0xab,0xe6,0xab,0x9d,0x05,0x6a,0x65,0xea,0x3b,
  0x70,0xdb,0x9e,0x58,0x27,0x0f,0x17,0xfd,0x7b,0x30,0xd2,0xe4,0xeb,0x96,0x5f,0xaf,
  0x3e,0x3d,0xba,0xe7,0xe9,0xac,0x9d,0xfe,0x83,0xe7,0x8a,0xfd,0x1f,0xc5,0xe2,0x5e,
  0xcd,0x4f,0xc4,0x2e,0xbb,0x3d,0xfa,0xa8,0xee,0x65,0x2b,0x64,0x4c,0xbf,0xbe,0xfd,
  0xaf,0x19,0xf3,0xeb,0xbf,0x4f,0x5f,0x3f,0x43,0xfe,0x8c,0xf9,0xb7,0x10,0x80,0x81,
  0x09,0x38,0x1f,0x81,0x6b,0x19,0x98,0x10,0x82,0x09,0x2a,0x78,0x10,0x83,0xf8,0x39,
  0x68,0x10,0x84,0x5e,0x49,0x38,0x21,0x85,0x50,0x59,0xe8,0x19,0x49,0x1c,0x72,0x54,
  0x5c,0x56,0x1d,0x76,0x58,0xde,0x78,0x98,0xe1,0x33,0xd0,0x53,0xda,0x41,0x37,0x22,
  0x78,0x25,0x9e,0xd8,0xd8,0x70,0xde,0x29,0x17,0x14,0x64,0xe4,0x55,0x28,0x10,0x8a,
  0x24,0xae,0x78,0x16,0x8d,0x2d,0xde,0xf8,0x22,0x6f,0x31,0x66,0x97,0x93,0x65,0xd4,
  0x45,0xe5,0x62,0x8a,0xc8,0x09,0x37,0x18,0x91,0xe1,0x65,0xe8,0x23,0x92,0xc1,0xed,
  0x86,0x1b,0x8f,0x1f,0x3e,0x49,0x55,0x7c,0x56,0x05,0xb9,0xdc,0x6c,0x4d,0xd9,0xb8,
  0x0e,0x8e,0x57,0xb6,0x27,0x9a,0x92,0xa9,0x71,0x59,0xa4,0x93,0x5f,0xbe,0x28,0xe6,
  0x98,0x52,0x86,0xb9,0x5a,0x97,0x46,0x5a,0xe9,0x58,0x65,0xba,0x01,0xb5,0xe4,0x9b,
  0x67,0x3e,0x75,0xe4,0x94,0xb9,0x45,0x46,0x66,0x99,0xa6,0xe5,0x69,0xa2,0x9c,0x77,
  0xf6,0xe9,0x67,0x9b,0x73,0x1a,0xda,0x63,0x9a,0xda,0x31,0xc9,0xa6,0x9d,0x33,0x05,
  0xda,0xa4,0x9e,0x84,0xe2,0x15,0x25,0xa4,0x91,0x2a,0x5a,0x25,0xa3,0x39,0xfe,0xe9,
  0xd8,0x62,0x71,0x72,0xca,0xa2,0x8e,0x67,0x81,0x8a,0x26,0x98,0xdd,0x91,0x0a,0xe4,
  0xa6,0xa8,0xc2,0xa8,0xea,0x56,0x1c,0x5a,0x76,0x7a,0xd2,0x79,0x22,0xd1,0x9a,0x10,
  0x96,0xa0,0x29,0x84,0x6b,0x65,0x1a,0xf6,0xea,0xeb,0xaf,0xc0,0x06,0x2b,0xec,0xb0,
  0x11,0x05,0x04,0x00,0x3b
  };

CDispenserArrowButton::CDispenserArrowButton()
{
	m_Properties.append(new CPropertyLayout<FXArrowButton>(this));
	m_Properties.append(new CPropertyFrame<FXArrowButton>(this));
	m_Properties.append(new CPropertyJustify<FXArrowButton>(this));
	m_Properties.append(new CPropertyArrowStyle<FXArrowButton>(this));
	m_Properties.append(new CPropertyBackColor<FXArrowButton, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyArrowColor<FXArrowButton, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyPadTop<FXArrowButton>(this));
	m_Properties.append(new CPropertyPadLeft<FXArrowButton>(this));
	m_Properties.append(new CPropertyPadBottom<FXArrowButton>(this));
	m_Properties.append(new CPropertyPadRight<FXArrowButton>(this));
	m_Properties.append(new CPropertyLeft<FXArrowButton>(this));
	m_Properties.append(new CPropertyTop<FXArrowButton>(this));
	m_Properties.append(new CPropertyWidth<FXArrowButton>(this));
	m_Properties.append(new CPropertyHeight<FXArrowButton>(this));
}

const char* CDispenserArrowButton::getWidgetName(void)
{
	return "FXArrowButton";
}

const void* CDispenserArrowButton::getWidgetIcon(void)
{
	return Arrow;
}

const char* CDispenserArrowButton::getWidgetCategory(void)
{
	return "Extended";
}

const int   CDispenserArrowButton::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserArrowButton::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserArrowButton::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserArrowButton::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserArrowButton::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserArrowButton::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserArrowButton::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserArrowButton::getClassInfo(void)
{
	return &FXArrowButton::metaClass;
}

FXbool CDispenserArrowButton::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserArrowButton::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserArrowButton::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserArrowButton::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXArrowButton::metaClass);
}

IDispenser* CDispenserArrowButton::clone(void)
{
	return new CDispenserArrowButton;
}

FXWindow* CDispenserArrowButton::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXArrowButton(p, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserArrowButton::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXArrowButton(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
