/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserButton.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyText.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyJustify.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyButtonStyle : public CProperty
{
public:
	CPropertyButtonStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXButton.h
template <class T>
OPTION CPropertyButtonStyle<T>::Style[] = 
{
	{"BUTTON_AUTOGRAY", BUTTON_AUTOGRAY, "Automatically gray out when not updated"},
	{"BUTTON_AUTOHIDE", BUTTON_AUTOHIDE, "Automatically hide button when not updated"},
	{"BUTTON_TOOLBAR" , BUTTON_TOOLBAR , "Toolbar style button [flat look"},
	{"BUTTON_DEFAULT" , BUTTON_DEFAULT , "Default button"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyButtonStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyButtonStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyButtonStyle<T>::description(void)
{
	return "The button style for this object";
}

template <class T>
const char* CPropertyButtonStyle<T>::tooltip(void)
{
	return "getButtonStyle()\nsetButtonStyle()";
}

template <class T>
void CPropertyButtonStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setButtonStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyButtonStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getButtonStyle() );
}

template <class T>
OPTION* CPropertyButtonStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyButtonStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getButtonStyle() == 0 )
		return;
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Button[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x04,0x04,0x04,0x0c,0x0c,
  0x0c,0x11,0x11,0x11,0x16,0x16,0x16,0x1c,0x1c,0x1c,0x22,0x22,0x22,0x55,0x55,0x55,
  0x4d,0x4d,0x4d,0x42,0x42,0x42,0x39,0x39,0x39,0xff,0x7c,0x80,0xff,0x50,0x50,0xd6,
  0x00,0x93,0xcc,0xec,0xff,0xef,0xd6,0xc6,0xe7,0xe7,0xd6,0xad,0xa9,0x90,0x33,0x00,
  0x00,0x66,0x00,0x00,0x99,0x00,0x00,0x00,0x33,0x00,0x33,0x33,0x00,0x66,0x33,0x00,
  0x99,0x33,0x00,0xcc,0x33,0x00,0xff,0x33,0x00,0x00,0x66,0x00,0x33,0x66,0x00,0x66,
  0x66,0x00,0x99,0x66,0x00,0xcc,0x66,0x00,0xff,0x66,0x00,0x00,0x99,0x00,0x33,0x99,
  0x00,0x66,0x99,0x00,0x99,0x99,0x00,0xcc,0x99,0x00,0xff,0x99,0x00,0x00,0xcc,0x00,
  0x33,0xcc,0x00,0x66,0xcc,0x00,0x99,0xcc,0x00,0xcc,0xcc,0x00,0xff,0xcc,0x00,0x66,
  0xff,0x00,0x99,0xff,0x00,0xcc,0xff,0x00,0x00,0x00,0x33,0x33,0x00,0x33,0x66,0x00,
  0x33,0x99,0x00,0x33,0xcc,0x00,0x33,0xff,0x00,0x33,0x00,0x33,0x33,0x33,0x33,0x33,
  0x66,0x33,0x33,0x99,0x33,0x33,0xcc,0x33,0x33,0xff,0x33,0x33,0x00,0x66,0x33,0x33,
  0x66,0x33,0x66,0x66,0x33,0x99,0x66,0x33,0xcc,0x66,0x33,0xff,0x66,0x33,0x00,0x99,
  0x33,0x33,0x99,0x33,0x66,0x99,0x33,0x99,0x99,0x33,0xcc,0x99,0x33,0xff,0x99,0x33,
  0x00,0xcc,0x33,0x33,0xcc,0x33,0x66,0xcc,0x33,0x99,0xcc,0x33,0xcc,0xcc,0x33,0xff,
  0xcc,0x33,0x33,0xff,0x33,0x66,0xff,0x33,0x99,0xff,0x33,0xcc,0xff,0x33,0xff,0xff,
  0x33,0x00,0x00,0x66,0x33,0x00,0x66,0x66,0x00,0x66,0x99,0x00,0x66,0xcc,0x00,0x66,
  0xff,0x00,0x66,0x00,0x33,0x66,0x33,0x33,0x66,0x66,0x33,0x66,0x99,0x33,0x66,0xcc,
  0x33,0x66,0xff,0x33,0x66,0x00,0x66,0x66,0x33,0x66,0x66,0x66,0x66,0x66,0x99,0x66,
  0x66,0xcc,0x66,0x66,0x00,0x99,0x66,0x33,0x99,0x66,0x66,0x99,0x66,0x99,0x99,0x66,
  0xcc,0x99,0x66,0xff,0x99,0x66,0x00,0xcc,0x66,0x33,0xcc,0x66,0x99,0xcc,0x66,0xcc,
  0xcc,0x66,0xff,0xcc,0x66,0x00,0xff,0x66,0x33,0xff,0x66,0x99,0xff,0x66,0xcc,0xff,
  0x66,0xff,0x00,0xcc,0xcc,0x00,0xff,0x00,0x99,0x99,0x99,0x33,0x99,0x99,0x00,0x99,
  0xcc,0x00,0x99,0x00,0x00,0x99,0x33,0x33,0x99,0x66,0x00,0x99,0xcc,0x33,0x99,0xff,
  0x00,0x99,0x00,0x66,0x99,0x33,0x66,0x99,0x66,0x33,0x99,0x99,0x66,0x99,0xcc,0x66,
  0x99,0xff,0x33,0x99,0x33,0x99,0x99,0x66,0x99,0x99,0x99,0x99,0x99,0xcc,0x99,0x99,
  0xff,0x99,0x99,0x00,0xcc,0x99,0x33,0xcc,0x99,0x66,0xcc,0x66,0x99,0xcc,0x99,0xcc,
  0xcc,0x99,0xff,0xcc,0x99,0x00,0xff,0x99,0x33,0xff,0x99,0x66,0xcc,0x99,0x99,0xff,
  0x99,0xcc,0xff,0x99,0xff,0xff,0x99,0x00,0x00,0xcc,0x33,0x00,0x99,0x66,0x00,0xcc,
  0x99,0x00,0xcc,0xcc,0x00,0xcc,0x00,0x33,0x99,0x33,0x33,0xcc,0x66,0x33,0xcc,0x99,
  0x33,0xcc,0xcc,0x33,0xcc,0xff,0x33,0xcc,0x00,0x66,0xcc,0x33,0x66,0xcc,0x66,0x66,
  0x99,0x99,0x66,0xcc,0xcc,0x66,0xcc,0xff,0x66,0x99,0x00,0x99,0xcc,0x33,0x99,0xcc,
  0x66,0x99,0xcc,0x99,0x99,0xcc,0xcc,0x99,0xcc,0xff,0x99,0xcc,0x00,0xcc,0xcc,0x33,
  0xcc,0xcc,0x66,0xcc,0xcc,0x99,0xcc,0xcc,0xcc,0xcc,0xcc,0xff,0xcc,0xcc,0x00,0xff,
  0xcc,0x33,0xff,0xcc,0x66,0xff,0x99,0x99,0xff,0xcc,0xcc,0xff,0xcc,0xff,0xff,0xcc,
  0x33,0x00,0xcc,0x66,0x00,0xff,0x99,0x00,0xff,0x00,0x33,0xcc,0x33,0x33,0xff,0x66,
  0x33,0xff,0x99,0x33,0xff,0xcc,0x33,0xff,0xff,0x33,0xff,0x00,0x66,0xff,0x33,0x66,
  0xff,0x66,0x66,0xcc,0x99,0x66,0xff,0xcc,0x66,0xff,0xff,0x66,0xcc,0x00,0x99,0xff,
  0x33,0x99,0xff,0x66,0x99,0xff,0x99,0x99,0xff,0xcc,0x99,0xff,0xff,0x99,0xff,0x00,
  0xcc,0xff,0x33,0xcc,0xff,0x66,0xcc,0xff,0x08,0x08,0x08,0x99,0xcc,0xff,0xcc,0xcc,
  0xff,0xff,0xcc,0xff,0x33,0xff,0xff,0x66,0xff,0xcc,0x29,0x29,0x29,0x99,0xff,0xff,
  0xcc,0xff,0xff,0xff,0x66,0x66,0x66,0xff,0x66,0xff,0xff,0x66,0x66,0x66,0xff,0xff,
  0x66,0xff,0x66,0xff,0xff,0xa5,0x00,0x21,0x5f,0x5f,0x5f,0x77,0x77,0x77,0x86,0x86,
  0x86,0x96,0x96,0x96,0xcb,0xcb,0xcb,0xb2,0xb2,0xb2,0xd7,0xd7,0xd7,0xdd,0xdd,0xdd,
  0xe3,0xe3,0xe3,0xea,0xea,0xea,0xf1,0xf1,0xf1,0xf8,0xf8,0xf8,0xcc,0x00,0x00,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xd7,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x6a,0xdc,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x5a,
  0xfc,0x47,0xb2,0xa4,0xc9,0x93,0x28,0x53,0xaa,0x5c,0x69,0x12,0x80,0xc3,0x7f,0xb9,
  0x62,0xca,0x9c,0x49,0xb3,0xa6,0xcd,0x9b,0x38,0x63,0xae,0x73,0xd9,0x10,0x26,0x4e,
  0x00,0x40,0x01,0xe4,0x1c,0x4a,0x74,0xe7,0xcb,0x9b,0x41,0x93,0x0a,0x25,0xca,0x94,
  0xa6,0xd1,0x9e,0xb9,0x96,0xca,0x0c,0x9a,0x0f,0xa8,0xbe,0xa0,0x4d,0xb3,0xe6,0x7a,
  0xca,0x10,0xa6,0xd0,0xaf,0x54,0xab,0x02,0xd0,0x77,0x55,0x6a,0x52,0xad,0x36,0xb9,
  0x2e,0xf4,0x19,0x55,0x69,0x3e,0xb1,0x64,0x81,0xc6,0x94,0xdb,0x76,0xae,0xd4,0x9c,
  0x74,0xb7,0xf2,0xec,0x9a,0x2b,0x1f,0x4d,0xa0,0x62,0xad,0xd2,0xcd,0x3b,0xf5,0xee,
  0x4f,0xa9,0x6a,0x15,0xc2,0xf4,0xcb,0x38,0xb0,0xd2,0xc1,0x86,0xdb,0x2e,0x3d,0x2b,
  0xb9,0xb2,0xde,0xa3,0x31,0xdf,0xbe,0x0d,0xca,0x0f,0xe8,0x3e,0xac,0x76,0xf3,0x12,
  0xb6,0x5b,0xb7,0xf4,0x65,0xa8,0xfc,0xfe,0x02,0xe0,0xd7,0x19,0xc0,0xbe,0xcf,0x91,
  0x21,0x17,0xc6,0x2a,0x5b,0xe7,0xde,0xb5,0xb9,0x52,0xeb,0x6e,0xcd,0xda,0x33,0xec,
  0x9a,0xb5,0x65,0xd7,0x3e,0xcd,0x37,0x26,0xeb,0xde,0xab,0x93,0xfe,0x2e,0x1d,0x7c,
  0x32,0x58,0xd2,0xb6,0x8f,0x1a,0x7e,0x4c,0x59,0xf2,0x5d,0xc8,0xb4,0xcd,0x2e,0x4d,
  0x9c,0xd0,0x6b,0x54,0xeb,0xd7,0x47,0xfe,0xa3,0xbd,0xc9,0x1d,0x21,0x5b,0xa5,0x48,
  0xc7,0x17,0xbd,0xad,0x58,0xbd,0x7b,0xf2,0xec,0xbb,0x8b,0x74,0x18,0x1f,0x21,0xf5,
  0xfb,0xf8,0xf3,0xeb,0x57,0x3a,0xbf,0xff,0x41,0x96,0x00,0x96,0x54,0x1f,0x48,0x6c,
  0xa9,0x57,0x1e,0x81,0xef,0x11,0xd7,0x5f,0x81,0xe3,0x1d,0xf8,0x11,0x83,0xaa,0xe5,
  0xe4,0xa0,0x47,0x10,0xce,0x16,0x99,0x4c,0x13,0x76,0x54,0x61,0x54,0x3b,0x99,0xe6,
  0xd4,0x80,0x0f,0x22,0x95,0x0d,0x50,0xff,0x5c,0xa8,0xe0,0x7c,0x1b,0x02,0xe0,0xc2,
  0x8a,0xdf,0xa5,0x05,0x22,0x85,0x48,0xad,0xe8,0x42,0x8b,0x35,0x71,0x15,0x20,0x4b,
  0x98,0x01,0xb7,0x22,0x49,0xe2,0x11,0xb7,0xe1,0x50,0x19,0x0e,0x54,0xa1,0x4b,0x27,
  0x5d,0x68,0xa3,0x81,0x2f,0x12,0x34,0x24,0x4a,0x34,0x62,0xb8,0xd7,0x8f,0x12,0x26,
  0x29,0x24,0x70,0x25,0xe5,0xd2,0x92,0x61,0x47,0xe2,0xe5,0x62,0x8e,0x53,0x91,0xb4,
  0x8e,0x95,0x02,0x62,0xf9,0xa4,0x6a,0x66,0x6d,0x09,0xd5,0x4c,0x44,0xfe,0xb3,0x4e,
  0x91,0x84,0x65,0x39,0xd7,0x54,0x6f,0xd6,0x28,0xa5,0x40,0x0c,0xba,0xb0,0xa6,0x9a,
  0x6c,0xde,0xe5,0xe6,0x74,0x2d,0xea,0x39,0xe7,0x9a,0x33,0xc9,0x98,0x92,0x0b,0x40,
  0xcd,0xe8,0xe4,0x94,0x71,0xc2,0xf9,0xd5,0x87,0x39,0xca,0xb8,0xa3,0x49,0xb9,0x38,
  0x6a,0xa8,0x8f,0x70,0xa2,0xe9,0x61,0x74,0x67,0x46,0xea,0x68,0x91,0x92,0x1e,0x4a,
  0x67,0xa5,0x8a,0x36,0x79,0x62,0x77,0x81,0x6e,0x5a,0x62,0xa1,0x32,0xce,0xe4,0x26,
  0x8d,0x93,0x25,0x3a,0xaa,0x79,0x3f,0x34,0xc9,0xb8,0xd3,0x59,0x2c,0xaa,0x3a,0x26,
  0x9a,0xa2,0x81,0x1a,0xe4,0xa7,0xef,0xad,0x9a,0xd5,0xae,0x80,0xf6,0x7a,0x2b,0x5a,
  0xc0,0x42,0xb9,0x1e,0xa2,0xc4,0xfe,0xa9,0xa6,0x45,0x4f,0x52,0xf4,0xe7,0x7e,0xd0,
  0x0e,0x04,0x2d,0x7e,0xfe,0x55,0x6b,0xed,0xb5,0xd8,0x66,0xab,0xed,0xb6,0xdc,0x76,
  0xeb,0xed,0xb7,0x09,0x05,0x04,0x00,0x3b
  };

CDispenserButton::CDispenserButton()
{
	m_Properties.append(new CPropertyLayout<FXButton>(this));
	m_Properties.append(new CPropertyFrame<FXButton>(this));
	m_Properties.append(new CPropertyJustify<FXButton>(this));
	m_Properties.append(new CPropertyButtonStyle<FXButton>(this));
	m_Properties.append(new CPropertyBackColor<FXButton, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTextColor<FXButton, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyText<FXButton>(this));
	m_Properties.append(new CPropertyTipText<FXButton>(this));
	m_Properties.append(new CPropertyPadTop<FXButton>(this));
	m_Properties.append(new CPropertyPadLeft<FXButton>(this));
	m_Properties.append(new CPropertyPadBottom<FXButton>(this));
	m_Properties.append(new CPropertyPadRight<FXButton>(this));
	m_Properties.append(new CPropertyLeft<FXButton>(this));
	m_Properties.append(new CPropertyTop<FXButton>(this));
	m_Properties.append(new CPropertyWidth<FXButton>(this));
	m_Properties.append(new CPropertyHeight<FXButton>(this));
}

const char* CDispenserButton::getWidgetName(void)
{
	return "FXButton";
}

const void* CDispenserButton::getWidgetIcon(void)
{
	return Button;
}

const char* CDispenserButton::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserButton::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserButton::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserButton::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserButton::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserButton::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserButton::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserButton::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserButton::getClassInfo(void)
{
	return &FXButton::metaClass;
}

FXbool CDispenserButton::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserButton::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserButton::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserButton::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXButton::metaClass);
}

IDispenser* CDispenserButton::clone(void)
{
	return new CDispenserButton;
}

FXWindow* CDispenserButton::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXButton(p, "Button1", /*icon*/0, tgt, sel,
				FRAME_RAISED|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserButton::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXButton(" << parent << ", 0, 0, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);

}
