/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserCheckButton.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyJustify.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyIconPosition.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyBoxColor : public CProperty
{
public:
	CPropertyBoxColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyBoxColor<T>::name(void)
{
	return "BoxColor";
}

template <class T>
ePropertyType CPropertyBoxColor<T>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T>
const char* CPropertyBoxColor<T>::description(void)
{
	return "The box color of this object";
}

template <class T>
const char* CPropertyBoxColor<T>::tooltip(void)
{
	return "getBoxColor()\nsetBoxColor()";
}

template <class T>
void CPropertyBoxColor<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setBoxColor( getColorFromString(strValue) );
}

template <class T>
FXString CPropertyBoxColor<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getBoxColor() );
}

template <class T>
FXString CPropertyBoxColor<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	FXString str = getFXRGBString(p->getBoxColor());

	if ( str != "FXRGB(255, 255, 255)" )
	{
		return "\t" + widget + "->setBoxColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char CheckButton[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xce,0xce,0xce,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcf,0x74,0x6d,0xdf,0x78,0x28,0xec,0x7c,0xef,0xff,0xc0,0xa0,0x50,0xa8,0x19,
  0x1a,0x8f,0xc8,0x60,0x51,0xc0,0x21,0x24,0x9f,0xc3,0x65,0x00,0x40,0xad,0x5a,0x07,
  0x4e,0xa8,0xd6,0x27,0x05,0x10,0xbe,0x60,0x30,0x76,0x4b,0xde,0x75,0xc3,0x60,0xef,
  0xb8,0x57,0x15,0x00,0xc8,0x6f,0x65,0xc6,0xec,0x45,0x53,0x09,0x6b,0xf6,0x2e,0x0e,
  0xa5,0x12,0xe7,0x4c,0x5e,0x75,0x04,0x55,0x78,0x59,0x7a,0x6e,0x7e,0x89,0x71,0x6d,
  0x8a,0x7b,0x7f,0x18,0x74,0x84,0x82,0x77,0x86,0x3f,0x8c,0x3c,0x7c,0x6f,0x98,0x6e,
  0x99,0x90,0x17,0x92,0x56,0x83,0x79,0x9e,0x9d,0x7b,0x56,0xa6,0x9c,0xa6,0x40,0x67,
  0x93,0x83,0x96,0x3e,0xaa,0x7c,0x8f,0xa9,0xa5,0xac,0x80,0x53,0x69,0x68,0xa4,0xb4,
  0x9a,0x9e,0x9c,0xbe,0x72,0x91,0x81,0x68,0x61,0xbc,0x6d,0x89,0xa7,0xc0,0xbd,0x8e,
  0x5c,0xb8,0x03,0xd0,0xd1,0xd2,0xbc,0x65,0x49,0x4b,0xc5,0xd8,0xd5,0x5b,0x4b,0xda,
  0xdd,0xb7,0xc3,0xde,0xe1,0x3c,0xdc,0xe2,0xe1,0xe4,0xe5,0xdd,0xe7,0xe8,0xd5,0xea,
  0xeb,0x64,0x52,0x1b,0x87,0xee,0x3f,0x5d,0xa2,0x57,0xf2,0xf3,0x3d,0xad,0xd8,0xd4,
  0xf9,0x4c,0xb8,0xaf,0x76,0xe1,0x6b,0xb4,0x05,0x19,0x3d,0x80,0xd8,0xbe,0x50,0x53,
  0x95,0x24,0x98,0xb3,0x61,0xb9,0x12,0x2e,0x54,0x46,0x71,0x51,0xb2,0x58,0xdf,0x40,
  0x11,0x93,0x88,0xaf,0x16,0xad,0x4e,0x5b,0xcb,0x6c,0x3d,0xd4,0x18,0x91,0x5f,0x47,
  0x59,0xa8,0x42,0xae,0x3a,0x08,0x31,0xa0,0xb1,0x93,0x1f,0x67,0x79,0x64,0xc8,0x92,
  0xa4,0x4b,0x31,0x30,0x57,0xf9,0x5a,0x26,0xb3,0xa6,0x05,0x49,0x1c,0x63,0x29,0x1a,
  0x3a,0xb4,0xd7,0xa2,0x66,0xe3,0x9e,0x4d,0x9b,0xd6,0xd1,0xdf,0xb5,0x84,0x61,0xfc,
  0xf9,0xac,0x20,0xb5,0x4c,0xbb,0xaa,0x47,0xae,0x62,0x8d,0x92,0xa3,0xab,0xd7,0xaf,
  0x60,0xc3,0x8a,0x1d,0x4b,0xb6,0xac,0xd9,0xb3,0x68,0xd3,0xaa,0x5d,0xcb,0xb6,0x6d,
  0x86,0x08,0x00,0x00,0x3b
  };

CDispenserCheckButton::CDispenserCheckButton()
{
	m_Properties.append(new CPropertyLayout<FXCheckButton>(this));
	m_Properties.append(new CPropertyFrame<FXCheckButton>(this));
	m_Properties.append(new CPropertyJustify<FXCheckButton>(this));
	m_Properties.append(new CPropertyIconPosition<FXCheckButton>(this));
	m_Properties.append(new CPropertyBackColor<FXCheckButton, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTextColor<FXCheckButton, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyBoxColor<FXCheckButton>(this));
	m_Properties.append(new CPropertyText<FXCheckButton>(this));
	m_Properties.append(new CPropertyTipText<FXCheckButton>(this));
	m_Properties.append(new CPropertyPadTop<FXCheckButton>(this));
	m_Properties.append(new CPropertyPadLeft<FXCheckButton>(this));
	m_Properties.append(new CPropertyPadBottom<FXCheckButton>(this));
	m_Properties.append(new CPropertyPadRight<FXCheckButton>(this));
	m_Properties.append(new CPropertyLeft<FXCheckButton>(this));
	m_Properties.append(new CPropertyTop<FXCheckButton>(this));
	m_Properties.append(new CPropertyWidth<FXCheckButton>(this));
	m_Properties.append(new CPropertyHeight<FXCheckButton>(this));
}

const char* CDispenserCheckButton::getWidgetName(void)
{
	return "FXCheckButton";
}

const void* CDispenserCheckButton::getWidgetIcon(void)
{
	return CheckButton;
}

const char* CDispenserCheckButton::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserCheckButton::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserCheckButton::getDefaultWidth(void)
{
	return 120;
}

FXint CDispenserCheckButton::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserCheckButton::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserCheckButton::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserCheckButton::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserCheckButton::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserCheckButton::getClassInfo(void)
{
	return &FXCheckButton::metaClass;
}

FXbool CDispenserCheckButton::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserCheckButton::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserCheckButton::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserCheckButton::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXCheckButton::metaClass);
}

IDispenser* CDispenserCheckButton::clone(void)
{
	return new CDispenserCheckButton;
}

FXWindow* CDispenserCheckButton::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXCheckButton((FXComposite*)p, "CheckBox1", 0, 0, 
				LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|ICON_BEFORE_TEXT|JUSTIFY_RIGHT);
}

void CDispenserCheckButton::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("IconPosition").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXCheckButton(" << parent << ", \"\", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream  << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);

}
