/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserGroupBox.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyText.h>

#include <CVariant.h>

const unsigned char GroupBox[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x5c,0xc7,0x40,0xdf,0x0b,0x80,0x55,0x50,0xf8,0xcb,
  0xe5,0x76,0xb0,0x21,0x6b,0xc8,0x5c,0x1a,0x71,0xc8,0x97,0x72,0xd9,0x74,0x05,0xaf,
  0x55,0xec,0x14,0x56,0xbb,0x00,0x7d,0x54,0x21,0x11,0x0b,0x14,0x37,0x9b,0x97,0x81,
  0xa0,0x3b,0x68,0xbb,0xcb,0xd6,0xaa,0x59,0x5c,0x3e,0x8b,0xdd,0xee,0x80,0x9a,0x2d,
  0xb5,0xd2,0x95,0x57,0x7f,0x70,0x81,0x70,0x2e,0x7a,0x6b,0x19,0x6b,0x6a,0x7e,0x2d,
  0x72,0x3e,0x84,0x5f,0x54,0x5a,0x3a,0x7b,0x89,0x87,0x4f,0x98,0x4e,0x34,0x95,0x18,
  0x8a,0x99,0x9f,0x5b,0x86,0x9c,0x5e,0x97,0x9f,0xa6,0x5c,0xa3,0x16,0x9e,0xa7,0xac,
  0x2d,0x87,0x7c,0xad,0xb1,0xaf,0x96,0x8b,0xb1,0xac,0xb3,0x9d,0xa5,0xb6,0xa6,0xb8,
  0xa4,0xb5,0xbb,0x9f,0xbd,0xaa,0xba,0xc0,0x98,0xc2,0x15,0xab,0xc5,0x99,0xc7,0x14,
  0xc9,0xca,0x4f,0xcc,0x13,0xce,0xcf,0x47,0xa9,0xc8,0xc4,0xd4,0x37,0xd1,0x12,0xd3,
  0xd9,0x36,0xdb,0x01,0xdd,0xde,0x9b,0x88,0xb9,0xbf,0xe3,0x33,0xe0,0xe2,0xe8,0x2f,
  0xea,0xd8,0xec,0xed,0xd6,0xcd,0xef,0xf0,0xae,0xf2,0xd2,0xf4,0xf5,0x2b,0xee,0xe7,
  0xfa,0xf6,0xe5,0xbe,0xfc,0x51,0x02,0x38,0xac,0x9f,0x40,0x4f,0xb0,0x0e,0xfe,0x4b,
  0xa8,0x70,0xdf,0x3d,0x6e,0xf9,0xf4,0xf1,0x6b,0xb8,0x90,0x16,0x45,0x16,0x13,0x2f,
  0x22,0xb4,0xa8,0x31,0xe3,0x45,0x8f,0x3b,0x14,0x41,0x36,0x14,0xa9,0x90,0xe4,0x41,
  0x93,0x02,0x51,0xfa,0x53,0x29,0xf1,0x61,0xb8,0x88,0xf5,0x58,0xc6,0x74,0xb9,0x6e,
  0x25,0x4d,0x98,0xf0,0xdc,0xa5,0x48,0x41,0x13,0x8f,0xcf,0x9f,0x40,0x83,0x0a,0x1d,
  0xfa,0x26,0x8a,0x46,0x8c,0x3b,0x93,0x2a,0x5d,0xca,0xb4,0xa9,0xd3,0xa7,0x50,0xa3,
  0x8e,0x88,0x00,0x00,0x3b
  };

CDispenserGroupBox::CDispenserGroupBox()
{
	m_Properties.append(new CPropertyLayout<FXGroupBox>(this));
	m_Properties.append(new CPropertyFrame<FXGroupBox>(this));
	m_Properties.append(new CPropertyText<FXGroupBox>(this));
	m_Properties.append(new CPropertyBackColor<FXGroupBox, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTextColor<FXGroupBox, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyLeft<FXGroupBox>(this));
	m_Properties.append(new CPropertyTop<FXGroupBox>(this));
	m_Properties.append(new CPropertyWidth<FXGroupBox>(this));
	m_Properties.append(new CPropertyHeight<FXGroupBox>(this));
}

const char* CDispenserGroupBox::getWidgetName(void)
{
	return "FXGroupBox";
}

const void* CDispenserGroupBox::getWidgetIcon(void)
{
	return GroupBox;
}

const char* CDispenserGroupBox::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserGroupBox::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserGroupBox::getDefaultWidth(void)
{
	return 120;
}

FXint CDispenserGroupBox::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserGroupBox::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserGroupBox::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserGroupBox::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserGroupBox::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserGroupBox::getClassInfo(void)
{
	return &FXGroupBox::metaClass;
}

FXbool CDispenserGroupBox::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserGroupBox::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserGroupBox::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserGroupBox::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXGroupBox::metaClass);
}

IDispenser* CDispenserGroupBox::clone(void)
{
	return new CDispenserGroupBox;
}

FXWindow* CDispenserGroupBox::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXGroupBox(p, "Frame1", 
				FRAME_RIDGE|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserGroupBox::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXGroupBox(" << parent << ", \"Caption\", ";
	stream << strOptions << ", " << pWindow->getX() << ", " << pWindow->getY() << ", ";
	stream << pWindow->getWidth() << ", " << pWindow->getHeight() << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
