/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserList.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertySelBackColor.h>
#include <CPropertySelTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyListing.h>
#include <CPropertyVisible.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyListStyle : public CProperty
{
public:
	CPropertyListStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);
	
	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

///////////////////////////////////////////////////////////////////////////////
//

// From FXList.h
template <class T>
OPTION CPropertyListStyle<T>::Style[] = 
{
	{"LIST_SINGLESELECT"  , LIST_SINGLESELECT  ,"At most one selected item"},
	{"LIST_BROWSESELECT"  , LIST_BROWSESELECT  ,"Always exactly one selected item"},
	{"LIST_MULTIPLESELECT", LIST_MULTIPLESELECT,"Multiple selection mode\nSame as LIST_SINGLESELECT | LIST_BROWSESELECT"},
	{"LIST_AUTOSELECT"    , LIST_AUTOSELECT    ,"Automatically select under cursor"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyListStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyListStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyListStyle<T>::description(void)
{
	return "The list style for this object";
}

template <class T>
const char* CPropertyListStyle<T>::tooltip(void)
{
	return "getListStyle()\nsetListStyle()";
}

template <class T>
void CPropertyListStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setListStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyListStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getListStyle() );
}

template <class T>
OPTION* CPropertyListStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyListStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getListStyle() == 0 )
		return;
}

const unsigned char ListBox[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x00,0x00,0x84,0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x50,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0x2b,0x0e,0x70,
  0x2c,0xcf,0x74,0x6d,0xdf,0xb7,0x36,0xb8,0x25,0xb1,0x67,0x3b,0x80,0x70,0x48,0x2c,
  0x1a,0x8f,0xc8,0x24,0x71,0xe0,0xd3,0x09,0x00,0x81,0xa8,0x74,0x4a,0xad,0x5a,0xad,
  0x38,0x18,0x80,0xf9,0xc3,0x04,0xaf,0xe0,0xf0,0x95,0x49,0x28,0x9b,0xcd,0x4f,0xae,
  0x13,0x1a,0x30,0xb7,0xc5,0x57,0xc2,0xd8,0x67,0x4b,0x37,0x81,0x4f,0xa9,0xfc,0x0d,
  0xa7,0x96,0xe7,0x39,0x5b,0x77,0x5e,0x79,0x51,0x7b,0x6e,0x6d,0x7f,0x89,0x72,0x8a,
  0x86,0x80,0x31,0x5b,0x31,0x76,0x5d,0x17,0x5f,0x8e,0x7c,0x98,0x8c,0x97,0x7b,0x98,
  0x54,0x5c,0x5a,0x42,0x92,0x82,0x94,0x16,0x96,0x7c,0x9c,0x67,0x9a,0xa7,0x7a,0x8f,
  0x45,0x30,0x93,0x6b,0xac,0x99,0x7e,0x9b,0xb2,0x55,0x5c,0x47,0x41,0x6a,0x78,0x6c,
  0xab,0x97,0x6f,0x87,0xbf,0x9c,0x9e,0x74,0x35,0xb0,0xbc,0x86,0x8a,0xca,0xca,0xb5,
  0x8b,0x58,0xc5,0x34,0xc7,0x84,0xbd,0x7d,0xd5,0x9e,0x1b,0xa3,0xb1,0xd6,0xdb,0x52,
  0x4a,0x43,0xbb,0xd3,0xa9,0xe2,0xe3,0xe4,0xe5,0x59,0x03,0xd9,0xbc,0xe5,0xeb,0xec,
  0xa9,0x64,0xe4,0xd2,0x95,0x4f,0x65,0x43,0x04,0x00,0xed,0xe2,0xf5,0xe2,0x9f,0xc6,
  0xe9,0xe1,0xf4,0x00,0xf1,0x99,0xb9,0x17,0xf0,0x0c,0xbf,0x68,0xfe,0xe4,0x11,0xb4,
  0x47,0x4f,0x48,0x43,0x82,0xf5,0x1c,0xa6,0x5a,0x68,0xe6,0xe0,0x8c,0x78,0xa5,0xe6,
  0x05,0x5c,0xc8,0x91,0xa1,0x47,0x8a,0xfe,0x14,0x2b,0x42,0xbb,0x98,0x30,0x63,0x47,
  0x8f,0xf6,0x88,0x7c,0x2c,0x18,0x52,0x64,0x20,0x70,0x0a,0x07,0xb2,0x9c,0xb8,0xb1,
  0xe0,0x3e,0x6c,0x30,0x4d,0xca,0x44,0xd9,0x11,0x22,0xc0,0x96,0x67,0xbc,0x85,0x1a,
  0x14,0x33,0x25,0xc4,0xa3,0x3e,0x57,0x16,0xd9,0xe7,0xa3,0x29,0x17,0xa7,0x39,0x2b,
  0x04,0x11,0x48,0x75,0xdc,0xd3,0xab,0x4e,0x89,0xea,0xac,0xca,0x35,0x2b,0xd6,0x4f,
  0x5a,0xa5,0x6a,0xec,0x5a,0xf5,0xab,0xd7,0xb0,0x14,0xa6,0x1a,0x45,0x49,0x55,0x9f,
  0xbb,0xb3,0x57,0xa3,0xa6,0x1d,0x9b,0xb4,0xed,0xce,0xb7,0x66,0xaf,0xae,0xd9,0xa9,
  0xaf,0xaf,0x43,0x89,0x41,0x99,0xe6,0xa5,0x83,0x76,0x82,0xda,0x95,0x33,0x93,0x72,
  0x04,0x0a,0x17,0xee,0xde,0x9a,0x0f,0xff,0x42,0xbe,0xeb,0xb2,0x31,0x8c,0xc2,0x12,
  0x0e,0xd7,0x0d,0xa9,0x98,0xa6,0x60,0xcb,0x98,0x05,0x68,0x9e,0xf9,0xb3,0xe6,0x49,
  0x83,0xa0,0x2f,0x93,0x12,0x8b,0xb4,0xe1,0x43,0xc8,0xad,0xad,0xa6,0x0e,0x7d,0x98,
  0x2c,0xbe,0xc1,0x60,0x57,0xcf,0x65,0x6c,0xdb,0x6a,0x96,0x92,0xac,0x7b,0x0b,0x7c,
  0x37,0x0e,0x63,0xf0,0xb5,0xbc,0xd9,0x01,0xb5,0x28,0xc3,0xf8,0x6e,0xbe,0x64,0x01,
  0xa3,0x46,0x67,0x44,0x17,0x6d,0xba,0xaf,0xd7,0x22,0xef,0x29,0x1b,0xd4,0x37,0xeb,
  0xba,0x0d,0x63,0xe7,0x29,0xb3,0x73,0xdd,0xe9,0xde,0x45,0xc9,0x15,0x7f,0x72,0x71,
  0x44,0xd3,0x2a,0xf1,0x42,0x6a,0x0e,0xfc,0xf9,0x64,0xce,0x93,0xd9,0xba,0xac,0x53,
  0x9f,0x3d,0xf4,0xc4,0xf9,0x2d,0x38,0x37,0x12,0x7d,0xeb,0x65,0xa6,0x91,0x5f,0xd9,
  0x75,0x46,0x59,0x45,0x38,0x5d,0x97,0x9c,0x70,0x03,0x09,0x55,0xa0,0x68,0xa2,0x9d,
  0x63,0xe1,0x85,0x38,0x5c,0x07,0xe1,0x86,0x06,0xe9,0x80,0xe1,0x87,0x20,0x5e,0xc4,
  0xc3,0x88,0x24,0x96,0x68,0xe2,0x89,0x28,0xa6,0xa8,0xe2,0x8a,0x24,0x46,0x00,0x00,
  0x3b
  };


CDispenserList::CDispenserList()
{
	m_Properties.append(new CPropertyLayout<FXList>(this));
	m_Properties.append(new CPropertyListStyle<FXList>(this));
	m_Properties.append(new CPropertyBackColor<FXList, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTextColor<FXList, FXRGB(0, 0, 0)>(this));
//	m_Properties.append(new CPropertyListing<FXList>);
	m_Properties.append(new CPropertyVisible<FXList, 10>(this));
	m_Properties.append(new CPropertySelBackColor<FXList>(this));
	m_Properties.append(new CPropertySelTextColor<FXList>(this));
	m_Properties.append(new CPropertyLeft<FXList>(this));
	m_Properties.append(new CPropertyTop<FXList>(this));
	m_Properties.append(new CPropertyWidth<FXList>(this));
	m_Properties.append(new CPropertyHeight<FXList>(this));
}

const char* CDispenserList::getWidgetName(void)
{
	return "FXList";
}

const void* CDispenserList::getWidgetIcon(void)
{
	return ListBox;
}

const char* CDispenserList::getWidgetCategory(void)
{
	return "Standard";
}

const int   CDispenserList::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserList::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserList::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserList::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserList::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserList::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserList::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserList::getClassInfo(void)
{
	return &FXList::metaClass;
}

FXbool CDispenserList::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserList::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserList::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserList::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXList::metaClass);
}

IDispenser* CDispenserList::clone(void)
{
	return new CDispenserList;
}

FXWindow* CDispenserList::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXList(p, 10, tgt, sel, 
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserList::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXList(" << parent << ", 10, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << ", " << getProperty("Top").get(pWindow) << ", ";
	stream << getProperty("Width").get(pWindow) << ", " << getProperty("Height").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
