/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserRadioButton.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyJustify.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyIconPosition.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyRadioColor : public CProperty
{
public:
	CPropertyRadioColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyRadioColor<T>::name(void)
{
	return "RadioColor";
}

template <class T>
ePropertyType CPropertyRadioColor<T>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T>
const char* CPropertyRadioColor<T>::description(void)
{
	return "The radio color of this object";
}

template <class T>
const char* CPropertyRadioColor<T>::tooltip(void)
{
	return "getRadioColor()\nsetRadioColor()";
}

template <class T>
void CPropertyRadioColor<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setRadioColor( getColorFromString(strValue) );
}

template <class T>
FXString CPropertyRadioColor<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getRadioColor() );
}

template <class T>
FXString CPropertyRadioColor<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	FXString str = getFXRGBString(p->getRadioColor());

	if ( str != "FXRGB(255, 255, 255)" )
	{
		return "\t" + widget + "->setRadioColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char RadioButton[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xce,0xce,0xce,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0xaa,0xae,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcf,0x74,0x6d,0xdf,0x78,0x28,0xec,0x7c,0xef,0xff,0xc0,0xa0,0x50,0xa8,0x19,
  0x1a,0x8f,0xc8,0x60,0x31,0xc9,0x6c,0x2a,0x33,0xbc,0x81,0x74,0x40,0x99,0x0e,0x9c,
  0xd8,0xde,0x52,0x20,0x95,0x00,0xbe,0x00,0x89,0x34,0x4b,0xde,0x52,0xbf,0x84,0x34,
  0x01,0x7d,0xed,0x81,0x05,0x80,0x6c,0xfc,0x89,0x89,0x06,0x00,0x6a,0xf5,0x80,0xd0,
  0x76,0xef,0xe6,0x4d,0x5f,0x44,0x50,0x3b,0x77,0x6a,0x68,0x69,0x7b,0x3f,0x73,0x6f,
  0x70,0x82,0x8e,0x71,0x8f,0x7f,0x83,0x75,0x85,0x78,0x6b,0x60,0x89,0x04,0x8b,0x93,
  0x9d,0x70,0x9f,0xa0,0x80,0x80,0x3f,0x5b,0x86,0x98,0x88,0x8a,0x3e,0x8c,0x3c,0x60,
  0x82,0xab,0xa0,0xb0,0xa4,0x84,0x02,0xa6,0x98,0x7a,0x9b,0xaa,0x9e,0xa3,0xa1,0xba,
  0x94,0x17,0x76,0x97,0x79,0x7b,0x7d,0xac,0xbd,0x93,0xaf,0xa2,0xbe,0x16,0x76,0x52,
  0xc2,0xc3,0xaa,0x8f,0xd1,0xd1,0x9d,0x8c,0x92,0x3e,0x66,0x7b,0x7c,0x53,0x89,0xc4,
  0x64,0x4c,0xd8,0xcd,0x7a,0x53,0xde,0x58,0xe0,0x56,0xe7,0xe4,0x4e,0x5b,0xe9,0xec,
  0x5a,0xb3,0xed,0xf0,0xeb,0xf0,0xec,0xf2,0xf3,0xe4,0xf5,0xf6,0x65,0xef,0xf9,0xde,
  0xe6,0x55,0xe3,0xfc,0xe8,0xfc,0xda,0xd1,0xe5,0x0e,0x18,0x31,0xdd,0x02,0xf2,0x30,
  0x63,0x30,0x0f,0x1b,0x68,0x91,0xb0,0x34,0x92,0x55,0x89,0x4b,0xad,0x5b,0x09,0x5f,
  0x31,0x49,0x46,0x71,0x20,0xad,0x60,0x77,0x79,0xf8,0xe0,0xf2,0x03,0xe9,0x8f,0xb4,
  0x48,0xbb,0x62,0x2d,0x9c,0x75,0x51,0xd8,0xc8,0x62,0xb0,0x92,0x21,0x23,0xd9,0x71,
  0x99,0xa5,0x90,0xb7,0x38,0xc5,0x6c,0xc5,0x2b,0xa6,0xb2,0x0a,0x0b,0x41,0xba,0xd4,
  0xc9,0x11,0xe6,0xcc,0x9f,0x14,0x80,0xe1,0x7c,0x96,0xcb,0xe7,0x31,0x6a,0x2a,0x6b,
  0x02,0x25,0x18,0x20,0x1c,0x37,0x3e,0x10,0x4d,0x9a,0x3c,0x8a,0x72,0xa2,0xbb,0x8a,
  0xe1,0xac,0x70,0x53,0x08,0xc4,0x9c,0xb3,0x31,0x64,0xaf,0xcd,0x3a,0xc7,0x36,0x21,
  0x59,0x7c,0x69,0x91,0xc0,0x8d,0x6b,0x24,0x87,0xdd,0xbb,0x78,0xf3,0xea,0xdd,0xcb,
  0xb7,0xaf,0xdf,0xbf,0x80,0x03,0x0b,0x1e,0x4c,0xb8,0xb0,0x61,0x0d,0x11,0x00,0x00,
  0x3b
  };

CDispenserRadioButton::CDispenserRadioButton()
{
	m_Properties.append(new CPropertyLayout<FXRadioButton>(this));
	m_Properties.append(new CPropertyFrame<FXRadioButton>(this));
	m_Properties.append(new CPropertyJustify<FXRadioButton>(this));
	m_Properties.append(new CPropertyIconPosition<FXRadioButton>(this));
	m_Properties.append(new CPropertyBackColor<FXRadioButton, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTextColor<FXRadioButton, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyRadioColor<FXRadioButton>(this));
	m_Properties.append(new CPropertyText<FXRadioButton>(this));
	m_Properties.append(new CPropertyTipText<FXRadioButton>(this));
	m_Properties.append(new CPropertyPadTop<FXRadioButton>(this));
	m_Properties.append(new CPropertyPadLeft<FXRadioButton>(this));
	m_Properties.append(new CPropertyPadBottom<FXRadioButton>(this));
	m_Properties.append(new CPropertyPadRight<FXRadioButton>(this));
	m_Properties.append(new CPropertyLeft<FXRadioButton>(this));
	m_Properties.append(new CPropertyTop<FXRadioButton>(this));
	m_Properties.append(new CPropertyWidth<FXRadioButton>(this));
	m_Properties.append(new CPropertyHeight<FXRadioButton>(this));
}

const char* CDispenserRadioButton::getWidgetName(void)
{
	return "FXRadioButton";
}

const void* CDispenserRadioButton::getWidgetIcon(void)
{
	return RadioButton;
}

const char* CDispenserRadioButton::getWidgetCategory(void)
{
	return "Standard";
}

const int CDispenserRadioButton::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserRadioButton::getDefaultWidth(void)
{
	return 120;
}

FXint CDispenserRadioButton::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserRadioButton::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserRadioButton::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserRadioButton::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserRadioButton::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserRadioButton::getClassInfo(void)
{
	return &FXRadioButton::metaClass;
}

FXbool CDispenserRadioButton::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserRadioButton::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserRadioButton::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserRadioButton::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXRadioButton::metaClass);
}

IDispenser* CDispenserRadioButton::clone(void)
{
	return new CDispenserRadioButton;
}

FXWindow* CDispenserRadioButton::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXRadioButton((FXComposite*)p, "Radio1", 0, 0, 
				LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT|ICON_BEFORE_TEXT|JUSTIFY_RIGHT);
}

void CDispenserRadioButton::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("IconPosition").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXRadioButton(" << parent << ", \"Caption\", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
