/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserShutter.h"

#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyJustify.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyFrame.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

class CPropertyShutterItemText : public CPropertyText<FXButton>
{
public:
	CPropertyShutterItemText(IDispenser* pDispenser) : CPropertyText<FXButton>(pDispenser) {}

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWidget, FXString& widget);
};

void CPropertyShutterItemText::set(FXWindow* pWindow, const FXString strValue)
{
	FXShutterItem* p = static_cast<FXShutterItem*>(pWindow);
	p->getButton()->setText(strValue);
}

FXString CPropertyShutterItemText::get(FXWindow* pWindow)
{
	FXShutterItem* p = static_cast<FXShutterItem*>(pWindow);
	return p->getButton()->getText();
}

FXString CPropertyShutterItemText::getImplementationCode(FXWindow* pWidget, FXString& widget)
{
	FXShutterItem* p = static_cast<FXShutterItem*>(pWidget);
	return "\t" + widget + "->getButton()->setText(\"" + p->getButton()->getText() + "\");\n";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char ShutterItem[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x5c,0xe7,0x7c,0xaf,0x67,0xbe,0xa0,0x70,0x27,0x2c,
  0xe2,0x88,0xab,0x81,0x72,0xc9,0x6c,0x3a,0x9f,0x50,0xa8,0x0b,0x29,0x18,0x18,0x8d,
  0x00,0x40,0x8b,0x6a,0x9d,0x65,0xb5,0xda,0x2b,0x0b,0x10,0x08,0x0b,0xb8,0xb7,0xb0,
  0x59,0x9c,0x2d,0xaf,0xd0,0x36,0x75,0x76,0xf5,0xa5,0xcf,0xe7,0xde,0x72,0xe0,0x0d,
  0x6c,0x75,0xbd,0x74,0x63,0x82,0x6a,0x81,0x32,0x6d,0x61,0x70,0x80,0x02,0x66,0x5f,
  0x77,0x85,0x6b,0x2f,0x64,0x64,0x7c,0x18,0x2e,0x7f,0x86,0x85,0x8b,0x2e,0x84,0x9a,
  0x86,0x92,0x94,0x17,0x96,0x69,0x99,0x8c,0x9d,0x9c,0x31,0x92,0x7b,0x67,0x7d,0x2c,
  0x97,0x30,0x75,0x8b,0x72,0x9c,0xa7,0xaf,0x9f,0xab,0x95,0x7e,0x62,0x42,0xa9,0xa0,
  0x16,0x9b,0x8d,0xc0,0xc1,0xc2,0xc3,0xc4,0xc0,0x6e,0xb7,0xa1,0x5b,0x67,0xcb,0x7b,
  0xcd,0xcc,0xcf,0xce,0xd1,0xd0,0xd3,0xcd,0x93,0xc8,0xbe,0xba,0x62,0x7a,0x88,0xac,
  0x6f,0xd9,0x41,0x7a,0xbd,0x15,0xdf,0x57,0xdb,0xe2,0x14,0x53,0xe4,0x3d,0xe1,0xd7,
  0xe3,0xea,0x43,0xb6,0x54,0xaa,0xef,0x37,0xec,0x54,0xf4,0x47,0xf1,0xdd,0xcb,0xf8,
  0x36,0xf6,0xfb,0xfd,0xfc,0xe9,0xc3,0xc5,0x62,0x5e,0x40,0x18,0xff,0x08,0x1e,0x14,
  0x78,0x4c,0xde,0x42,0x1d,0xd6,0xee,0x3d,0x7c,0x61,0xae,0x1d,0x3a,0x65,0x13,0xa7,
  0x44,0x04,0x98,0x71,0xcb,0xc0,0x64,0x79,0x05,0x3b,0x7a,0x54,0x25,0x51,0xe4,0x99,
  0x8f,0xd8,0x42,0x9a,0x3c,0x49,0x92,0xa3,0xc9,0x8a,0x0e,0x57,0x26,0x04,0xb9,0xb2,
  0x20,0x4a,0x77,0x2a,0x5f,0x6e,0x54,0x58,0x93,0x25,0x37,0x9e,0x06,0x3b,0xce,0x4c,
  0xd9,0xd3,0xe7,0xb9,0x09,0xe9,0x64,0xee,0xa4,0xd9,0x13,0xe6,0xbe,0xa0,0x19,0x87,
  0xe2,0x2c,0xea,0x14,0x68,0x4d,0xa9,0x17,0x95,0x49,0xdb,0x4a,0xad,0x2b,0xd7,0x6a,
  0x0d,0x01,0xa6,0x1a,0xbb,0xad,0x2c,0xd9,0xb3,0x66,0xd3,0x2e,0x25,0x3a,0x46,0xad,
  0x5b,0xb4,0x70,0xdf,0x1e,0x95,0x80,0xea,0xad,0xdd,0xb8,0x78,0x5b,0xf2,0x2c,0xba,
  0xc5,0x25,0xdf,0x14,0x80,0x03,0x0b,0x1e,0x4c,0xb8,0xb0,0xe1,0xc3,0x88,0x09,0x47,
  0x00,0x00,0x3b
  };

CDispenserShutterItem::CDispenserShutterItem()
{
	m_Properties.append(new CPropertyShutterItemText(this));
	m_Properties.append(new CPropertyLayout<FXShutterItem>(this));
	m_Properties.append(new CPropertyFrame<FXShutterItem>(this));
	m_Properties.append(new CPropertyPadTop<FXShutterItem>(this));
	m_Properties.append(new CPropertyPadLeft<FXShutterItem>(this));
	m_Properties.append(new CPropertyPadBottom<FXShutterItem>(this));
	m_Properties.append(new CPropertyPadRight<FXShutterItem>(this));
	m_Properties.append(new CPropertyLeft<FXShutterItem>(this));
	m_Properties.append(new CPropertyTop<FXShutterItem>(this));
	m_Properties.append(new CPropertyWidth<FXShutterItem>(this));
	m_Properties.append(new CPropertyHeight<FXShutterItem>(this));
}

const char* CDispenserShutterItem::getWidgetName(void)
{
	return "FXShutterItem";
}

const void* CDispenserShutterItem::getWidgetIcon(void)
{
	return ShutterItem;
}

const char* CDispenserShutterItem::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserShutterItem::getWidgetGrabbers(void)
{
	return GS_NONE;
}

FXint CDispenserShutterItem::getDefaultWidth(void)
{
	return 40;
}

FXint CDispenserShutterItem::getDefaultHeight(void)
{
	return 80;
}

FXint CDispenserShutterItem::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserShutterItem::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserShutterItem::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserShutterItem::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserShutterItem::getClassInfo(void)
{
	return &FXShutterItem::metaClass;
}

FXbool CDispenserShutterItem::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserShutterItem::allowParent(FXWindow* pParent)
{
	return pParent->isMemberOf(&FXShutter::metaClass);
}

FXWindow* CDispenserShutterItem::getCreateParent(FXWindow* pParent)
{
	FXShutterItem* pItem = static_cast<FXShutterItem*>(pParent);
	return pItem->getContent();
}

/*
	FXShutterItem
	  FXButton
	  FXScrollWindow
	    FXVerticalFrame
*/
FXbool CDispenserShutterItem::IsValidWidget(FXWindow** ppWindow)
{
	if ( (*ppWindow)->isMemberOf(&FXShutterItem::metaClass) )
	{
		return TRUE;
	}
	else if ( (*ppWindow)->isMemberOf(&FXButton::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		return *ppWindow && (*ppWindow)->isMemberOf(&FXShutterItem::metaClass);
	}
	else if ( (*ppWindow)->isMemberOf(&FXVerticalFrame::metaClass) )
	{
		*ppWindow = (*ppWindow)->getParent();
		if ( *ppWindow && (*ppWindow)->isMemberOf(&FXScrollWindow::metaClass) ) 
		{
			*ppWindow = (*ppWindow)->getParent();
			return *ppWindow && (*ppWindow)->isMemberOf(&FXShutterItem::metaClass);
		}
	}

	return FALSE;
}

IDispenser* CDispenserShutterItem::clone(void)
{
	return new CDispenserShutterItem;
}

FXWindow* CDispenserShutterItem::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	FXShutterItem* pItem = new FXShutterItem((FXShutter*)p, "Shutter", 0,
				LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT);

	pItem->enable();
	pItem->getContent()->enable();

	return pItem;
}

void CDispenserShutterItem::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXShutterItem(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Shutter[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x0c,0x64,0x79,0xef,0xd3,0xbb,0xcb,0x6f,0x48,0xdc,
  0x05,0x2b,0xc4,0x64,0x2f,0x70,0x44,0xb6,0x06,0x02,0xa8,0x34,0x4a,0x9d,0x5a,0xab,
  0xd8,0xab,0xf6,0x65,0x14,0x2a,0xbf,0x2d,0x80,0x8b,0x89,0x71,0x41,0xc1,0xe8,0xae,
  0x05,0x9d,0x1e,0x37,0x29,0x66,0x36,0x58,0xed,0x94,0x27,0xc5,0x2d,0xb2,0x97,0x75,
  0xb6,0x17,0xf5,0x75,0x7e,0x3f,0x78,0x2c,0x80,0x81,0x54,0x82,0x43,0x74,0x70,0x89,
  0x83,0x6e,0x65,0x4f,0x8d,0x83,0x86,0x13,0x5c,0x02,0x62,0x98,0x97,0x9a,0x99,0x9c,
  0x9b,0x9e,0x9d,0xa0,0x8f,0x7b,0x2b,0x01,0x02,0xa5,0xa7,0xa6,0xa9,0xa8,0xab,0xaa,
  0xad,0xac,0xaf,0x5c,0x94,0x12,0x92,0x3e,0xa5,0x79,0x6f,0x95,0x79,0xb4,0x3d,0x8b,
  0xb9,0xbb,0x37,0xb6,0x85,0xb8,0xb3,0xba,0xbf,0x37,0xbd,0xc4,0xc6,0x33,0xc1,0xa4,
  0xc3,0xcc,0xa9,0xca,0x33,0xc8,0xcf,0xd1,0x63,0xa2,0x6b,0xc5,0xd5,0x30,0xd3,0xda,
  0x31,0xcf,0xb2,0xc9,0xa4,0xdd,0xdb,0xe0,0xe3,0x2f,0xdf,0xce,0xd6,0xe6,0x61,0xe5,
  0xeb,0x85,0xd7,0x87,0xd4,0xe3,0xdc,0xee,0xe2,0xb7,0x90,0xef,0xf5,0x97,0xed,0xf5,
  0xe8,0xf8,0xf6,0xf5,0xe8,0xf5,0x83,0xc7,0x28,0x5f,0x40,0x7e,0xee,0xfc,0x8d,0x82,
  0x76,0x70,0x98,0x3e,0x86,0xcd,0xfe,0x41,0x5c,0x27,0x30,0x21,0x41,0x5f,0x00,0xdd,
  0x55,0x5c,0xa7,0x10,0x9b,0x41,0x8d,0x43,0x08,0x39,0x5e,0x0c,0x37,0xd1,0xdc,0x46,
  0x57,0x28,0x61,0xa9,0x4c,0x89,0xf2,0xde,0xc2,0x87,0xfb,0x1c,0xc2,0x74,0xe9,0x71,
  0x66,0xcc,0x97,0x0f,0xc1,0xc9,0xd3,0x37,0x8d,0x4c,0x97,0x9f,0x3e,0x83,0x02,0x1d,
  0x2a,0xb4,0x28,0x51,0x67,0x12,0x8e,0x1a,0x5d,0xaa,0xb4,0xe9,0xd2,0x0c,0x4e,0xa3,
  0x32,0x9d,0x4a,0x34,0x85,0xd5,0xab,0x58,0x51,0x44,0x00,0x00,0x3b
  };

CDispenserShutter::CDispenserShutter()
{
	m_Properties.append(new CPropertyLayout<FXShutter>(this));
	m_Properties.append(new CPropertyPadTop<FXShutter>(this));
	m_Properties.append(new CPropertyPadLeft<FXShutter>(this));
	m_Properties.append(new CPropertyPadBottom<FXShutter>(this));
	m_Properties.append(new CPropertyPadRight<FXShutter>(this));
	m_Properties.append(new CPropertyLeft<FXShutter>(this));
	m_Properties.append(new CPropertyTop<FXShutter>(this));
	m_Properties.append(new CPropertyWidth<FXShutter>(this));
	m_Properties.append(new CPropertyHeight<FXShutter>(this));
}

const char* CDispenserShutter::getWidgetName(void)
{
	return "FXShutter";
}

const void* CDispenserShutter::getWidgetIcon(void)
{
	return Shutter;
}

const char* CDispenserShutter::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserShutter::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserShutter::getDefaultWidth(void)
{
	return 40;
}

FXint CDispenserShutter::getDefaultHeight(void)
{
	return 80;
}

FXint CDispenserShutter::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserShutter::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserShutter::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserShutter::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserShutter::getClassInfo(void)
{
	return &FXShutter::metaClass;
}

FXbool CDispenserShutter::allowChild(const FXMetaClass* pChildClass)
{
	return pChildClass->isSubClassOf(&FXShutterItem::metaClass);
}

FXbool CDispenserShutter::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserShutter::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserShutter::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXShutter::metaClass);
}

IDispenser* CDispenserShutter::clone(void)
{
	return new CDispenserShutter;
}

FXWindow* CDispenserShutter::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	FXShutter* pShutter = new FXShutter(p, tgt, sel, 
				LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);

	return pShutter;
}

void CDispenserShutter::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXShutter(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}




