/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserSplitter.h"

#include <CPropertyLeft.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyText.h>
#include <CPropertyTipText.h>
#include <CPropertyTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertySplitterStyle : public CProperty
{
public:
	CPropertySplitterStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
//	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXSplitter.h
template <class T>
OPTION CPropertySplitterStyle<T>::Style[] = 
{
	{"SPLITTER_HORIZONTAL", SPLITTER_HORIZONTAL, "Split horizontally"},
	{"SPLITTER_VERTICAL"  , SPLITTER_VERTICAL  , "Split vertically"},
	{"SPLITTER_REVERSED"  , SPLITTER_REVERSED  , "Reverse-anchored"},
	{"SPLITTER_TRACKING"  , SPLITTER_TRACKING  , "Track continuous during split"},
	{0, 0, 0}
};


template <class T>
const char* CPropertySplitterStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertySplitterStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertySplitterStyle<T>::description(void)
{
	return "The splitter style for this object";
}

template <class T>
const char* CPropertySplitterStyle<T>::tooltip(void)
{
	return "getNumColumns()\nsetNumColumns()";
}

template <class T>
void CPropertySplitterStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setSplitterStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertySplitterStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getSplitterStyle() );
}

template <class T>
OPTION* CPropertySplitterStyle<T>::option(void)
{
	return Style;
}
/*
template <class T>
void CPropertySplitterStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getSplitterStyle();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}
*/
///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyBarSize : public CProperty
{
public:
	CPropertyBarSize(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);
};


template <class T>
const char* CPropertyBarSize<T>::name(void)
{
	return "Bar Size";
}

template <class T>
ePropertyType CPropertyBarSize<T>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T>
const char* CPropertyBarSize<T>::description(void)
{
	return "Sets or returns the splitter's\nbar size -- the area between\nadjacent child widgets.";
}

template <class T>
const char* CPropertyBarSize<T>::tooltip(void)
{
	return "getBarSize()\nsetBarSize()";
}

template <class T>
void CPropertyBarSize<T>::set(FXWindow* pWindow, FXString strValue)
{
	FXSplitter* p = static_cast<FXSplitter*>(pWindow);

	p->setBarSize( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyBarSize<T>::get(FXWindow* pWindow)
{
	FXSplitter* p = static_cast<FXSplitter*>(pWindow);
	return getStringFromInt( p->getBarSize() );
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Splitter[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x0c,0x64,0x79,0xef,0xd3,0xbb,0xcb,0x6f,0x48,0xdc,
  0x05,0x2b,0xc4,0x64,0x2f,0x70,0x44,0xb6,0x4a,0x03,0x01,0x29,0xda,0x32,0x0a,0x9f,
  0x80,0xac,0x76,0xcb,0xdd,0x0a,0xa2,0xcc,0xae,0xd8,0x4b,0x65,0x31,0x31,0x2e,0xe6,
  0x60,0xcd,0x6e,0xbb,0xd9,0x5f,0x29,0xe0,0x4d,0x6f,0xc7,0xab,0x67,0x4b,0x7a,0x5e,
  0xef,0xc7,0xd5,0x7d,0x74,0x77,0x66,0x4d,0x14,0x7b,0x81,0x75,0x7f,0x7c,0x88,0x76,
  0x65,0x2b,0x56,0x7a,0x58,0x8c,0x6f,0x8a,0x93,0x8d,0x7b,0x68,0x92,0x96,0x70,0x60,
  0x8b,0x96,0x83,0x8f,0x79,0x4e,0x84,0x9b,0x9c,0x72,0xa5,0x51,0x8e,0x72,0x99,0xa4,
  0xa8,0x95,0xa5,0xa0,0x02,0x90,0xa3,0x2b,0x80,0xb0,0x9d,0xae,0xaa,0xa2,0xb4,0xa7,
  0xb9,0xbd,0x9b,0xb1,0xb3,0x86,0x9a,0xb7,0xbf,0x9f,0xba,0x85,0x13,0x87,0xbe,0xb6,
  0xc7,0x2e,0xc2,0xca,0xc4,0xc0,0xb8,0xc5,0x58,0xac,0xb5,0x9e,0xce,0xcd,0x93,0xc1,
  0xbb,0xd1,0xad,0xc5,0xdb,0x8c,0xb1,0xde,0xdf,0xd8,0xa8,0xa9,0xc6,0xdc,0xaa,0xd0,
  0x12,0xcb,0xe1,0xd9,0xe3,0xc8,0xd7,0xea,0xeb,0xf5,0x88,0xdd,0xc9,0x01,0x69,0x49,
  0x60,0x44,0xf3,0x57,0x58,0xa0,0x6b,0x34,0x90,0x5d,0xb9,0x7d,0x4a,0x12,0xce,0x38,
  0x88,0xd0,0xcc,0x14,0x29,0x23,0x0c,0xea,0xdb,0x33,0xa6,0x62,0x96,0x4e,0x16,0xc5,
  0x90,0xd3,0xd7,0xf0,0x1c,0xb3,0x78,0xb4,0xf8,0x24,0x06,0xf4,0x08,0xcf,0x97,0xb5,
  0x91,0xf7,0x42,0xa6,0xf4,0x23,0x32,0x12,0xb8,0x69,0x2b,0x13,0x01,0x74,0x49,0x12,
  0xa6,0x38,0x95,0x78,0x26,0x4a,0xd3,0x06,0x32,0xd0,0x46,0x7a,0x37,0x55,0x06,0x65,
  0xf9,0xec,0xe0,0x3b,0x9b,0x3d,0x89,0x9e,0xa4,0x19,0x53,0x66,0x53,0x4a,0x2d,0x79,
  0x0d,0x25,0x3a,0x55,0xd0,0x4c,0xa9,0x49,0xa9,0x66,0xb5,0x5a,0x54,0xe7,0x4b,0x9e,
  0x26,0x09,0x01,0xdd,0xca,0xb5,0x2a,0xd4,0xae,0x28,0xcd,0x9e,0x55,0x7b,0x69,0x29,
  0x56,0x74,0xaf,0x60,0xe6,0x4c,0x4b,0x76,0x6d,0xdd,0xb6,0x62,0xe9,0xc2,0xa5,0x26,
  0x97,0x45,0xbb,0x8e,0x4f,0xed,0x86,0xc5,0x36,0xf6,0x9f,0x14,0xc3,0x73,0x99,0x16,
  0x5c,0xf3,0xa5,0x20,0x26,0x94,0x0a,0x23,0x87,0xf2,0x2a,0xb9,0x32,0xc3,0xca,0x98,
  0x65,0x19,0xcd,0x2c,0xf9,0x32,0xe7,0xc8,0x7f,0xcf,0x58,0x19,0x2d,0xba,0x34,0xe9,
  0xd3,0xa6,0x53,0xa3,0xe6,0x28,0x61,0xb5,0xea,0xd7,0xae,0x63,0xbf,0xce,0x20,0xbb,
  0x36,0xec,0xdb,0xa8,0x53,0xe8,0xde,0xcd,0x1b,0x45,0x04,0x00,0x00,0x3b
  };


CDispenserSplitter::CDispenserSplitter()
{
	m_Properties.append(new CPropertyLayout<FXSplitter>(this));
	m_Properties.append(new CPropertySplitterStyle<FXSplitter>(this));
	m_Properties.append(new CPropertyLeft<FXSplitter>(this));
	m_Properties.append(new CPropertyTop<FXSplitter>(this));
	m_Properties.append(new CPropertyWidth<FXSplitter>(this));
	m_Properties.append(new CPropertyHeight<FXSplitter>(this));
	m_Properties.append(new CPropertyBarSize<FXSplitter>(this));
}

const char* CDispenserSplitter::getWidgetName(void)
{
	return "FXSplitter";
}

const void* CDispenserSplitter::getWidgetIcon(void)
{
	return Splitter;
}

const char* CDispenserSplitter::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserSplitter::getWidgetGrabbers(void)
{
	return GS_NONE;
}

FXint CDispenserSplitter::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserSplitter::getDefaultHeight(void)
{
	return 16;
}

FXint CDispenserSplitter::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserSplitter::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserSplitter::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserSplitter::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserSplitter::getClassInfo(void)
{
	return &FXSplitter::metaClass;
}

FXbool CDispenserSplitter::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserSplitter::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserSplitter::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserSplitter::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXSplitter::metaClass);
}

IDispenser* CDispenserSplitter::clone(void)
{
	return new CDispenserSplitter;
}

FXWindow* CDispenserSplitter::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXSplitter(p, LAYOUT_SIDE_TOP|LAYOUT_FILL_X|LAYOUT_FILL_Y|SPLITTER_VERTICAL|SPLITTER_TRACKING);
}

void CDispenserSplitter::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXSplitter(" << parent << ", ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << ", " << getProperty("Top").get(pWindow) << ", ";
	stream << getProperty("Width").get(pWindow) << ", " << getProperty("Height").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}

