/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserTab.h"

#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyText.h>
#include <CPropertyJustify.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyFrame.h>
#include <CPropertyTipText.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyOrientation : public CProperty
{
public:
	CPropertyOrientation(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXTab.h
template <class T>
OPTION CPropertyOrientation<T>::Style[] = 
{
	{"TAB_TOP"   , TAB_TOP   , ""},
	{"TAB_LEFT"  , TAB_LEFT  , ""},
	{"TAB_RIGHT" , TAB_RIGHT , ""},
	{"TAB_BOTTOM", TAB_BOTTOM, ""},
	{0, 0, 0}
};


template <class T>
const char* CPropertyOrientation<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyOrientation<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyOrientation<T>::description(void)
{
	return "Identifies the position of the\ntab on this object";
}

template <class T>
const char* CPropertyOrientation<T>::tooltip(void)
{
	return "getTabOrientation()\nsetTabOrientation()";
}

template <class T>
void CPropertyOrientation<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setTabOrientation( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyOrientation<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getTabOrientation() );
}

template <class T>
OPTION* CPropertyOrientation<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyOrientation<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getTabOrientation();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char TabItem[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x5c,0xe7,0x7c,0xaf,0x67,0xbe,0xa0,0x70,0x27,0x2c,
  0xe2,0x88,0xaf,0x81,0x72,0xc9,0x5c,0xb2,0x9a,0xd0,0x81,0x0f,0xd9,0x92,0xde,0x06,
  0x01,0xeb,0x4c,0x7b,0x04,0x26,0x79,0x58,0x1c,0xd7,0x46,0x5d,0x8d,0xb7,0x00,0x31,
  0xaf,0x2c,0x38,0xcb,0xb0,0x69,0x9b,0xfb,0x87,0x81,0xcd,0xed,0x80,0x40,0xfc,0xbd,
  0xf6,0xba,0xee,0x49,0x7a,0x01,0x72,0x7d,0x75,0x5f,0x39,0x03,0x79,0x7b,0x31,0x80,
  0x2e,0x6c,0x8d,0x55,0x82,0x84,0x39,0x8f,0x60,0x8a,0x93,0x5d,0x86,0x4f,0x4a,0x60,
  0x51,0x9e,0x9f,0x8b,0x02,0x54,0x90,0x46,0x32,0x00,0x8b,0xa3,0xa5,0x45,0x97,0x2b,
  0xa9,0xaa,0x41,0xa7,0x7a,0xad,0x7e,0x66,0xaf,0xb0,0x92,0xa2,0xb4,0x6d,0xb6,0x3e,
  0xb1,0x71,0xae,0xbc,0x39,0x8a,0x83,0xb9,0x9a,0xb5,0xc1,0xc2,0xac,0xc0,0xc8,0x36,
  0xc3,0xb3,0xc6,0xbb,0xcc,0x37,0xbe,0xcf,0x17,0x7f,0xd2,0xd3,0xb8,0xcb,0xd8,0x30,
  0xd4,0xc5,0xd6,0x55,0xdc,0x33,0xce,0xdf,0x16,0xd7,0xe2,0x31,0xde,0xdb,0xe8,0x2c,
  0xe4,0xeb,0xec,0x02,0xea,0xba,0xa4,0xcc,0xee,0xf3,0xf0,0x2f,0xf2,0xd0,0xf4,0xc8,
  0xf6,0xfb,0xf8,0x2e,0xf4,0x81,0x7b,0x02,0xb0,0x85,0xbf,0x81,0xc7,0x0a,0xc6,0x53,
  0x76,0x4f,0x61,0x3c,0x6d,0x0d,0x15,0x0a,0x34,0x17,0xce,0xe1,0x41,0x8a,0x04,0x2d,
  0x32,0xfc,0x67,0x11,0x22,0x47,0x89,0x5b,0x1b,0x11,0x46,0x93,0xe8,0x51,0x24,0xbf,
  0x60,0x13,0x2b,0x9c,0x23,0x99,0xa7,0x1a,0xc6,0x84,0x05,0x53,0x52,0x58,0x19,0xb3,
  0xe4,0xcb,0x36,0xa7,0x72,0xea,0xdc,0xc9,0xb3,0xa7,0xcf,0x9f,0xb2,0xca,0xa9,0x0c,
  0x28,0x68,0x98,0xd1,0xa2,0x48,0x8f,0x2a,0x4d,0x8a,0xd4,0xe5,0x50,0xa2,0x4b,0xa3,
  0x32,0x95,0x1a,0xd5,0xe9,0xcc,0x6e,0x53,0xb3,0x52,0xd5,0xda,0x54,0xe8,0x55,0x87,
  0x64,0x74,0x81,0x85,0x91,0xa2,0xac,0xd9,0xb3,0x68,0xd3,0xaa,0x5d,0xcb,0xb6,0x6d,
  0xda,0x08,0x00,0x00,0x3b
  };

CDispenserTabItem::CDispenserTabItem()
{
	m_Properties.append(new CPropertyOrientation<FXTabItem>(this));
	m_Properties.append(new CPropertyText<FXTabItem>(this));
	m_Properties.append(new CPropertyTipText<FXTabItem>(this));
	m_Properties.append(new CPropertyPadTop<FXTabItem>(this));
	m_Properties.append(new CPropertyPadLeft<FXTabItem>(this));
	m_Properties.append(new CPropertyPadBottom<FXTabItem>(this));
	m_Properties.append(new CPropertyPadRight<FXTabItem>(this));
	m_Properties.append(new CPropertyLeft<FXTabItem>(this));
	m_Properties.append(new CPropertyTop<FXTabItem>(this));
	m_Properties.append(new CPropertyWidth<FXTabItem>(this));
	m_Properties.append(new CPropertyHeight<FXTabItem>(this));

	m_pFrame = 0;
	m_pTabItem = 0;
}

CDispenserTabItem::~CDispenserTabItem()
{
//	if ( m_pFrame )
//		delete m_pFrame;
}

const char* CDispenserTabItem::getWidgetName(void)
{
	return "FXTabItem";
}

const void* CDispenserTabItem::getWidgetIcon(void)
{
	return TabItem;
}

const char* CDispenserTabItem::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserTabItem::getWidgetGrabbers(void)
{
	return GS_NONE;
}

FXint CDispenserTabItem::getDefaultWidth(void)
{
	return 40;
}

FXint CDispenserTabItem::getDefaultHeight(void)
{
	return 80;
}

FXint CDispenserTabItem::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserTabItem::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserTabItem::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserTabItem::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserTabItem::getClassInfo(void)
{
	return &FXTabItem::metaClass;
}

FXbool CDispenserTabItem::allowChild(const FXMetaClass*)
{
	return TRUE;
}

FXbool CDispenserTabItem::allowParent(FXWindow* pParent)
{
	return pParent->isMemberOf(&FXTabBook::metaClass);
}

FXWindow* CDispenserTabItem::getCreateParent(FXWindow* pParent)
{
	// Return the embedded horizontal frame, not the tab item
	return m_pFrame;
}

/*
   FXTabBook
     FXTabItem              ) One to
	 FXHorizontalFrame      ) Many
*/
FXbool CDispenserTabItem::IsValidWidget(FXWindow** ppWindow)
{
	if ( (*ppWindow)->isMemberOf(&FXTabItem::metaClass) )
	{
		return TRUE;
	}
	else if ( (*ppWindow)->isMemberOf(&FXHorizontalFrame::metaClass) )
	{
		if ( *ppWindow != m_pFrame )
			return FALSE;

		*ppWindow = (*ppWindow)->getParent();

		// The parent of the horizontal frame is the FXTabBook, not the
		// FXTabItem as you would perhaps expect.
		
		// This horizontal frame is perfectly valid and may need to be used
		// as the parent for other child windows. But this frame was
		// created by this dispenser and is hidden from the remainder
		// of the DlgEdit machinery.
		//
		// We can't return a simple 
		//
		// (*ppWindow)->isMemberOf(&FXHorizontalFrame::metaClass)
		//
		// here because the calling code (CAppDialog::getWidget) won't
		// recognize this frame. 
		
		// CAppDialog::getWidget is expecting a pointer to an FXTabItem
		// as a valid widget (returned from CDispenserTabItem::createWidget)
		// so we return m_pTabItem instead to fool CAppDialog::getWidget

		FXbool bReturn = /*(*ppWindow == m_pFrame) &&*/ (*ppWindow)->isMemberOf(&FXTabBook::metaClass);
		*ppWindow = m_pTabItem;
		return bReturn;
	}

	return FALSE;
}

IDispenser* CDispenserTabItem::clone(void)
{
	return new CDispenserTabItem;
}

FXWindow* CDispenserTabItem::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	if ( m_pFrame )
		delete m_pFrame;

	FXTabBook* pBook = (FXTabBook*)p;

	m_pTabItem = new FXTabItem(pBook, "TabItem", 0);

	m_pFrame = new FXHorizontalFrame(p,FRAME_THICK|FRAME_RAISED);
	m_pFrame->enable();
	m_pFrame->create();
	m_pFrame->show();
/*	
	char c[50];
	sprintf(c, "m_pFrame = %p\n", m_pFrame);
	OutputDebugString(c);
	sprintf(c, "m_pTabItem = %p\n", m_pTabItem);
	OutputDebugString(c);
*/
	pBook->recalc();

	return m_pTabItem;
}

void CDispenserTabItem::getDefinitionCode(ITextStream& stream, FXWindow* pWidget, FXString& widget, FXString& parent)
{
	stream << "\tFXHorizontalFrame* " << widget << "Frame;\n";
}

void CDispenserTabItem::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXTabItem(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	stream << "\t" << widget << "Frame = new FXHorizontalFrame(" << parent << ",FRAME_THICK|FRAME_RAISED);\n";

	getPropertyCode(stream, pWindow, widget);
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyTabStyle : public CProperty
{
public:
	CPropertyTabStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

// From FXTab.h
template <class T>
OPTION CPropertyTabStyle<T>::Style[] = 
{
	{"TABBOOK_TOPTABS",    TABBOOK_TOPTABS,    ""},
	{"TABBOOK_BOTTOMTABS", TABBOOK_BOTTOMTABS, ""},
	{"TABBOOK_SIDEWAYS",   TABBOOK_SIDEWAYS ,  ""},
	{"TABBOOK_LEFTTABS",   TABBOOK_LEFTTABS,   ""},
	{"TABBOOK_RIGHTTABS",  TABBOOK_RIGHTTABS,  ""},
	{0, 0, 0}
};

template <class T>
const char* CPropertyTabStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyTabStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyTabStyle<T>::description(void)
{
	return "Identifies the position of the\ntab on this object";
}

template <class T>
const char* CPropertyTabStyle<T>::tooltip(void)
{
	return "getTabStyle()\nsetTabStyle()";
}

template <class T>
void CPropertyTabStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setTabStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyTabStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getTabStyle() );
}

template <class T>
OPTION* CPropertyTabStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyTabStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getTabStyle();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

const unsigned char Tab[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xb3,0x00,0x00,0x00,0x00,0x00,
  0x84,0x84,0x84,0xc6,0xc6,0xc6,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,
  0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x04,0xfe,0x30,0xc8,0x49,0xab,0xbd,0x38,0xeb,
  0xcd,0xbb,0xff,0x60,0x28,0x8e,0x64,0x69,0x9e,0x68,0x2a,0xac,0x6c,0xeb,0xbe,0x70,
  0x2c,0xcb,0xda,0x6c,0xdf,0x78,0x0c,0x64,0x79,0xef,0xd3,0xbb,0xcb,0x6f,0x48,0xdc,
  0x05,0x2b,0xc4,0x64,0x2f,0x70,0x44,0xc6,0x06,0x02,0xa8,0x34,0x4a,0x85,0x2a,0x67,
  0x46,0xe1,0x75,0x1b,0x63,0x62,0x60,0x56,0x5c,0x98,0xcb,0xca,0x5a,0xc8,0x68,0x96,
  0x57,0xeb,0x1a,0xdb,0xdc,0x64,0xb3,0x33,0x4d,0x5e,0x9f,0x5f,0x70,0x59,0x7e,0x2b,
  0xa7,0xd0,0xd1,0x76,0x73,0x2d,0x7b,0x60,0x7f,0x02,0x7d,0x13,0x86,0x5c,0x81,0x7e,
  0x2d,0x01,0x02,0x8f,0x91,0x90,0x93,0x92,0x95,0x94,0x97,0x96,0x99,0x2f,0x88,0x12,
  0x8a,0x3f,0x8f,0x8e,0x4d,0x8d,0x6a,0x9e,0x3e,0x9c,0xa0,0xa5,0x38,0xa8,0x2b,0x8c,
  0x89,0x8e,0xa9,0x39,0xa7,0xb0,0xaa,0x2e,0xad,0x9d,0xaf,0xb3,0x36,0xb2,0xb9,0x33,
  0xab,0x90,0xa2,0xae,0xa4,0xbc,0x32,0xbb,0xc3,0x30,0xbe,0xb6,0xc8,0xc6,0x3a,0xb6,
  0xcb,0xc7,0xb5,0xc0,0xb7,0xc2,0xce,0x2d,0xc5,0xd4,0xac,0xd0,0x5f,0xb8,0xd7,0x2b,
  0xd6,0xd7,0xc8,0xd1,0xca,0xdc,0xdd,0xcd,0xe3,0x93,0xa1,0xda,0xd3,0xdc,0xde,0xd4,
  0xe0,0xe9,0xd8,0xe6,0xec,0xce,0xee,0x6c,0xf0,0xe3,0xf2,0xcb,0xf4,0x77,0xea,0xd7,
  0xf8,0xc6,0xfa,0x82,0xce,0xdd,0x2b,0x37,0x0e,0xe0,0x28,0x7b,0xeb,0x08,0x72,0x33,
  0x18,0x0c,0x61,0x3f,0x85,0xdf,0xb2,0xd5,0x13,0x98,0x30,0xda,0x0b,0x4d,0x18,0x31,
  0x69,0xcc,0x28,0x09,0xdd,0x44,0x73,0x31,0xe4,0x2c,0x82,0x94,0xb8,0x6f,0x64,0xc8,
  0x8f,0x20,0x93,0x99,0xac,0x96,0x2c,0x8b,0x4b,0x2f,0x2f,0x63,0xc2,0x9c,0x29,0xb3,
  0xe6,0xcc,0x0c,0x36,0x73,0xd2,0xdc,0xa9,0xd3,0x26,0x4e,0x9e,0x40,0x7b,0x06,0x1d,
  0x1a,0x2e,0x85,0xd1,0xa3,0x48,0x43,0x44,0x00,0x00,0x3b
  };

CDispenserTabBook::CDispenserTabBook()
{
	m_Properties.append(new CPropertyLayout<FXTabBook>(this));
	m_Properties.append(new CPropertyTabStyle<FXTabBook>(this));
	m_Properties.append(new CPropertyPadTop<FXTabBook>(this));
	m_Properties.append(new CPropertyPadLeft<FXTabBook>(this));
	m_Properties.append(new CPropertyPadBottom<FXTabBook>(this));
	m_Properties.append(new CPropertyPadRight<FXTabBook>(this));
	m_Properties.append(new CPropertyHeight<FXTabBook>(this));
	m_Properties.append(new CPropertyLeft<FXTabBook>(this));
	m_Properties.append(new CPropertyTop<FXTabBook>(this));
	m_Properties.append(new CPropertyWidth<FXTabBook>(this));
}

const char* CDispenserTabBook::getWidgetName(void)
{
	return "FXTabBook";
}

const void* CDispenserTabBook::getWidgetIcon(void)
{
	return Tab;
}

const char* CDispenserTabBook::getWidgetCategory(void)
{
	return "Containers";
}

const int CDispenserTabBook::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserTabBook::getDefaultWidth(void)
{
	return 40;
}

FXint CDispenserTabBook::getDefaultHeight(void)
{
	return 80;
}

FXint CDispenserTabBook::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserTabBook::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserTabBook::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserTabBook::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserTabBook::getClassInfo(void)
{
	return &FXTabBook::metaClass;
}

FXbool CDispenserTabBook::allowChild(const FXMetaClass* pChildClass)
{
	return pChildClass->isSubClassOf(&FXTabItem::metaClass);
}

FXbool CDispenserTabBook::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserTabBook::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserTabBook::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXTabBook::metaClass);
}

IDispenser* CDispenserTabBook::clone(void)
{
	return new CDispenserTabBook;
}

FXWindow* CDispenserTabBook::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXTabBook(p, tgt, sel, 
				LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserTabBook::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXTabBook(" << parent << ", 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}


