/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserToggleButton.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyText.h>
#include <CPropertyJustify.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>
#include <CPropertyPadBottom.h>
#include <CPropertyPadLeft.h>
#include <CPropertyPadRight.h>
#include <CPropertyPadTop.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyToggleStyle : public CProperty
{
public:
	CPropertyToggleStyle(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

///////////////////////////////////////////////////////////////////////////////
//

// From FXToggleButton.h
template <class T>
OPTION CPropertyToggleStyle<T>::Style[] = 
{
	{"TOGGLEBUTTON_AUTOGRAY", TOGGLEBUTTON_AUTOGRAY, "Automatically gray out when not updated"},
	{"TOGGLEBUTTON_AUTOHIDE", TOGGLEBUTTON_AUTOHIDE, "Automatically hide toggle button when not updated"},
	{"TOGGLEBUTTON_TOOLBAR",  TOGGLEBUTTON_TOOLBAR,  "Toolbar style toggle button [flat look]"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyToggleStyle<T>::name(void)
{
	return "Style";
}

template <class T>
ePropertyType CPropertyToggleStyle<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyToggleStyle<T>::description(void)
{
	return "The toggle style of this object";
}

template <class T>
const char* CPropertyToggleStyle<T>::tooltip(void)
{
	return "getToggleStyle()\nsetToggleStyle()";
}

template <class T>
void CPropertyToggleStyle<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setToggleStyle( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyToggleStyle<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt( p->getToggleStyle() );
}

template <class T>
OPTION* CPropertyToggleStyle<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyToggleStyle<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p->getToggleStyle() == 0 )
		return;

	CProperty::getOptions(pWindow, strOptions);
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyAltText : public CProperty
{
public:
	CPropertyAltText(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyAltText<T>::name(void)
{
	return "AltText";
}

template <class T>
ePropertyType CPropertyAltText<T>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T>
const char* CPropertyAltText<T>::description(void)
{
	return "The alternate text\nassociated with this object";
}

template <class T>
const char* CPropertyAltText<T>::tooltip(void)
{
	return "getAltText()\nsetAltText()";
}

template <class T>
void CPropertyAltText<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setAltText(strValue);
}

template <class T>
FXString CPropertyAltText<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return p->getAltText();
}

template <class T>
FXString CPropertyAltText<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	FXString str = get(pWindow);

	if ( str != "" )
	{
		return "\t" + widget + "->setAltText(\"" + get(pWindow) + "\");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

template <class T>
class CPropertyAltTipText : public CProperty
{
public:
	CPropertyAltTipText(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyAltTipText<T>::name(void)
{
	return "AltTipText";
}

template <class T>
ePropertyType CPropertyAltTipText<T>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T>
const char* CPropertyAltTipText<T>::description(void)
{
	return "The alternate tip text\nassociated with this object";
}

template <class T>
const char* CPropertyAltTipText<T>::tooltip(void)
{
	return "getAltTipText()\nsetAltTipText()";
}

template <class T>
void CPropertyAltTipText<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setAltTipText(strValue);
}

template <class T>
FXString CPropertyAltTipText<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return p->getAltTipText();
}

template <class T>
FXString CPropertyAltTipText<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	FXString str = get(pWindow);

	if ( str != "" )
	{
		return "\t" + widget + "->setAltTipText(\"" + get(pWindow) + "\");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char Toggle[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0xfe,0xdb,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x8a,
  0x0c,0x09,0x40,0xe1,0xbf,0x03,0x28,0x53,0xaa,0x5c,0xc9,0xb2,0xa5,0xcb,0x97,0x2c,
  0xdb,0x95,0x4c,0x78,0x12,0xa6,0xcd,0x9b,0x38,0x55,0xca,0x34,0x99,0xb3,0xa7,0xcf,
  0x98,0x33,0x11,0xd6,0xfc,0xd9,0x13,0x80,0xcf,0x9d,0x34,0x61,0x02,0x58,0xba,0x94,
  0xe8,0x01,0xa3,0x3d,0x91,0x0a,0xbd,0x09,0xd5,0x69,0x55,0x9c,0x52,0x0f,0x0e,0x7d,
  0x79,0x95,0x69,0x4a,0xa6,0x50,0xc1,0x3e,0x45,0xe9,0x15,0x66,0x56,0x83,0x5b,0x5d,
  0x56,0x5d,0x3b,0x96,0x6c,0xdb,0xb1,0x46,0xd9,0xbe,0x3c,0x5b,0x30,0x6d,0x4b,0xb9,
  0x70,0xbf,0xbe,0x8d,0xfb,0x54,0xec,0xdc,0xa0,0x5a,0xa9,0xea,0x75,0xcb,0xb6,0xf0,
  0x5b,0x9b,0x74,0x09,0xda,0x65,0x89,0x97,0x2f,0x61,0xbd,0x8e,0x0f,0x03,0xe5,0x69,
  0xb3,0x6b,0x53,0xb2,0x65,0xc5,0x86,0xbd,0xfc,0x97,0xb2,0xd3,0x95,0x57,0x25,0x63,
  0x05,0x8c,0xf6,0xf3,0x57,0xce,0x65,0x7f,0x26,0x1e,0xb8,0xd8,0xb4,0xeb,0x03,0xab,
  0x05,0xb6,0x7e,0xfd,0x39,0x36,0xbe,0x7f,0xed,0x72,0xeb,0xde,0xcd,0xbb,0xb7,0xef,
  0xdf,0xc0,0x09,0xea,0x26,0x5d,0x10,0xac,0xf1,0xe3,0xc8,0x93,0x2b,0x5f,0x6e,0x3c,
  0xa3,0xf3,0xe7,0xd0,0xa3,0x4b,0x9f,0x4e,0xbd,0xba,0xf5,0xeb,0xd8,0xb3,0x6b,0xdf,
  0xce,0xbd,0xbb,0xf7,0xef,0xe0,0xc3,0xae,0x8b,0x1f,0x4f,0xbe,0xbc,0xf9,0xf3,0x08,
  0x99,0xab,0x5f,0xcf,0x9e,0xe9,0x3f,0x85,0xc4,0xc5,0xbf,0x4f,0x58,0x92,0xb6,0x7d,
  0x94,0xf3,0xd3,0xe3,0xbb,0x6f,0x3f,0xff,0xc1,0xfa,0xfc,0xbd,0xe6,0x9f,0x41,0x00,
  0x06,0xa8,0x52,0x68,0x36,0x0d,0x58,0xdc,0x7e,0x5c,0xf9,0xf5,0x13,0x82,0x30,0x29,
  0x48,0x50,0x81,0x5c,0x99,0x06,0xe1,0x4b,0x12,0x0e,0x44,0xa1,0x5a,0x07,0xa2,0x96,
  0x99,0x57,0x9b,0x5d,0x98,0x52,0x86,0x02,0x6d,0x78,0xd7,0x60,0x79,0x3d,0xf6,0x18,
  0x5e,0x2d,0x91,0x88,0x8f,0x89,0x8c,0xa1,0x18,0xd9,0x8c,0x70,0x39,0xd8,0x22,0x7c,
  0x0c,0x2a,0x25,0xe3,0x5e,0x3c,0x46,0x96,0x20,0x8e,0x82,0xb9,0xa5,0x62,0x8a,0x79,
  0xb1,0xc8,0x92,0x8b,0x30,0x82,0xd6,0xe1,0x5a,0x1f,0x36,0x15,0xe2,0x8f,0xf4,0xe5,
  0xf8,0x5a,0x68,0x22,0x62,0x08,0xa4,0x6b,0xa9,0xa5,0xe6,0x13,0x92,0x52,0x1a,0x48,
  0x14,0x92,0xed,0x78,0x69,0x9a,0x8b,0x23,0x95,0x69,0xe6,0x99,0x21,0xa1,0xa7,0xe6,
  0x9a,0x6c,0xb6,0xc9,0x66,0x40,0x00,0x3b
  };

CDispenserToggleButton::CDispenserToggleButton()
{
	m_Properties.append(new CPropertyLayout<FXToggleButton>(this));
	m_Properties.append(new CPropertyFrame<FXToggleButton>(this));
	m_Properties.append(new CPropertyJustify<FXToggleButton>(this));
	m_Properties.append(new CPropertyToggleStyle<FXToggleButton>(this));
	m_Properties.append(new CPropertyBackColor<FXToggleButton, FXRGB(192, 192, 192)>(this));
	m_Properties.append(new CPropertyTextColor<FXToggleButton, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertyText<FXToggleButton>(this));
	m_Properties.append(new CPropertyTipText<FXToggleButton>(this));
	m_Properties.append(new CPropertyAltText<FXToggleButton>(this));
	m_Properties.append(new CPropertyAltTipText<FXToggleButton>(this));
	m_Properties.append(new CPropertyPadTop<FXToggleButton>(this));
	m_Properties.append(new CPropertyPadLeft<FXToggleButton>(this));
	m_Properties.append(new CPropertyPadBottom<FXToggleButton>(this));
	m_Properties.append(new CPropertyPadRight<FXToggleButton>(this));
	m_Properties.append(new CPropertyLeft<FXToggleButton>(this));
	m_Properties.append(new CPropertyTop<FXToggleButton>(this));
	m_Properties.append(new CPropertyWidth<FXToggleButton>(this));
	m_Properties.append(new CPropertyHeight<FXToggleButton>(this));
}

const char* CDispenserToggleButton::getWidgetName(void)
{
	return "FXToggleButton";
}

const void* CDispenserToggleButton::getWidgetIcon(void)
{
	return Toggle;
}

const char* CDispenserToggleButton::getWidgetCategory(void)
{
	return "Extended";
}

const int   CDispenserToggleButton::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserToggleButton::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserToggleButton::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserToggleButton::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserToggleButton::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserToggleButton::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserToggleButton::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserToggleButton::getClassInfo(void)
{
	return &FXToggleButton::metaClass;
}

FXbool CDispenserToggleButton::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserToggleButton::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserToggleButton::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserToggleButton::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXToggleButton::metaClass);
}

IDispenser* CDispenserToggleButton::clone(void)
{
	return new CDispenserToggleButton;
}

FXWindow* CDispenserToggleButton::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXToggleButton(p, "", "", 0, 0, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserToggleButton::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Frame").getOptions(pWindow, strOptions);
	getProperty("Layout").getOptions(pWindow, strOptions);
	getProperty("Justify").getOptions(pWindow, strOptions);
	getProperty("Style").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXToggleButton(" << parent << ", 0, 0, 0, 0, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow);

	stream << ", " << getProperty("PadLeft").get(pWindow) << "," << getProperty("PadRight").get(pWindow) << ",";
	stream << getProperty("PadTop").get(pWindow) << "," << getProperty("PadBottom").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
