/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserTreeList.h"

#include <CPropertyBackColor.h>
#include <CPropertyTextColor.h>
#include <CPropertySelBackColor.h>
#include <CPropertySelTextColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyVisible.h>

#include <CVariant.h>

///////////////////////////////////////////////////////////////////////////////
//

template <class T, FXColor C>
class CPropertyLineColor : public CProperty
{
public:
	CPropertyLineColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXColor C>
const char* CPropertyLineColor<T, C>::name(void)
{
	return "LineColor";
}

template <class T, FXColor C>
ePropertyType CPropertyLineColor<T, C>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T, FXColor C>
const char* CPropertyLineColor<T, C>::description(void)
{
	return "The line color of this object";
}

template <class T, FXColor C>
const char* CPropertyLineColor<T, C>::tooltip(void)
{
	return "getLineColor()\nsetLineColor()";
}

template <class T, FXColor C>
void CPropertyLineColor<T, C>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setLineColor( getColorFromString(strValue) );
}

template <class T, FXColor C>
FXString CPropertyLineColor<T, C>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getLineColor() );
}

template <class T, FXColor C>
FXString CPropertyLineColor<T, C>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);

	if ( p->getLineColor() != C )
	{
		FXString str = getFXRGBString(p->getLineColor());
		return "\t" + widget + "->setLineColor(" + str + ");\n";
	}

	return "";
}

///////////////////////////////////////////////////////////////////////////////
//

const unsigned char TreeList[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x1e,0xd8,0xc8,0xb1,0xa3,0xc7,0x8f,0x20,0x43,0x8a,
  0x14,0xa9,0xf0,0x40,0xc6,0x93,0xf8,0x4c,0x26,0x34,0x09,0xa0,0xa5,0xcb,0x97,0x30,
  0x63,0xca,0x9c,0x49,0x13,0xe6,0x3f,0x95,0x08,0x59,0xfe,0xdb,0xc9,0xb3,0xa7,0xcf,
  0x9f,0x40,0x83,0x0a,0xe5,0x79,0xe0,0x66,0x49,0x7c,0x00,0x78,0x16,0x1c,0xca,0xb4,
  0x29,0xd0,0xa2,0x38,0x0f,0xea,0xfc,0x97,0x72,0xdf,0x01,0xab,0xf8,0x9c,0x6a,0x6d,
  0x0a,0xf5,0x68,0xd2,0xaa,0x57,0xc3,0x1a,0xdc,0x4a,0x76,0x67,0xd7,0x95,0x48,0xa9,
  0x36,0x4d,0xfb,0xd3,0x66,0x59,0xb3,0x46,0xd1,0x7e,0xfd,0x67,0xb5,0x6e,0xd8,0xba,
  0x6c,0x81,0x26,0x7d,0x4b,0x34,0x6e,0xce,0xb4,0x59,0xef,0x0a,0xc6,0xba,0x57,0x6f,
  0x4f,0x97,0x3b,0x5f,0x0a,0x3d,0xfb,0x77,0xae,0xdd,0xc7,0x57,0xf3,0xb6,0xe5,0x59,
  0x38,0x69,0xe5,0xa0,0x8c,0xa5,0x02,0xbe,0x09,0xd9,0xae,0x64,0x9f,0x97,0x13,0xff,
  0x0b,0xfd,0x33,0xb3,0x41,0x9d,0x59,0x3b,0xdf,0xfd,0x7c,0x98,0xb2,0x6b,0xd2,0x3e,
  0x4d,0x17,0x44,0xcd,0x79,0xb0,0xd8,0xc2,0x93,0x45,0x8b,0x86,0xdd,0x53,0x36,0x41,
  0xda,0xaa,0xf1,0xe2,0x06,0x7d,0xb8,0x65,0x62,0xc4,0x98,0xfd,0x6a,0xfe,0xda,0x70,
  0xb8,0x56,0xe7,0xa5,0x95,0x9f,0x66,0x5b,0x53,0xf1,0x5b,0xe8,0xb1,0xa5,0xcf,0x66,
  0xcd,0xb7,0x7b,0xdf,0xa8,0xd3,0x87,0xfe,0x67,0xf5,0xde,0xdd,0xf7,0xc0,0xa9,0x4c,
  0x65,0x92,0x8f,0x0e,0x7e,0xbb,0x78,0xc3,0xc5,0xd7,0xf7,0xd6,0xfe,0x9b,0xfb,0xe4,
  0x45,0xf8,0x47,0xeb,0x97,0x7f,0x93,0xfe,0xf9,0xcf,0xe3,0x11,0x57,0x9c,0x71,0x94,
  0x29,0x86,0x9d,0x6e,0xfd,0xb5,0x57,0xdf,0x81,0xad,0xe1,0x97,0x1f,0x7e,0x00,0x2c,
  0xe2,0xda,0x56,0x85,0x99,0x27,0x10,0x7a,0x3b,0x05,0xd8,0x9a,0x5e,0x12,0x22,0xb8,
  0xdb,0x7e,0x88,0x21,0x66,0x61,0x4a,0xf6,0x11,0xe7,0xa0,0x83,0xa3,0x75,0x38,0x9a,
  0x81,0x2b,0x7a,0x58,0xa1,0x7f,0x17,0x02,0xa8,0x56,0x56,0x59,0x61,0x17,0xe1,0x84,
  0xaf,0x15,0x88,0xe3,0x88,0x18,0x0a,0x95,0xd4,0x89,0xf9,0xdd,0xe8,0xa1,0x7e,0xc3,
  0x19,0xf7,0xa2,0x82,0xff,0xbd,0x97,0x5b,0x5b,0x2a,0x16,0xb9,0x5f,0x6b,0x47,0x7a,
  0xe5,0xd4,0x8f,0x40,0xa6,0x88,0xe3,0x87,0xc7,0x55,0x96,0x14,0x8f,0x32,0x2e,0x09,
  0x9a,0x8a,0xde,0x71,0xc9,0x60,0x81,0x33,0x81,0x59,0x1e,0x8c,0x24,0x2a,0xf9,0x13,
  0x90,0x55,0xae,0x27,0x26,0x7f,0x6e,0xa2,0xd9,0xa3,0x86,0x43,0x2d,0x45,0xde,0x9b,
  0x6f,0x81,0x85,0xd5,0x9d,0x72,0x02,0x86,0x50,0x50,0x7a,0x8a,0x65,0xa7,0x56,0x6f,
  0xd2,0x99,0x21,0xa0,0x6b,0x8d,0x39,0x1f,0x92,0x31,0xce,0xf5,0x93,0xa1,0x87,0x06,
  0x17,0x99,0xa2,0xdf,0x49,0x09,0x29,0xa4,0x6a,0xd9,0xe6,0x19,0xa5,0x70,0x31,0x9a,
  0xa6,0x5a,0x36,0xfd,0x99,0x9a,0xa6,0x93,0x6e,0x55,0x28,0x50,0x98,0x06,0x26,0x69,
  0x89,0xec,0x59,0x8a,0x2a,0xa2,0x55,0x92,0xee,0xc3,0x6a,0x76,0x9e,0xd2,0xf6,0x28,
  0xa2,0xa4,0x12,0x66,0x6a,0x9f,0xcc,0x1d,0x04,0x6b,0xae,0xb3,0x2e,0x2a,0x65,0x59,
  0x0e,0x71,0x9a,0xe0,0xb0,0x4a,0x4d,0x59,0x1d,0x81,0x4e,0xe1,0x09,0xe7,0x99,0xb5,
  0x06,0xfb,0xec,0x62,0xbc,0x4e,0x0b,0x2d,0xb2,0xd6,0x92,0x25,0xe6,0x48,0xdc,0x76,
  0xeb,0x2d,0x47,0x7d,0x66,0x1b,0x66,0x49,0xdf,0x96,0x6b,0x6e,0x48,0x28,0xa5,0xab,
  0xee,0xba,0xec,0xb6,0xeb,0xee,0xbb,0xf0,0xc6,0xab,0x50,0x40,0x00,0x3b
  };


CDispenserTreeList::CDispenserTreeList()
{
	m_Properties.append(new CPropertyLayout<FXTreeList>(this));
	m_Properties.append(new CPropertyBackColor<FXTreeList, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTextColor<FXTreeList, FXRGB(0, 0, 0)>(this));
	m_Properties.append(new CPropertySelBackColor<FXTreeList>(this));
	m_Properties.append(new CPropertySelTextColor<FXTreeList>(this));
	m_Properties.append(new CPropertyLineColor<FXTreeList, FXRGB(128, 128, 128)>(this));
	m_Properties.append(new CPropertyVisible<FXTreeList, 0>(this));
	m_Properties.append(new CPropertyLeft<FXTreeList>(this));
	m_Properties.append(new CPropertyTop<FXTreeList>(this));
	m_Properties.append(new CPropertyWidth<FXTreeList>(this));
	m_Properties.append(new CPropertyHeight<FXTreeList>(this));
}

const char* CDispenserTreeList::getWidgetName(void)
{
	return "FXTreeList";
}

const void* CDispenserTreeList::getWidgetIcon(void)
{
	return TreeList;
}

const char* CDispenserTreeList::getWidgetCategory(void)
{
	return "Extended";
}

const int CDispenserTreeList::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserTreeList::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserTreeList::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserTreeList::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserTreeList::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserTreeList::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserTreeList::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserTreeList::getClassInfo(void)
{
	return &FXTreeList::metaClass;
}

FXbool CDispenserTreeList::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserTreeList::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserTreeList::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserTreeList::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXTreeList::metaClass);
}

IDispenser* CDispenserTreeList::clone(void)
{
	return new CDispenserTreeList;
}

FXWindow* CDispenserTreeList::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXTreeList(p, 0, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserTreeList::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
	FXString strOptions;

	getProperty("Layout").getOptions(pWindow, strOptions);

	stream << "\t" << widget << " = new FXTreeList(" << parent << ", 0, 0, 0, ";

	stream << strOptions << ", " << getProperty("Left").get(pWindow) << "," << getProperty("Top").get(pWindow) << ",";
	stream << getProperty("Width").get(pWindow) << "," << getProperty("Height").get(pWindow) << ");\n";

	getPropertyCode(stream, pWindow, widget);
}
