/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CDispenser.h>
#include "CDispenserTreeListBox.h"

#include <CPropertyBackColor.h>
#include <CPropertyTop.h>
#include <CPropertyLeft.h>
#include <CPropertyFrame.h>
#include <CPropertyLayout.h>
#include <CPropertyWidth.h>
#include <CPropertyHeight.h>
#include <CPropertyTipText.h>

#include <CVariant.h>

const unsigned char TextBox[]={
  0x47,0x49,0x46,0x38,0x37,0x61,0x40,0x00,0x40,0x00,0xf7,0x00,0x00,0x00,0x00,0x00,
  0x80,0x00,0x00,0x00,0x80,0x00,0x80,0x80,0x00,0x00,0x00,0x80,0x80,0x00,0x80,0x00,
  0x80,0x80,0xc0,0xc0,0xc0,0xc0,0xdc,0xc0,0xa6,0xca,0xf0,0x00,0x00,0x00,0x00,0x00,
  0x2a,0x00,0x00,0x55,0x00,0x00,0x7f,0x00,0x00,0xaa,0x00,0x00,0xd4,0x00,0x2a,0x00,
  0x00,0x2a,0x2a,0x00,0x2a,0x55,0x00,0x2a,0x7f,0x00,0x2a,0xaa,0x00,0x2a,0xd4,0x00,
  0x55,0x00,0x00,0x55,0x2a,0x00,0x55,0x55,0x00,0x55,0x7f,0x00,0x55,0xaa,0x00,0x55,
  0xd4,0x00,0x7f,0x00,0x00,0x7f,0x2a,0x00,0x7f,0x55,0x00,0x7f,0x7f,0x00,0x7f,0xaa,
  0x00,0x7f,0xd4,0x00,0xaa,0x00,0x00,0xaa,0x2a,0x00,0xaa,0x55,0x00,0xaa,0x7f,0x00,
  0xaa,0xaa,0x00,0xaa,0xd4,0x00,0xd4,0x00,0x00,0xd4,0x2a,0x00,0xd4,0x55,0x00,0xd4,
  0x7f,0x00,0xd4,0xaa,0x00,0xd4,0xd4,0x2a,0x00,0x00,0x2a,0x00,0x2a,0x2a,0x00,0x55,
  0x2a,0x00,0x7f,0x2a,0x00,0xaa,0x2a,0x00,0xd4,0x2a,0x2a,0x00,0x2a,0x2a,0x2a,0x2a,
  0x2a,0x55,0x2a,0x2a,0x7f,0x2a,0x2a,0xaa,0x2a,0x2a,0xd4,0x2a,0x55,0x00,0x2a,0x55,
  0x2a,0x2a,0x55,0x55,0x2a,0x55,0x7f,0x2a,0x55,0xaa,0x2a,0x55,0xd4,0x2a,0x7f,0x00,
  0x2a,0x7f,0x2a,0x2a,0x7f,0x55,0x2a,0x7f,0x7f,0x2a,0x7f,0xaa,0x2a,0x7f,0xd4,0x2a,
  0xaa,0x00,0x2a,0xaa,0x2a,0x2a,0xaa,0x55,0x2a,0xaa,0x7f,0x2a,0xaa,0xaa,0x2a,0xaa,
  0xd4,0x2a,0xd4,0x00,0x2a,0xd4,0x2a,0x2a,0xd4,0x55,0x2a,0xd4,0x7f,0x2a,0xd4,0xaa,
  0x2a,0xd4,0xd4,0x55,0x00,0x00,0x55,0x00,0x2a,0x55,0x00,0x55,0x55,0x00,0x7f,0x55,
  0x00,0xaa,0x55,0x00,0xd4,0x55,0x2a,0x00,0x55,0x2a,0x2a,0x55,0x2a,0x55,0x55,0x2a,
  0x7f,0x55,0x2a,0xaa,0x55,0x2a,0xd4,0x55,0x55,0x00,0x55,0x55,0x2a,0x55,0x55,0x55,
  0x55,0x55,0x7f,0x55,0x55,0xaa,0x55,0x55,0xd4,0x55,0x7f,0x00,0x55,0x7f,0x2a,0x55,
  0x7f,0x55,0x55,0x7f,0x7f,0x55,0x7f,0xaa,0x55,0x7f,0xd4,0x55,0xaa,0x00,0x55,0xaa,
  0x2a,0x55,0xaa,0x55,0x55,0xaa,0x7f,0x55,0xaa,0xaa,0x55,0xaa,0xd4,0x55,0xd4,0x00,
  0x55,0xd4,0x2a,0x55,0xd4,0x55,0x55,0xd4,0x7f,0x55,0xd4,0xaa,0x55,0xd4,0xd4,0x7f,
  0x00,0x00,0x7f,0x00,0x2a,0x7f,0x00,0x55,0x7f,0x00,0x7f,0x7f,0x00,0xaa,0x7f,0x00,
  0xd4,0x7f,0x2a,0x00,0x7f,0x2a,0x2a,0x7f,0x2a,0x55,0x7f,0x2a,0x7f,0x7f,0x2a,0xaa,
  0x7f,0x2a,0xd4,0x7f,0x55,0x00,0x7f,0x55,0x2a,0x7f,0x55,0x55,0x7f,0x55,0x7f,0x7f,
  0x55,0xaa,0x7f,0x55,0xd4,0x7f,0x7f,0x00,0x7f,0x7f,0x2a,0x7f,0x7f,0x55,0x7f,0x7f,
  0x7f,0x7f,0x7f,0xaa,0x7f,0x7f,0xd4,0x7f,0xaa,0x00,0x7f,0xaa,0x2a,0x7f,0xaa,0x55,
  0x7f,0xaa,0x7f,0x7f,0xaa,0xaa,0x7f,0xaa,0xd4,0x7f,0xd4,0x00,0x7f,0xd4,0x2a,0x7f,
  0xd4,0x55,0x7f,0xd4,0x7f,0x7f,0xd4,0xaa,0x7f,0xd4,0xd4,0xaa,0x00,0x00,0xaa,0x00,
  0x2a,0xaa,0x00,0x55,0xaa,0x00,0x7f,0xaa,0x00,0xaa,0xaa,0x00,0xd4,0xaa,0x2a,0x00,
  0xaa,0x2a,0x2a,0xaa,0x2a,0x55,0xaa,0x2a,0x7f,0xaa,0x2a,0xaa,0xaa,0x2a,0xd4,0xaa,
  0x55,0x00,0xaa,0x55,0x2a,0xaa,0x55,0x55,0xaa,0x55,0x7f,0xaa,0x55,0xaa,0xaa,0x55,
  0xd4,0xaa,0x7f,0x00,0xaa,0x7f,0x2a,0xaa,0x7f,0x55,0xaa,0x7f,0x7f,0xaa,0x7f,0xaa,
  0xaa,0x7f,0xd4,0xaa,0xaa,0x00,0xaa,0xaa,0x2a,0xaa,0xaa,0x55,0xaa,0xaa,0x7f,0xaa,
  0xaa,0xaa,0xaa,0xaa,0xd4,0xaa,0xd4,0x00,0xaa,0xd4,0x2a,0xaa,0xd4,0x55,0xaa,0xd4,
  0x7f,0xaa,0xd4,0xaa,0xaa,0xd4,0xd4,0xd4,0x00,0x00,0xd4,0x00,0x2a,0xd4,0x00,0x55,
  0xd4,0x00,0x7f,0xd4,0x00,0xaa,0xd4,0x00,0xd4,0xd4,0x2a,0x00,0xd4,0x2a,0x2a,0xd4,
  0x2a,0x55,0xd4,0x2a,0x7f,0xd4,0x2a,0xaa,0xd4,0x2a,0xd4,0xd4,0x55,0x00,0xd4,0x55,
  0x2a,0xd4,0x55,0x55,0xd4,0x55,0x7f,0xd4,0x55,0xaa,0xd4,0x55,0xd4,0xd4,0x7f,0x00,
  0xd4,0x7f,0x2a,0xd4,0x7f,0x55,0xd4,0x7f,0x7f,0xd4,0x7f,0xaa,0xd4,0x7f,0xd4,0xd4,
  0xaa,0x00,0xd4,0xaa,0x2a,0xd4,0xaa,0x55,0xd4,0xaa,0x7f,0xd4,0xaa,0xaa,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0xd4,0xd4,0x2a,0xd4,0xd4,0x55,0xd4,0xd4,0x7f,0xd4,0xd4,0xaa,
  0xd4,0xd4,0xd4,0x00,0x00,0x00,0x0c,0x0c,0x0c,0x19,0x19,0x19,0x26,0x26,0x26,0x33,
  0x33,0x33,0x3f,0x3f,0x3f,0x4c,0x4c,0x4c,0x59,0x59,0x59,0x66,0x66,0x66,0x72,0x72,
  0x72,0x7f,0x7f,0x7f,0x8c,0x8c,0x8c,0x99,0x99,0x99,0xa5,0xa5,0xa5,0xb2,0xb2,0xb2,
  0xbf,0xbf,0xbf,0xcc,0xcc,0xcc,0xd8,0xd8,0xd8,0xe5,0xe5,0xe5,0xf2,0xf2,0xf2,0xff,
  0xfb,0xf0,0xa0,0xa0,0xa4,0x80,0x80,0x80,0xff,0x00,0x00,0x00,0xff,0x00,0xff,0xff,
  0x00,0x00,0x00,0xff,0xff,0x00,0xff,0x00,0xff,0xff,0xff,0xff,0xff,0x2c,0x00,0x00,
  0x00,0x00,0x40,0x00,0x40,0x00,0x00,0x08,0xfe,0x00,0xf1,0x09,0x1c,0x48,0xb0,0xa0,
  0xc1,0x83,0x08,0x13,0x2a,0x5c,0xc8,0xb0,0xa1,0xc3,0x87,0x10,0x23,0x4a,0x9c,0x48,
  0xb1,0xa2,0xc5,0x8b,0x18,0x33,0x6a,0xdc,0xc8,0xb1,0xa3,0xc4,0x03,0x20,0x43,0x8a,
  0x1c,0x49,0xb2,0xa4,0xc9,0x93,0x27,0x15,0x1e,0xf0,0x68,0x71,0x65,0x42,0x97,0x2c,
  0x23,0xfe,0xc3,0x07,0xf3,0xa0,0x4b,0x00,0x38,0x73,0xea,0xdc,0xc9,0xb3,0xa7,0xcf,
  0x9f,0x38,0x0f,0xcc,0xac,0x69,0xf0,0xe6,0xbf,0xa3,0x48,0x93,0x2a,0x5d,0xca,0xb4,
  0xa9,0xd3,0x7f,0x42,0x69,0xaa,0x14,0x08,0xe0,0xa9,0xd5,0xab,0x58,0xa3,0x12,0x2d,
  0x68,0x14,0xab,0xd7,0xaf,0x48,0xb5,0x4e,0xc5,0x57,0x15,0xac,0xd9,0xab,0x62,0x5f,
  0x52,0x3d,0xcb,0xd6,0x69,0x5a,0x84,0x5d,0xdb,0xca,0x0d,0x3b,0x74,0x6c,0xd9,0xb9,
  0x73,0xdf,0xda,0x5c,0x8b,0x37,0x6f,0x5d,0xb5,0x64,0xfb,0xfa,0x95,0x0a,0xf8,0x6e,
  0x52,0xc3,0x82,0xdd,0xfe,0x85,0xbb,0x76,0xe7,0x3f,0xc4,0x88,0x0f,0x0f,0xde,0x4a,
  0x30,0xee,0xdd,0xc8,0x89,0x97,0xea,0x2d,0xca,0xf7,0x31,0x52,0x9d,0x47,0xcb,0xe6,
  0x94,0xfc,0x18,0xa7,0xd9,0xcd,0x5c,0x3b,0x5f,0xfe,0xec,0x19,0x73,0x55,0xcc,0x59,
  0x17,0xef,0x0d,0x1c,0x9a,0x75,0xed,0xd7,0x91,0x71,0xb3,0x45,0x5d,0x59,0xb5,0xed,
  0xd6,0x9f,0x0d,0x5f,0x86,0xad,0x98,0x30,0x63,0xda,0x9e,0x6b,0xdf,0x26,0xad,0xbc,
  0xb9,0x57,0xde,0x03,0x2d,0xff,0x7e,0x5d,0x1a,0x72,0xf5,0xb3,0xd0,0x05,0xc6,0xcd,
  0x0c,0x36,0x3b,0x61,0xe2,0xdc,0x8b,0xd3,0x53,0x8e,0xde,0x39,0xfc,0x73,0xd9,0x9c,
  0x91,0x9b,0x8f,0x6d,0x7c,0x36,0xf8,0xf5,0xac,0xbd,0x6f,0x6f,0x4b,0x1d,0x6c,0x55,
  0xf9,0xe5,0xe9,0x27,0xb7,0xba,0x1a,0xbf,0x7a,0xfd,0xef,0x35,0x77,0x1f,0x7a,0xa9,
  0xd1,0x36,0xda,0x7e,0xa2,0x99,0x76,0x1d,0x70,0x0b,0x82,0x76,0xdd,0x80,0xed,0xa5,
  0x57,0x5f,0x72,0xf5,0x09,0xc7,0x1a,0x75,0xab,0x65,0x18,0x1f,0x81,0xbd,0x05,0xe6,
  0x98,0x80,0x87,0x81,0x86,0xa1,0x88,0x17,0x6e,0x18,0x61,0x81,0x13,0x32,0x67,0xa1,
  0x64,0x29,0x22,0xe8,0xa2,0x7f,0x13,0x8a,0x46,0xa1,0x8a,0xc0,0x69,0xb8,0x5c,0x6d,
  0x30,0x86,0xa6,0xe0,0x6a,0x0d,0xee,0x48,0xa1,0x8f,0xca,0x99,0x06,0xe1,0x78,0xda,
  0xe5,0x07,0x1f,0x53,0x43,0xda,0x75,0x24,0x7f,0x50,0x71,0x48,0xde,0x7f,0x4b,0x32,
  0x95,0x63,0x94,0x4f,0x4d,0x49,0x65,0x53,0x56,0x5e,0xa9,0x99,0x93,0x45,0x42,0xa9,
  0xe5,0x51,0x59,0x7e,0x49,0xd7,0x89,0x1d,0x06,0xb8,0x64,0x98,0x62,0x36,0x49,0xe6,
  0x93,0x66,0x1e,0xe9,0x1f,0x4a,0x70,0xc6,0x29,0x27,0x48,0x5c,0x4a,0x95,0xa6,0x53,
  0x6b,0x76,0x19,0xd3,0x47,0x2a,0xcd,0xe9,0xe7,0x9f,0x26,0xed,0x29,0xe8,0xa0,0x84,
  0x16,0x6a,0xe8,0xa1,0x88,0x26,0xaa,0xa8,0x43,0x01,0x01,0x00,0x3b
  };


CDispenserTreeListBox::CDispenserTreeListBox()
{
	m_Properties.append(new CPropertyLayout<FXTreeListBox>(this));
	m_Properties.append(new CPropertyFrame<FXTreeListBox>(this));
	m_Properties.append(new CPropertyBackColor<FXTreeListBox, FXRGB(255, 255, 255)>(this));
	m_Properties.append(new CPropertyTipText<FXTreeListBox>(this));
	m_Properties.append(new CPropertyLeft<FXTreeListBox>(this));
	m_Properties.append(new CPropertyTop<FXTreeListBox>(this));
	m_Properties.append(new CPropertyWidth<FXTreeListBox>(this));
	m_Properties.append(new CPropertyHeight<FXTreeListBox>(this));
}

const char* CDispenserTreeListBox::getWidgetName(void)
{
	return "FXTreeListBox";
}

const void* CDispenserTreeListBox::getWidgetIcon(void)
{
	return TextBox;
}

const char* CDispenserTreeListBox::getWidgetCategory(void)
{
	return "Standard";
}

const int   CDispenserTreeListBox::getWidgetGrabbers(void)
{
	return GS_ALL;
}

FXint CDispenserTreeListBox::getDefaultWidth(void)
{
	return 80;
}

FXint CDispenserTreeListBox::getDefaultHeight(void)
{
	return 40;
}

FXint CDispenserTreeListBox::getMinimumWidth(void)
{
	return 16;
}

FXint CDispenserTreeListBox::getMinimumHeight(void)
{
	return 16;
}

FXint CDispenserTreeListBox::getMaximumWidth(void)
{
	return 0xFFFF;
}

FXint CDispenserTreeListBox::getMaximumHeight(void)
{
	return 0xFFFF;
}

const FXMetaClass* CDispenserTreeListBox::getClassInfo(void)
{
	return &FXTreeListBox::metaClass;
}

FXbool CDispenserTreeListBox::allowChild(const FXMetaClass*)
{
	return FALSE;
}

FXbool CDispenserTreeListBox::allowParent(FXWindow*)
{
	return TRUE;
}

FXWindow* CDispenserTreeListBox::getCreateParent(FXWindow* pParent)
{
	return pParent;
}

FXbool CDispenserTreeListBox::IsValidWidget(FXWindow** ppWindow)
{
	return (*ppWindow)->isMemberOf(&FXTreeListBox::metaClass);
}

IDispenser* CDispenserTreeListBox::clone(void)
{
	return new CDispenserTreeListBox;
}

FXWindow* CDispenserTreeListBox::createWidget(FXComposite* p, FXObject* tgt, FXSelector sel)
{
	return new FXTreeListBox(p, 10, tgt, sel,
				FRAME_SUNKEN|FRAME_THICK|LAYOUT_FILL_X|LAYOUT_FIX_X|LAYOUT_FIX_Y|LAYOUT_FIX_WIDTH|LAYOUT_FIX_HEIGHT);
}

void CDispenserTreeListBox::getImplementationCode(ITextStream& stream, FXWindow* pWindow, FXString& widget, FXString& parent)
{
}
