/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyColumns_H__
#define __CPropertyColumns_H__

template <class T, FXint I>
class CPropertyColumns : public CProperty
{
public:
	CPropertyColumns(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T, FXint I>
const char* CPropertyColumns<T, I>::name(void)
{
	return "Columns";
}

template <class T, FXint I>
ePropertyType CPropertyColumns<T, I>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T, FXint I>
const char* CPropertyColumns<T, I>::description(void)
{
	return "Sets or returns the columns\n of this object";
}

template <class T, FXint I>
const char* CPropertyColumns<T, I>::tooltip(void)
{
	return "getNumColumns()\nsetNumColumns()";
}

template <class T, FXint I>
void CPropertyColumns<T, I>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = (T*)pWindow;
	p->setNumColumns( getIntFromString(strValue) );
}

template <class T, FXint I>
FXString CPropertyColumns<T, I>::get(FXWindow* pWindow)
{
	T* p = (T*)pWindow;
	return getStringFromInt( p->getNumColumns() );
}

template <class T, FXint I>
FXString CPropertyColumns<T, I>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	
	if ( p && p->getNumColumns() != I )
	{
		return "\t" + widget + "->setNumColumns(" + get(pWindow) + ");\n";
	}

	return "";
}

#endif // __CPropertyColumns_H__
