/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyDirectory_H__
#define __CPropertyDirectory_H__

template <class T>
class CPropertyDirectory : public CProperty
{
public:
	CPropertyDirectory(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);
	virtual FXint style(void) {return PROPERTY_MULTIPLE;}

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWidget, FXString& widget);
};

template <class T>
const char* CPropertyDirectory<T>::name(void)
{
	return "Directory";
}

template <class T>
ePropertyType CPropertyDirectory<T>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T>
const char* CPropertyDirectory<T>::description(void)
{
	return "The directory associated with this\nobject";
}

template <class T>
const char* CPropertyDirectory<T>::tooltip(void)
{
	return "getDirectory()\nsetDirectory()";
}

template <class T>
void CPropertyDirectory<T>::set(FXWindow* pWindow, const FXString strValue)
{
	T* p = (T*)pWindow;
	p->setDirectory(strValue);
}

template <class T>
FXString CPropertyDirectory<T>::get(FXWindow* pWindow)
{
	T* p = (T*)pWindow;
	return p->getDirectory();
}

template <class T>
FXString CPropertyDirectory<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	FXString str = get(pWindow);

	if ( str == "" )
		return "";

	FXString s;
	for ( FXint n = 0; n < str.length(); n++ )
	{
		if ( str[n] == '\\' )
			s += str[n];

		s += str[n];
	}

	return "\t" + widget + "->setDirectory(\"" + s + "\");\n";
}

#endif // __CPropertyDirectory_H__
