/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyJustify_H__
#define __CPropertyJustify_H__

// From FXLabel.h
extern OPTION JustifyStyle[];

template <class T>
class CPropertyJustify : public CProperty
{
public:
	CPropertyJustify(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
};

template <class T>
OPTION CPropertyJustify<T>::Style[] = 
{
	{"JUSTIFY_LEFT"    , JUSTIFY_LEFT      ,""},
	{"JUSTIFY_RIGHT"   , JUSTIFY_RIGHT     ,""},
	{"JUSTIFY_HZ_APART", JUSTIFY_HZ_APART  ,"Same as JUSTIFY_LEFT | JUSTIFY_RIGHT"},
	{"JUSTIFY_TOP"     , JUSTIFY_TOP       ,""},
	{"JUSTIFY_BOTTOM"  , JUSTIFY_BOTTOM    ,""},
	{"JUSTIFY_VT_APART", JUSTIFY_VT_APART  ,"Same as JUSTIFY_TOP | JUSTIFY_BOTTOM"},
	{0, 0, 0}
};

template <class T>
const char* CPropertyJustify<T>::name(void)
{
	return "Justify";
}

template <class T>
ePropertyType CPropertyJustify<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyJustify<T>::description(void)
{
	return "The position of this object's\ncontents relative to the boundary\nof this object";
}

template <class T>
const char* CPropertyJustify<T>::tooltip(void)
{
	return "getJustify()\nsetJustify()";
}

template <class T>
void CPropertyJustify<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setJustify(getIntFromString(strValue));
}

template <class T>
FXString CPropertyJustify<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromInt(p->getJustify());
}

template <class T>
OPTION* CPropertyJustify<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyJustify<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getJustify();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

#endif // __CPropertyJustify_H__
