/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyLayout_H__
#define __CPropertyLayout_H__

template <class T>
class CPropertyLayout : public CProperty
{
public:
	CPropertyLayout(IDispenser* pDispenser) : CProperty(pDispenser) {};

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);
	virtual void validate(IConnector* pConnector, FXString& strNewValue);
	virtual void validate(IConnector* pConnector);

	OPTION* option(void);
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions);

private:
	static OPTION Style[];
	IDispenser* m_pDispenser;
};

// From FXWindow.h
template <class T>
OPTION CPropertyLayout<T>::Style[] = 
{
	{"LAYOUT_SIDE_BOTTOM", LAYOUT_SIDE_BOTTOM, ""},
	{"LAYOUT_SIDE_LEFT"  , LAYOUT_SIDE_LEFT  , ""},
	{"LAYOUT_SIDE_RIGHT" , LAYOUT_SIDE_RIGHT , ""},
	{"LAYOUT_RIGHT"      , LAYOUT_RIGHT      , ""},
	{"LAYOUT_CENTER_X"   , LAYOUT_CENTER_X   , ""},
	{"LAYOUT_BOTTOM"     , LAYOUT_BOTTOM     , ""},
	{"LAYOUT_CENTER_Y"   , LAYOUT_CENTER_Y   , ""},
	{"LAYOUT_FIX_X"      , LAYOUT_FIX_X      , ""},
	{"LAYOUT_FIX_Y"      , LAYOUT_FIX_Y      , ""},
	{"LAYOUT_FIX_WIDTH"  , LAYOUT_FIX_WIDTH  , ""},
	{"LAYOUT_FIX_HEIGHT" , LAYOUT_FIX_HEIGHT , ""},
	{"LAYOUT_FILL_X"     , LAYOUT_FILL_X     , ""},
	{"LAYOUT_FILL_Y"     , LAYOUT_FILL_Y     , ""},
	{0, 0, 0}
};

template <class T>
const char* CPropertyLayout<T>::name(void)
{
	return "Layout";
}

template <class T>
ePropertyType CPropertyLayout<T>::type(void)
{
	return PROPERTY_COMBO;
}

template <class T>
const char* CPropertyLayout<T>::description(void)
{
	return "The position of this object relative\nto the parent or container";
}

template <class T>
const char* CPropertyLayout<T>::tooltip(void)
{
	return "getLayoutHints()\nsetLayoutHints()";
}

template <class T>
void CPropertyLayout<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setLayoutHints( getIntFromString(strValue) );
}

template <class T>
FXString CPropertyLayout<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	FXint nValue = p->getLayoutHints();
	return getStringFromInt(nValue);
}

template <class T>
void CPropertyLayout<T>::validate(IConnector* pConnector, FXString& strNewValue)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nOldValue = p->getLayoutHints();
	FXint nNewValue = getIntFromString(strNewValue);

	toggleAttribute(nOldValue, nNewValue, LAYOUT_FILL_X, LAYOUT_FIX_X);
	toggleAttribute(nOldValue, nNewValue, LAYOUT_FILL_Y, LAYOUT_FIX_Y);

	strNewValue = getStringFromInt(nNewValue);
}

template <class T>
void CPropertyLayout<T>::validate(IConnector* pConnector)
{
	T* p = static_cast<T*>(pConnector->getWidget());
	FXint nValue = p->getLayoutHints();

	IProperty* pProperty = 0;
	if ( getProperty("Top", &pProperty) )
	{
		if ( nValue & LAYOUT_FILL_Y )
		{
			pProperty->disable();
			pConnector->refreshProperty();
		}
		if ( nValue & LAYOUT_FIX_Y )
		{
			pProperty->enable();
			pConnector->refreshProperty();
		}
	}

	pProperty = 0;
	if ( getProperty("Left", &pProperty) )
	{
		if ( nValue & LAYOUT_FILL_X )
		{
			pProperty->disable();
			pConnector->refreshProperty();
		}
		if ( nValue & LAYOUT_FIX_X )
		{
			pProperty->enable();
			pConnector->refreshProperty();
		}
	}

	pProperty = 0;
	if ( getProperty("Width", &pProperty) )
	{
		pProperty->disable();
		if ( (nValue & LAYOUT_FIX_X) && (nValue & LAYOUT_FIX_WIDTH) )
		{
			pProperty->enable();
		}
		pConnector->refreshProperty();
	}

	pProperty = 0;
	if ( getProperty("Height", &pProperty) )
	{
		pProperty->disable();
		if ( (nValue & LAYOUT_FIX_Y) && (nValue & LAYOUT_FIX_HEIGHT) )
		{
			pProperty->enable();
		}
		pConnector->refreshProperty();
	}
}

template <class T>
OPTION* CPropertyLayout<T>::option(void)
{
	return Style;
}

template <class T>
void CPropertyLayout<T>::getOptions(FXWindow* pWindow, FXString& strOptions)
{
	T* p = static_cast<T*>(pWindow);
	FXuint option = p->getLayoutHints();

	for (OPTION* pOption = Style; pOption->pszName; pOption++)
	{
		if ( (option & pOption->value) == pOption->value )
		{
			if ( strOptions.length() > 0 )
			{
				strOptions += "|";
			}
			strOptions += pOption->pszName;
		}
	}
}

#endif // __CPropertyLayout_H__
