/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertySelTextColor_H__
#define __CPropertySelTextColor_H__

template <class T>
class CPropertySelTextColor : public CProperty
{
public:
	CPropertySelTextColor(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertySelTextColor<T>::name(void)
{
	return "SelTextColor";
}

template <class T>
ePropertyType CPropertySelTextColor<T>::type(void)
{
	return PROPERTY_COLOR;
}

template <class T>
const char* CPropertySelTextColor<T>::description(void)
{
	return "The color of selected text in this\nobject";
}

template <class T>
const char* CPropertySelTextColor<T>::tooltip(void)
{
	return "getSelTextColor()\nsetSelTextColor()";
}

template <class T>
void CPropertySelTextColor<T>::set(FXWindow* pWindow, FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setSelTextColor( getColorFromString(strValue) );
}

template <class T>
FXString CPropertySelTextColor<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return getStringFromColor( p->getSelTextColor() );
}

template <class T>
FXString CPropertySelTextColor<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	T* p = static_cast<T*>(pWindow);
	FXString str = getFXRGBString(p->getSelTextColor());

	if ( str != "FXRGB(255, 255, 255)" )
	{
		return "\t" + widget + "->setSelTextColor(" + str + ");\n";
	}

	return "";
}

#endif // __CPropertySelTextColor_H__
