/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CPropertyText_H__
#define __CPropertyText_H__

template <class T>
class CPropertyText : public CProperty
{
public:
	CPropertyText(IDispenser* pDispenser) : CProperty(pDispenser) {}

	const char* name(void);
	virtual ePropertyType type(void);
	virtual const char* description(void);
	virtual const char* tooltip(void);
	virtual FXint style(void) {return PROPERTY_UPDATEEVERYKEY;}

	void set(FXWindow* pWindow, FXString strValue);
	FXString get(FXWindow* pWindow);

	virtual FXString getImplementationCode(FXWindow* pWindow, FXString& widget);
};

template <class T>
const char* CPropertyText<T>::name(void)
{
	return "Text";
}

template <class T>
ePropertyType CPropertyText<T>::type(void)
{
	return PROPERTY_TEXT;
}

template <class T>
const char* CPropertyText<T>::description(void)
{
	return "The text contained within this\nobject";
}

template <class T>
const char* CPropertyText<T>::tooltip(void)
{
	return "getText()\nsetText()";
}

template <class T>
void CPropertyText<T>::set(FXWindow* pWindow, const FXString strValue)
{
	T* p = static_cast<T*>(pWindow);
	p->setText(strValue);
}

template <class T>
FXString CPropertyText<T>::get(FXWindow* pWindow)
{
	T* p = static_cast<T*>(pWindow);
	return p->getText();
}

template <class T>
FXString CPropertyText<T>::getImplementationCode(FXWindow* pWindow, FXString& widget)
{
	FXString str = get(pWindow);

	if ( str != "" )
	{
		return "\t" + widget + "->setText(\"" + get(pWindow) + "\");\n";
	}

	return "";
}

#endif // __CPropertyText_H__
