/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __IProperty_H__
#define __IProperty_H__

struct OPTION
{
	char*  pszName;
	FXuint value;
	char*  pszTip;
};

enum ePropertyType
{
	PROPERTY_NONE,
	PROPERTY_COMBO,
	PROPERTY_TEXT,
	PROPERTY_COLOR,
	PROPERTY_BOOL,
	PROPERTY_LIST
};

const FXint PROPERTY_MULTIPLE       = 1;
const FXint PROPERTY_UPDATEEVERYKEY = 2;

struct IConnector;

struct IProperty
{
	virtual void enable(void) = 0;
	virtual void disable(void) = 0;
	virtual FXbool isEnabled(void) = 0;

	virtual const char* name(void) = 0;
	virtual ePropertyType type(void) = 0;
	virtual const char* description(void) = 0;
	virtual FXint style(void) = 0;
	virtual const char* tooltip(void) {return "";}

	virtual void set(FXWindow* pWindow, FXString strValue) = 0;
	virtual FXString get(FXWindow* pWindow) = 0;

	// This version of validate is used to update options within this property
	// -- See CPropertyLayout for an example.
	virtual void validate(IConnector* pConnector, FXString& strNewValue) = 0;
	
	// This version of validate is used to update other properties
	// -- See CPropertyLayout for an example.
	virtual void validate(IConnector* pConnector) = 0;

	virtual OPTION* option(void) = 0;
	virtual void getOptions(FXWindow* pWindow, FXString& strOptions) = 0;

	virtual FXString getImplementationCode(FXWindow* pWidget, FXString& widget) = 0;
};


#endif // __IProperty_H__