/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <CComboBox.h>
#include "CBoolComboBox.h"

///////////////////////////////////////////////////////////////////////////////
//
/*
 *  CBoolListItem overrides FXListItem to update the connector
 *  when the user clicks on an item
 */

FXIMPLEMENT(CBoolListItem,FXListItem,NULL,0)

CBoolListItem::CBoolListItem(CConnector* pConnector, FXString strText)
: FXListItem(strText)
{
	m_pConnector = pConnector;
}

FXint CBoolListItem::hitItem(const FXList* list,FXint x,FXint y) const 
{
	FXint nReturn = FXListItem::hitItem(list, x, y);

	m_pConnector->set(label);

	return nReturn;
}

///////////////////////////////////////////////////////////////////////////////
//
/*
 *  CBoolList overrides FXList to close the CBoolComboBox when
 *  the user releases the mouse.
 */

FXDEFMAP(CBoolList) CBoolListMap[]=
{
	FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,CBoolList::onLeftBtnRelease)
};

FXIMPLEMENT(CBoolList, FXList, CBoolListMap, ARRAYNUMBER(CBoolListMap)) 

CBoolList::CBoolList(CConnector* pConnector, FXComposite *p,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h)
: FXList(p,nvis,tgt,sel,opts,x,y,w,h)
{
	m_pConnector = pConnector;
}

long CBoolList::onLeftBtnRelease(FXObject* o,FXSelector s,void* ptr)
{
	long l = FXList::onLeftBtnRelease(o, s, ptr);

	FXEvent* event = (FXEvent*)ptr;
	FXint index = getItemAt(event->win_x,event->win_y);
	FXString str = getItemText(index);
	m_pConnector->set(str);
	m_pConnector->close();

	return l;
}

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CBoolComboBox) CBoolComboBoxMap[]=
{
	FXMAPFUNC(SEL_UPDATE,0,CBoolComboBox::onUpdate)
};

FXIMPLEMENT(CBoolComboBox,CComboBox,CBoolComboBoxMap,ARRAYNUMBER(CBoolComboBoxMap))

CBoolComboBox::CBoolComboBox(CConnector* pConnector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
   CComboBox(p,cols,2/*nvis*/,tgt,sel,opts,x,y,w,h, pl,pr,pt,pb)
{
	// Note the 2,0,2,0 padding -- this is necessary to align the 
	// "(Select)" with the parent CPropertyItem "(Select)".
	text=new FXTextField(this,cols,this,0,0, 0,0,0,0, 2,0,2,0/*pl,pr,pt,pb*/);
	text->setText( pConnector->pProperty->get(pConnector->pWidget) );
	text->setEditable(FALSE);

	//  Note that LIST_BROWSESELECT|LIST_AUTOSELECT have not been defined,
	//  allowing CBoolListItem::hitItem to be called
	list=new CBoolList(pConnector, pane,nvis,NULL,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|SCROLLERS_TRACK|HSCROLLER_NEVER);
	list->appendItem( new CBoolListItem(pConnector, "True") );
	list->appendItem( new CBoolListItem(pConnector, "False") );

	if ( pConnector->get() == "True" )
		list->selectItem(0);
	else
		list->selectItem(1);

	m_bInitialised = FALSE;
}

CBoolComboBox::~CBoolComboBox()
{
	text=(FXTextField*)-1;
	list=(FXList*)-1;
}

// Recalculate layout
void CBoolComboBox::layout()
{
	FXint buttonWidth,textWidth,itemHeight;
	itemHeight=height-(border<<1);
	buttonWidth=button->getDefaultWidth();
	textWidth=width-buttonWidth-(border<<1);
	text->position(border,border,textWidth,itemHeight);
	button->position(border+textWidth,border,buttonWidth,itemHeight);

	pane->setX(pane->getX()-width);

	/*
	 *  Note that we are setting the size of the drop down part
	 *  of the combo box here. The first parameter, width, has
	 *  already been calculated as half the width of the combo 
	 *  box which is just what we want for the CBoolComboBox.
	 */
	pane->resize(width,itemHeight*2+text->getBorderWidth()*2+text->getPadTop()+
		text->getPadBottom()+4);
  
	flags&=~FLAG_DIRTY;
}

long CBoolComboBox::onUpdate(FXObject* o,FXSelector s,void* ptr)
{
/*
 *  Here's a way that OnInitialUpdate could be implemented
 *  This also shows how to invoke message handlers in 
 *  another class.
 *
 *	if ( !m_bInitialised )
 *	{
 *		button->setState(STATE_UP);
 *		button->handle(this,MKUINT(ID_POST,SEL_COMMAND),NULL);
 *
 *		pane->setFocus();
 *
 *		m_bInitialised = TRUE;
 *	}
 */
	return CComboBox::onUpdate(o, s, ptr);
}

