/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <CVariant.h>

#include <CComboBox.h>
#include <CColorComboBox.h>

struct COLOR
{
	FXColor  Color;
	char*    Name;
};

COLOR InternetColors[] = 
{
	{FXRGB(0xF0, 0xF8, 0xFF), "AliceBlue"},
	{FXRGB(0xFA, 0xEB, 0xD7), "AntiqueWhite"},
	{FXRGB(0x00, 0xFF, 0xFF), "Aqua"},
	{FXRGB(0x7F, 0xFF, 0xD4), "AquaMarine"},
	{FXRGB(0xF0, 0xFF, 0xFF), "Azure"},
	{FXRGB(0xF5, 0xF5, 0xDC), "Beige"},
	{FXRGB(0xFF, 0xE4, 0xC4), "Bisque"},
	{FXRGB(0x00, 0x00, 0x00), "Black"},
	{FXRGB(0xFF, 0xEB, 0xCD), "BlanchedAlmond"},
	{FXRGB(0x00, 0x00, 0xFF), "Blue"},
	{FXRGB(0x8A, 0x2B, 0xE2), "BlueViolet"},
	{FXRGB(0xA5, 0x2A, 0x2A), "Brown"},
	{FXRGB(0xDE, 0xB8, 0x87), "Burlywood"},
	{FXRGB(0x5F, 0x9E, 0xA0), "CadetBlue"},
	{FXRGB(0x7F, 0xFF, 0x00), "Chartreuse"},
	{FXRGB(0xD2, 0x69, 0x1E), "Chocolate"},
	{FXRGB(0xFF, 0x7F, 0x50), "Coral"},
	{FXRGB(0x64, 0x95, 0xED), "Cornflower"},
	{FXRGB(0xFF, 0xF8, 0xDC), "Cornsilk"},
	{FXRGB(0xDC, 0x14, 0x3C), "Crimson"},
	{FXRGB(0x00, 0xFF, 0xFF), "Cyan"},
	{FXRGB(0x00, 0x00, 0x8B), "DarkBlue"},
	{FXRGB(0x00, 0x8B, 0x8B), "DarkCyan"},
	{FXRGB(0xB8, 0x86, 0x0B), "DarkGoldenrod"},
	{FXRGB(0xA9, 0xA9, 0xA9), "DarkGray"},
	{FXRGB(0x00, 0x64, 0x00), "DarkGreen"},
	{FXRGB(0xBD, 0xB7, 0x6B), "DarkKhaki"},
	{FXRGB(0x8B, 0x00, 0x8B), "DarkMagenta"},
	{FXRGB(0x55, 0x6B, 0x2F), "DarkOliveGreen"},
	{FXRGB(0xFF, 0x8C, 0x00), "DarkOrange"},
	{FXRGB(0x99, 0x32, 0xCC), "DarkOrchid"},
	{FXRGB(0x8B, 0x00, 0x00), "DarkRed"},
	{FXRGB(0xE9, 0x96, 0x7A), "DarkSalmon"},
	{FXRGB(0x8F, 0xBC, 0x8B), "DarkSeaGreen"},
	{FXRGB(0x48, 0x3D, 0x8B), "DarkSlateBlue"},
	{FXRGB(0x2F, 0x4F, 0x4F), "DarkSlateGray"},
	{FXRGB(0x00, 0xCE, 0xD1), "DarkTurquoise"},
	{FXRGB(0x94, 0x00, 0xD3), "DarkViolet"},
	{FXRGB(0xFF, 0x14, 0x93), "DeepPink"},
	{FXRGB(0x00, 0xBF, 0xFF), "DeepSkyBlue"},
	{FXRGB(0x69, 0x69, 0x69), "DimGray"},
	{FXRGB(0x1E, 0x90, 0xFF), "DodgerBlue"},
	{FXRGB(0xB2, 0x22, 0x22), "Firebrick"},
	{FXRGB(0xFF, 0xFA, 0xF0), "FloralWhite"},
	{FXRGB(0x22, 0x8B, 0x22), "ForestGreen"},
	{FXRGB(0xFF, 0x00, 0xFF), "Fuchsia"},
	{FXRGB(0xDC, 0xDC, 0xDC), "Gainsboro"},
	{FXRGB(0xF8, 0xF8, 0xFF), "GhostWhite"},
	{FXRGB(0xFF, 0xD7, 0x00), "Gold"},
	{FXRGB(0xDA, 0xA5, 0x20), "Goldenrod"},
	{FXRGB(0x80, 0x80, 0x80), "Gray"},
	{FXRGB(0x00, 0x80, 0x00), "Green"},
	{FXRGB(0xAD, 0xFF, 0x2F), "GreenYellow"},
	{FXRGB(0xF0, 0xFF, 0xF0), "Honeydew"},
	{FXRGB(0xFF, 0x69, 0xB4), "HotPink"},
	{FXRGB(0xCD, 0x5C, 0x5C), "IndianRed"},
	{FXRGB(0x4B, 0x00, 0x82), "Indigo"},
	{FXRGB(0xFF, 0xFF, 0xF0), "Ivory"},
	{FXRGB(0xF0, 0xE6, 0x8C), "Khaki"},
	{FXRGB(0xE6, 0xE6, 0xFA), "Lavender"},
	{FXRGB(0xFF, 0xF0, 0xF5), "LavenderBlush"},
	{FXRGB(0x7C, 0xFC, 0x00), "LawnGreen"},
	{FXRGB(0xFF, 0xFA, 0xCD), "LemonChiffon"},
	{FXRGB(0xAD, 0xD8, 0xE6), "LightBlue"},
	{FXRGB(0xF0, 0x80, 0x80), "LightCoral"},
	{FXRGB(0xE0, 0xFF, 0xFF), "LightCyan"},
	{FXRGB(0xFA, 0xFA, 0xD2), "LightGoldenrodYellow"},
	{FXRGB(0x90, 0xEE, 0x90), "LightGreen"},
	{FXRGB(0xD3, 0xD3, 0xD3), "LightGray"},
	{FXRGB(0xFF, 0xB6, 0xC1), "LightPink"},
	{FXRGB(0xFF, 0xA0, 0x7A), "LightSalmon"},
	{FXRGB(0x20, 0xB2, 0xAA), "LightSeaGreen"},
	{FXRGB(0x87, 0xCE, 0xFA), "LightSkyBlue"},
	{FXRGB(0x77, 0x88, 0x99), "LightSlateGray"},
	{FXRGB(0xB0, 0xC4, 0xDE), "LightSteelBlue"},
	{FXRGB(0xFF, 0xFF, 0xE0), "LightYellow"},
	{FXRGB(0x00, 0xFF, 0x00), "Lime"},
	{FXRGB(0x32, 0xCD, 0x32), "Limegreen"},
	{FXRGB(0xFA, 0xF0, 0xE6), "Linen"},
	{FXRGB(0xFF, 0x00, 0xFF), "Magenta"},
	{FXRGB(0x80, 0x00, 0x00), "Maroon"},
	{FXRGB(0x66, 0xCD, 0xAA), "MediumAquaMarine"},
	{FXRGB(0x00, 0x00, 0xCD), "MediumBlue"},
	{FXRGB(0xBA, 0x55, 0xD3), "MediumOrchid"},
	{FXRGB(0x93, 0x70, 0xDB), "MediumPurple"},
	{FXRGB(0x3C, 0xB3, 0x71), "MediumSeaGreen"},
	{FXRGB(0x7B, 0x68, 0xEE), "MediumSlateBlue"},
	{FXRGB(0x00, 0xFA, 0x9A), "MediumSpringGreen"},
	{FXRGB(0x48, 0xD1, 0xCC), "MediumTurquoise"},
	{FXRGB(0xC7, 0x15, 0x85), "MediumVioletRed"},
	{FXRGB(0x19, 0x19, 0x70), "MidnightBlue"},
	{FXRGB(0xF5, 0xFF, 0xFA), "MintCream"},
	{FXRGB(0xFF, 0xE4, 0xE1), "MistyRose"},
	{FXRGB(0xFF, 0xE4, 0xB5), "Moccasin"},
	{FXRGB(0xFF, 0xDE, 0xAD), "NavajoWhite"},
	{FXRGB(0x00, 0x00, 0x80), "Navy"},
	{FXRGB(0xFD, 0xF5, 0xE6), "OldLace"},
	{FXRGB(0x80, 0x80, 0x00), "Olive"},
	{FXRGB(0x6B, 0x8E, 0x23), "OliveDrab"},
	{FXRGB(0xFF, 0xA5, 0x00), "Orange"},
	{FXRGB(0xFF, 0x45, 0x00), "OrangeRed"},
	{FXRGB(0xDA, 0x70, 0xD6), "Orchid"},
	{FXRGB(0xEE, 0xE8, 0xAA), "PaleGoldenrod"},
	{FXRGB(0x98, 0xFB, 0x98), "PaleGreen"},
	{FXRGB(0xAF, 0xEE, 0xEE), "PaleTurquoise"},
	{FXRGB(0xDB, 0x70, 0x93), "PaleVioletRed"},
	{FXRGB(0xFF, 0xEF, 0xD5), "Papayawhip"},
	{FXRGB(0xFF, 0xDA, 0xB9), "PeachPuff"},
	{FXRGB(0xCD, 0x85, 0x3F), "Peru"},
	{FXRGB(0xFF, 0xC0, 0xCB), "Pink"},
	{FXRGB(0xDD, 0xA0, 0xDD), "Plum"},
	{FXRGB(0xB0, 0xE0, 0xE6), "PowderBlue"},
	{FXRGB(0x80, 0x00, 0x80), "Purple"},
	{FXRGB(0xFF, 0x00, 0x00), "Red"},
	{FXRGB(0xBC, 0x8F, 0x8F), "RosyBrown"},
	{FXRGB(0x41, 0x69, 0xE1), "RoyalBlue"},
	{FXRGB(0x8B, 0x45, 0x13), "SaddleBrown"},
	{FXRGB(0xFA, 0x80, 0x72), "Salmon"},
	{FXRGB(0xF4, 0xA4, 0x60), "SandyBrown"},
	{FXRGB(0x2E, 0x8B, 0x57), "Seagreen"},
	{FXRGB(0xFF, 0xF5, 0xEE), "Seashell"},
	{FXRGB(0xA0, 0x52, 0x2D), "Sienna"},
	{FXRGB(0xC0, 0xC0, 0xC0), "Silver"},
	{FXRGB(0x87, 0xCE, 0xEB), "SkyBlue"},
	{FXRGB(0x6A, 0x5A, 0xCD), "SlateBlue"},
	{FXRGB(0x70, 0x80, 0x90), "SlateGray"},
	{FXRGB(0xFF, 0xFA, 0xFA), "Snow"},
	{FXRGB(0x00, 0xFF, 0x7F), "SpringGreen"},
	{FXRGB(0x46, 0x82, 0xB4), "SteelBlue"},
	{FXRGB(0xD2, 0xB4, 0x8C), "Tan"},
	{FXRGB(0x00, 0x80, 0x80), "Teal"},
	{FXRGB(0xD8, 0xBF, 0xD8), "Thistle"},
	{FXRGB(0xFF, 0x63, 0x47), "Tomato"},
	{FXRGB(0x40, 0xE0, 0xD0), "Turquoise"},
	{FXRGB(0xEE, 0x82, 0xEE), "Violet"},
	{FXRGB(0xF5, 0xDE, 0xB3), "Wheat"},
	{FXRGB(0xFF, 0xFF, 0xFF), "White"},
	{FXRGB(0xF5, 0xF5, 0xF5), "WhiteSmoke"},
	{FXRGB(0xFF, 0xFF, 0x00), "Yellow"},
	{FXRGB(0x9A, 0xCD, 0x32), "YellowGreen"},
	{0, 0}
};

COLOR PaletteColors[] = 
{
	{FXRGB(0xFF, 0xFF, 0xFF), "White"},
	{FXRGB(0xFF, 0xC0, 0xC0), ""},
	{FXRGB(0xFF, 0xE0, 0xC0), ""},
	{FXRGB(0xFF, 0xFF, 0xC0), ""},
	{FXRGB(0xC0, 0xFF, 0xC0), ""},
	{FXRGB(0xC0, 0xFF, 0xCF), ""},
	{FXRGB(0xC0, 0xC0, 0xFF), ""},
	{FXRGB(0xFF, 0xC0, 0xFF), ""},

	{FXRGB(0xE0, 0xE0, 0xE0), ""},
	{FXRGB(0xFF, 0x80, 0x80), ""},
	{FXRGB(0xFF, 0xC0, 0x80), ""},
	{FXRGB(0xFF, 0xFF, 0x80), ""},
	{FXRGB(0x80, 0xFF, 0x80), ""},
	{FXRGB(0x80, 0xFF, 0xFF), ""},
	{FXRGB(0x80, 0x80, 0xFF), ""},
	{FXRGB(0xFF, 0x80, 0xFF), ""},

	{FXRGB(0xC0, 0xC0, 0xC0), "Silver"},
	{FXRGB(0xFF, 0x00, 0x00), "Red"},
	{FXRGB(0xFF, 0x80, 0x00), ""},
	{FXRGB(0xFF, 0xFF, 0x00), "Yellow"},
	{FXRGB(0x00, 0xFF, 0x00), "Green"},
	{FXRGB(0x00, 0xFF, 0xFF), "Cyan"},
	{FXRGB(0x00, 0x00, 0xFF), "Blue"},
	{FXRGB(0xFF, 0x00, 0xFF), "Magenta"},

	{FXRGB(0x80, 0x80, 0x80), "Gray"},
	{FXRGB(0xC0, 0x00, 0x00), ""},
	{FXRGB(0xC0, 0x40, 0x00), ""},
	{FXRGB(0xC0, 0xC0, 0x00), ""},
	{FXRGB(0x00, 0xC0, 0x00), ""},
	{FXRGB(0x00, 0xC0, 0xC0), ""},
	{FXRGB(0x00, 0x00, 0xC0), ""},
	{FXRGB(0xC0, 0x00, 0xC0), ""},

	{FXRGB(0x40, 0x40, 0x40), ""},
	{FXRGB(0x80, 0x00, 0x00), "Maroon"},
	{FXRGB(0x80, 0x40, 0x00), ""},
	{FXRGB(0x80, 0x80, 0x00), "Olive"},
	{FXRGB(0x00, 0x80, 0x00), "Dark Green"},
	{FXRGB(0x00, 0x80, 0x80), "Teal"},
	{FXRGB(0x00, 0x00, 0x80), "Navy"},
	{FXRGB(0x80, 0x00, 0x80), "Purple"},

	{FXRGB(0x00, 0x00, 0x00), "Black"},
	{FXRGB(0x40, 0x00, 0x00), ""},
	{FXRGB(0x80, 0x40, 0x40), ""},
	{FXRGB(0x40, 0x40, 0x00), ""},
	{FXRGB(0x00, 0x40, 0x00), ""},
	{FXRGB(0x00, 0x40, 0x40), ""},
	{FXRGB(0x00, 0x00, 0x40), ""},
	{FXRGB(0x40, 0x00, 0x40), ""},

	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},

	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},
	{FXRGB(0xFF, 0xFF, 0xFF), ""},

	{0, 0}
};

///////////////////////////////////////////////////////////////////////////////
//

/*
 *  CColorTextField subclasses the CColorComboBox's text child to:
 *  
 *  o  Set it to read only
 *  o  Provide an additional Color property 
 *  o  Draw a small square the same colour as the currently selected colour
 *
 *  See also: 
 * 
 *  CColorComboBox::CColorComboBox - creates a CColorTextField.
 */

FXDEFMAP(CColorTextField) CColorTextFieldMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CColorTextField::onPaint)
};

FXIMPLEMENT(CColorTextField,FXTextField,CColorTextFieldMap,ARRAYNUMBER(CColorTextFieldMap))

CColorTextField::CColorTextField()
:FXTextField()
{
	setEditable(FALSE);
	m_SelectedColor = FXRGB(0,0,0);
	contents = "#000000";
}

CColorTextField::CColorTextField(FXComposite* p,FXint ncols,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb)
: FXTextField(p,ncols,tgt,sel,opts,x,y,w,h,pl,pr,pt, pb)
{
	setEditable(FALSE);
	m_SelectedColor = FXRGB(0,0,0);
	contents = "#000000";
}

void CColorTextField::setSelectedColor(FXColor Color)
{
	m_SelectedColor = Color;

	FXchar pszColor[12];
	sprintf(pszColor,"#%02x%02x%02x",FXREDVAL(Color),FXGREENVAL(Color),FXBLUEVAL(Color));

	contents = pszColor;
	contents.upper();
}

FXColor CColorTextField::getSelectedColor(void)
{
	return m_SelectedColor;
}

long CColorTextField::onPaint(FXObject* o, FXSelector s, void* ptr)
{
	FXEvent *ev=(FXEvent*)ptr;
	FXDCWindow dc(this,ev);

	FXint x = getX();
	FXint y = getY();
	FXint w = getWidth();
	FXint h = getHeight();
  
	drawFrame(dc,0,0,width,height);

	FXint th = getFont()->getFontHeight();

	dc.setForeground(getBackColor());
	dc.fillRectangle(x,y,w,h);

	x+=th+3;

	if(contents.text())
	{
		if(!isEnabled())
		{
			dc.setForeground(makeShadowColor(getBackColor()));
		}
		else
		{
			dc.setForeground(getTextColor());
		}

		dc.drawText(x,y+1+(h-th)/2+getFont()->getFontAscent(),contents.text(),contents.length());
    }

	dc.setForeground(FXRGB(0,0,0));
	dc.drawRectangle(3,y+3,th-3,th-3);
	dc.setForeground(m_SelectedColor);
	dc.fillRectangle(4,y+4,th-4,th-4);

	return 1;
}

///////////////////////////////////////////////////////////////////////////////
//

FXIMPLEMENT(CColorListItem,FXListItem,NULL,0)

CColorListItem::CColorListItem(CColorTextField* pTextField, CConnector* pConnector, FXString strName, FXColor rgb)
{
	label = strName;
	m_rgb = rgb;
	m_pConnector = pConnector;
	m_pTextField = pTextField;
}

CColorListItem::~CColorListItem()
{
}

FXint CColorListItem::hitItem(const FXList* list,FXint x,FXint y) const 
{
	FXint nReturn = FXListItem::hitItem(list, x, y);

	m_pConnector->set(getStringFromColor(m_rgb));

	m_pTextField->setSelectedColor(m_rgb);
	m_pTextField->update();

	return nReturn;
}

void CColorListItem::draw(const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h)
{
	FXint th = list->getFont()->getFontHeight();

	if(isSelected())
		dc.setForeground(list->getSelBackColor());
	else
		dc.setForeground(list->getBackColor());

	dc.fillRectangle(x,y,w,h);

	x+=th+3;

	if(!label.empty())
	{
		dc.setTextFont(list->getFont());

		if(!isEnabled())
			dc.setForeground(makeShadowColor(list->getBackColor()));
		else if(state&SELECTED)
			dc.setForeground(list->getSelTextColor());
		else
			dc.setForeground(list->getTextColor());
    
		dc.drawText(x,y+(h-th)/2+list->getFont()->getFontAscent(),label.text(),label.length());
    }

	dc.setForeground(FXRGB(0,0,0));
	dc.drawRectangle(3,y+3,th-3,th-3);
	dc.setForeground(m_rgb);
	dc.fillRectangle(4,y+4,th-4,th-4);
}

///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CColorChart) CColorChartMap[]=
{
	FXMAPFUNC(SEL_PAINT,0,CColorChart::onPaint),
	FXMAPFUNC(SEL_MOTION,0,CColorChart::onMotion),
	FXMAPFUNC(SEL_LEFTBUTTONPRESS,0,CColorChart::onLeftBtnPress),
	FXMAPFUNC(SEL_UPDATE,FXWindow::ID_QUERY_TIP,CColorChart::onQueryTip),
	FXMAPFUNC(SEL_ENTER,0,CColorChart::onEnter)
};

FXIMPLEMENT(CColorChart,FXFrame,CColorChartMap,ARRAYNUMBER(CColorChartMap))

CColorChart::CColorChart()
: FXFrame()
{
	flags|=FLAG_ENABLED|FLAG_SHOWN;

	m_pConnector = 0;
	m_pTextField = 0;

	m_nRow = -1;
	m_nCol = -1;
}

CColorChart::CColorChart(CColorTextField* pTextField, CConnector* pConnector, FXComposite* p,FXuint opts,FXint x,FXint y,FXint w,FXint h)
: FXFrame(p,opts,x,y,w,h)
{
	flags|=FLAG_ENABLED|FLAG_SHOWN;

	m_pConnector = pConnector;
	m_pTextField = pTextField;

	m_nRow = -1;
	m_nCol = -1;
}

FXint CColorChart::getDefaultWidth()
{
	return 142;
}

FXint CColorChart::getDefaultHeight()
{
	return 147;
}

void CColorChart::drawPaletteItem(FXDCWindow& dc, FXint nRow, FXint nCol)
{
	if ( nRow == -1 || nCol == -1 )
		return;

	// dark grey interior
	dc.setForeground(FXRGB(128, 128, 128));
	dc.fillRectangle(nCol*17+5,nRow*17+5,19,19);

	if ( nRow == m_nRow && nCol == m_nCol )
	{
		// Black Cursor border
		dc.setForeground(FXRGB(0,0,0));
		dc.drawRectangle(nCol*17+5,nRow*17+5,18,18);
		// White Cursor border
		dc.setForeground(FXRGB(255,255,255));
		dc.drawRectangle(nCol*17+6,nRow*17+6,16,16);
	}

	// Black border
	dc.setForeground(FXRGB(0,0,0));
	dc.fillRectangle(nCol*17+7,nRow*17+7,15,15);
	// Contents
	dc.setForeground(PaletteColors[nRow*8+nCol].Color);
	dc.fillRectangle(nCol*17+7+1,nRow*17+7+1,13,13);
}

void CColorChart::erasePaletteItem(FXDCWindow& dc, FXint nRow, FXint nCol)
{
	if ( nRow != -1 && nCol != -1 )
	{
		// dark grey interior
		dc.setForeground(FXRGB(128, 128, 128));
		dc.fillRectangle(nCol*17+5,nRow*17+5,18,18);
	}
}

void CColorChart::identifyPaletteItem(void)
{
	FXColor rgbCurrent = getColorFromString( m_pConnector->get() );

	FXint nItem = 0;
	for (COLOR* pColors = PaletteColors; pColors->Name; pColors++, nItem++)
	{
		if ( pColors->Color == rgbCurrent )
		{
			m_nRow = nItem / 8;
			m_nCol = nItem % 8;
			return;
		}
	}

	m_nRow = -1;
	m_nCol = -1;
}

long CColorChart::onPaint(FXObject*,FXSelector,void* ptr)
{
	identifyPaletteItem();

	FXEvent *ev=(FXEvent*)ptr;
	FXDCWindow dc(this,ev);

	// Entire area in light grey
	dc.setForeground(FXRGB(192, 192, 192));
	dc.fillRectangle(0,0, width,height);

	// Black Border...
	dc.setForeground(FXRGB(0, 0, 0));
	dc.drawRectangle(2,2,width-4,height-4);

	// ...With a dark grey interior
	dc.setForeground(FXRGB(128, 128, 128));
	dc.fillRectangle(3,3,width-5,height-5);

	for (int i = 0; i < 8; i++ ) // Row
	{
		for (int j = 0; j < 8; j++ ) // Col
		{
			drawPaletteItem(dc, i, j);
		}
	}

	drawPaletteItem(dc, m_nRow, m_nCol);

	return 1;
}

long CColorChart::onLeftBtnPress(FXObject* o,FXSelector s,void* ptr)
{ 
	FXEvent* event = (FXEvent*)ptr;
	FXDCWindow dc(this);

	FXint nRow = m_nRow;
	FXint nCol = m_nCol;

	m_nRow = -1;
	m_nCol = -1;

	drawPaletteItem(dc, nRow, nCol);
	
	m_nCol = (event->win_x-5) / 17;
	if ( m_nCol > 7 || event->win_x < m_nCol*17+7 || event->win_x > m_nCol*17+7+14 )
		m_nCol = -1;

	m_nRow = (event->win_y-5) / 17;
	if ( m_nRow > 7 || event->win_y < m_nRow*17+7 || event->win_y > m_nRow*17+7+14 )
		m_nRow = -1;

	if ( m_nRow < 0 || m_nCol < 0 )
	{
		m_nCol = nCol;
		m_nRow = nRow;
	}
	else
	{
		if ( m_pConnector )
		{
			m_pConnector->set( getStringFromColor(PaletteColors[m_nRow*8+m_nCol].Color) );

			m_pTextField->setSelectedColor(PaletteColors[m_nRow*8+m_nCol].Color);
			m_pTextField->update();
		}
	}

	drawPaletteItem(dc, m_nRow, m_nCol);

	return 1;
}

long CColorChart::onMotion(FXObject*,FXSelector,void* ptr)
{
	FXEvent* event = (FXEvent*)ptr;

	FXint nRow = -1;
	FXint nCol = -1;

	nCol = (event->win_x-5) / 17;
	if ( nCol > 7 || event->win_x < nCol*17+7 || event->win_x > nCol*17+7+14 )
		nCol = -1;

	nRow = (event->win_y-5) / 17;
	if ( nRow > 7 || event->win_y < nRow*17+7 || event->win_y > nRow*17+7+14 )
		nRow = -1;

	if ( nRow != -1 && nCol != -1 )
	{
		tip = PaletteColors[nRow*8+nCol].Name;
	}
	else
	{
		tip = "";
	}

	return 1;
}

long CColorChart::onQueryTip(FXObject* sender,FXSelector,void*)
{
	if( !tip.empty() && (flags&FLAG_TIP) )
	{
		sender->handle(this,MKUINT(ID_SETSTRINGVALUE,SEL_COMMAND),(void*)&tip);
		return 1;
    }

	return 0;
}

long CColorChart::onEnter(FXObject* o,FXSelector s,void* p)
{
	return FXWindow::onEnter(o, s, p);
}
  
///////////////////////////////////////////////////////////////////////////////
//

FXDEFMAP(CColorList) CColorListMap[]=
{
//	FXMAPFUNC(SEL_FOCUSIN,0,CColorList::onEnter),
//	FXMAPFUNC(SEL_ENTER,0,CColorList::onEnter),
//	FXMAPFUNC(SEL_UPDATE,0,CColorList::onUpdate),
	FXMAPFUNC(SEL_PAINT,0,CColorList::onPaint)
};

FXIMPLEMENT(CColorList, FXList, CColorListMap, ARRAYNUMBER(CColorListMap)) 

CColorList::CColorList(FXComposite* p, CConnector* pConnector, FXuint opts)
: FXList(p,1,NULL,0,opts,0,0,0,0)
{
	flags|=FLAG_ENABLED|FLAG_SHOWN|LIST_BROWSESELECT;

	m_pConnector = pConnector;
}

CColorList::~CColorList()
{
}

void CColorList::identifyInternetItem(void)
{
	static FXbool bSelected = FALSE;

	FXColor rgbCurrent = getColorFromString( m_pConnector->get() );

	FXint nItem = 0;
	for (COLOR* pColors = InternetColors; pColors->Name; pColors++, nItem++)
	{
		if ( pColors->Color == rgbCurrent )
		{
			selectItem(nItem);
//			makeItemVisible(nItem); // CMW 04.12.00 Commented out - couldn't scroll
			                        //              beyond selected item introduced
			                        //              in 0.99.142>
			bSelected = TRUE;
			return;
		}
	}

	if ( bSelected )
	{
		killSelection();
		makeItemVisible(0);
		bSelected = FALSE;
	}
}

long CColorList::onPaint(FXObject* o, FXSelector sel, void* ptr)
{
	identifyInternetItem();

	return FXList::onPaint(o, sel, ptr);
}

///////////////////////////////////////////////////////////////////////////////
//

FXIMPLEMENT(CColorComboBox,FXPacker,0,0)

CColorComboBox::CColorComboBox(CConnector* pConnector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h,FXint pl,FXint pr,FXint pt,FXint pb):
  CComboBox(p,cols,nvis,tgt,sel,opts,x,y,w,h, pl,pr,pt,pb)
{
	// CMW Note the 2,0,2,0 padding -- this is necessary to align the "(Select)" with the parent CPropertyItem "(Select)".
	text=new CColorTextField(this,cols,this,0,0, 0,0,0,0, 2,0,2,0/*pl,pr,pt,pb*/);

	tabbook=new FXTabBook(pane,NULL,0,FRAME_RAISED|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_RIGHT);

	tab1=new FXTabItem(tabbook,"Palette");
	FXHorizontalFrame* pFrame1 = new FXHorizontalFrame(tabbook,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_RAISED);
	m_pColorChart = new CColorChart((CColorTextField*)text, pConnector, pFrame1, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_SUNKEN, 0, 0, 142, 147);

	tab2=new FXTabItem(tabbook,"Internet");
	FXHorizontalFrame* pFrame2 = new FXHorizontalFrame(tabbook,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_RAISED);
//	FXList* pList = new FXList(pFrame2,1,0,0,LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_SUNKEN);
	CColorList* pList = new CColorList(pFrame2, pConnector, LAYOUT_FILL_X|LAYOUT_FILL_Y|FRAME_THICK|FRAME_SUNKEN);

	for (COLOR* pColors = InternetColors; pColors->Name; pColors++)
	{
		pList->appendItem( new CColorListItem((CColorTextField*)text, pConnector, pColors->Name, pColors->Color) );
	}
}

CColorComboBox::~CColorComboBox()
{
	text=(FXTextField*)-1;
	tabbook=(FXTabBook*)-1;
}

FXint CColorComboBox::getDefaultWidth()
{
	FXint ww,pw;
	ww=text->getDefaultWidth()+button->getDefaultWidth()+(border<<1);
	pw=pane->getDefaultWidth();
	return FXMAX(ww,pw);
}

FXint CColorComboBox::getDefaultHeight()
{
	FXint th,bh;
	th=text->getDefaultHeight();
	bh=button->getDefaultHeight();
	return FXMAX(th,bh)+(border<<1);
}

// Recalculate layout
void CColorComboBox::layout()
{
	FXint buttonWidth,textWidth,itemHeight;
	itemHeight=height-(border<<1);
	buttonWidth=button->getDefaultWidth();
	textWidth=width-buttonWidth-(border<<1);
	text->position(border,border,textWidth,itemHeight);
	button->position(border+textWidth,border,buttonWidth,itemHeight);

	pane->setX(pane->getX()-width);
  
	flags&=~FLAG_DIRTY;
}

