/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CColorComboBox_H__
#define __CColorComboBox_H__

class CColorTextField : public FXTextField
{
	FXDECLARE(CColorTextField)

// Services
public:
	CColorTextField();
	CColorTextField(FXComposite* p,FXint ncols,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_SUNKEN|FRAME_THICK,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

// Properties
public:
	void setSelectedColor(FXColor Color);
	FXColor getSelectedColor(void);

// Events
public:
	long onPaint(FXObject*,FXSelector,void*);

// Attributes
protected:
	FXColor m_SelectedColor;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CColorListItem : public FXListItem
{
	FXDECLARE(CColorListItem)
	friend class FXList;

// Services
public:
	CColorListItem() {}
	CColorListItem(CColorTextField* pTextField, CConnector* pConnector, FXString strName, FXColor rgb);
	virtual ~CColorListItem();

// Properties
public:
	void setText(FXString str) {label = str;}

	CConnector* getConnector(void) {return m_pConnector;}

protected:
	virtual void draw(const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h);
	virtual FXint hitItem(const FXList* list,FXint x,FXint y) const;

// Attributes
private:
	// Use the base class label member to hold the name
	FXColor m_rgb;

	CConnector* m_pConnector;
	CColorTextField* m_pTextField;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CColorChart : public FXFrame
{
	FXDECLARE(CColorChart)

// Services
public:
	CColorChart();
	CColorChart(CColorTextField* pTextField, CConnector* pConnector, FXComposite* p,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0);

// Operations
public:
	virtual FXint getDefaultWidth();
	virtual FXint getDefaultHeight();

	void drawPaletteItem(FXDCWindow& dc, FXint nRow, FXint nCol);
	void erasePaletteItem(FXDCWindow& dc, FXint nRow, FXint nCol);
	void identifyPaletteItem(void);

// Events
public:
	long onPaint(FXObject*,FXSelector,void*);
	long onLeftBtnPress(FXObject* o,FXSelector s,void* ptr);
	long onMotion(FXObject*,FXSelector,void* ptr);
	long onQueryTip(FXObject* sender,FXSelector,void*);
	long onEnter(FXObject* o,FXSelector s,void* p);

// Attributes
private:
	CConnector*      m_pConnector;
	CColorTextField* m_pTextField;

	FXint m_nRow;
	FXint m_nCol;

	FXString tip;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CColorList : public FXList
{
	FXDECLARE(CColorList)

// Services
public:
	CColorList(FXComposite* p, CConnector* pConnector, FXuint opts=0); 

	CColorList() : FXList() {m_pConnector = 0;}
	CColorList(const CColorList&) : FXList() {}
	virtual ~CColorList();

// Operations
public:	
	void identifyInternetItem(void);

// Events
public:
	long onPaint(FXObject* o, FXSelector s, void* ptr);
//	long onEnter(FXObject* sender,FXSelector sel,void* ptr);
//	long onUpdate(FXObject* o, FXSelector sel, void* ptr);

// Attributes
private:
	CConnector* m_pConnector;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CColorComboBox : public CComboBox
{
	FXDECLARE(CColorComboBox)

// Services
public:
	CColorComboBox(CConnector* pConnector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
	virtual ~CColorComboBox();

protected:
	CColorComboBox() {}

// Properties
public:
	virtual FXint getDefaultWidth();
	virtual FXint getDefaultHeight();

	void setSelectedColor(FXColor Color) {((CColorTextField*)text)->setSelectedColor(Color);}
	FXColor getSelectedColor(void) {return ((CColorTextField*)text)->getSelectedColor();}

protected:
	virtual void layout();

// Attributes
protected:
	FXTextField* text;
	FXTabBook*   tabbook;
	FXTabItem*   tab1;
	FXTabItem*   tab2;

	CColorChart* m_pColorChart;
};

#endif // __CColorComboBox_H__
