/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>
#include "fxkeys.h"

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include <CComboBox.h>

FXDEFMAP(CComboPopup) CComboPopupMap[]=
{
	FXMAPFUNC(SEL_KEYPRESS,0,CComboPopup::onKeyPress),
	FXMAPFUNC(SEL_KEYRELEASE,0,CComboPopup::onKeyRelease)
};

FXIMPLEMENT(CComboPopup,FXPopup,CComboPopupMap,ARRAYNUMBER(CComboPopupMap))

CComboPopup::CComboPopup()
:FXPopup()
{
	m_pParent = 0;
}

CComboPopup::CComboPopup(FXObject* pParent, FXWindow* owner,FXuint opts,FXint x,FXint y,FXint w,FXint h)
: FXPopup(owner,opts,x,y,w,h)
{
	m_pParent = pParent;
}

long CComboPopup::onKeyPress(FXObject*,FXSelector,void* ptr)
{
	if ( m_pParent ) m_pParent->handle(this,MKUINT(message,SEL_KEYPRESS),ptr);
	return 1;
}

long CComboPopup::onKeyRelease(FXObject*,FXSelector,void* ptr)
{
	if ( m_pParent ) m_pParent->handle(this,MKUINT(message,SEL_KEYRELEASE),ptr);
	return 1;
}

///////////////////////////////////////////////////////////////////////////////
//

FXIMPLEMENT(CComboBox, FXPacker, 0, 0) 

CComboBox::CComboBox(FXComposite *p,FXint cols,FXint nvis,FXObject* tgt,FXSelector sel,FXuint opts,FXint x,FXint y,FXint w,FXint h, FXint pl,FXint pr,FXint pt,FXint pb):
  FXPacker(p,opts,x,y,w,h, pl,pr,pt,pb, 0,0)
{
	flags|=FLAG_ENABLED;

	message=sel;

	pane=new CComboPopup(tgt, this, FRAME_GROOVE);

	button=new CMenuButton(this,NULL,NULL,pane,FRAME_RAISED|FRAME_THICK|MENUBUTTON_DOWN|MENUBUTTON_ATTACH_RIGHT, 0,0,0,0, 0,0,0,0);
	button->setXOffset(border);
	button->setYOffset(border);

//	button->setState(STATE_DOWN);

	flags&=~FLAG_UPDATE;  // Never GUI update
}

CComboBox::~CComboBox()
{
	delete pane;
	pane=(FXPopup*)-1;
	button=(CMenuButton*)-1;
}

void CComboBox::create()
{
	FXPacker::create();
	pane->create();
}

void CComboBox::detach()
{
	FXPacker::detach();
	pane->detach();
}

void CComboBox::destroy()
{
	pane->destroy();
	FXPacker::destroy();
}
