/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CNameComboBox_H__
#define __CNameComboBox_H__

///////////////////////////////////////////////////////////////////////////////
//

class CNameTextField : public FXTextField
{
	FXDECLARE(CNameTextField)

// Services
public:
	CNameTextField();
	CNameTextField(FXComposite* p,FXint ncols,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=FRAME_SUNKEN|FRAME_THICK,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);

// Properties
public:
	void setClass(FXString strClass) {m_strClass = strClass;}

// Events
public:
	long onPaint(FXObject*,FXSelector,void*);
	long onFocusIn(FXObject* sender,FXSelector sel,void* ptr);
	long onFocusOut(FXObject* sender,FXSelector sel,void* ptr);

// Attributes
protected:
	// Use the base class contents member to hold the name
	FXString m_strClass;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CNameListItem : public FXListItem
{
	FXDECLARE(CNameListItem)
	friend class FXList;

// Services
public:
	CNameListItem() {}
	CNameListItem(CConnector connector, FXString strName, FXString strClass);
	virtual ~CNameListItem();

// Properties
public:
	void setText(FXString str) {label = str;}

	void setClass(FXString strClass) {m_strClass = strClass;}
	FXString getClass(void) {return m_strClass;}

	CConnector* getConnector(void) {return &m_Connector;}

protected:
	virtual void draw(const FXList* list,FXDC& dc,FXint x,FXint y,FXint w,FXint h);

// Attributes
private:
	// Use the base class label member to hold the name
	FXString m_strClass;

	CConnector m_Connector;
};

///////////////////////////////////////////////////////////////////////////////
//

class FXAPI CNameComboBox : public CComboBox
{
	FXDECLARE(CNameComboBox)

// Services
public:
	CNameComboBox(CConnector& connector, FXComposite *p,FXint cols,FXint nvis,FXObject* tgt=NULL,FXSelector sel=0,FXuint opts=0,FXint x=0,FXint y=0,FXint w=0,FXint h=0,FXint pl=DEFAULT_PAD,FXint pr=DEFAULT_PAD,FXint pt=DEFAULT_PAD,FXint pb=DEFAULT_PAD);
	virtual ~CNameComboBox();

protected:
	CNameComboBox() {}

// Properties
public:
	virtual FXint getDefaultWidth();
	virtual FXint getDefaultHeight();

	void setText(FXString str) {text->setText(str); text->update();}
	void setClass(FXString str) {text->setClass(str);}

// Operations
public:
	virtual void create();

	FXint appendItem(CNameListItem* item);
	void removeItem(CConnector& connector);
	void removeItem(FXint index);

	void setCurrentItem(FXint index);
	void setCurrentItem(CConnector& connector, FXbool bMultiple = FALSE);

protected:
	virtual void layout();

// Events
public:
  enum{
    ID_LIST=FXPacker::ID_LAST,
    ID_LAST
    };

	long onListClicked(FXObject*,FXSelector,void* ptr);

// Attributes
protected:
	CNameTextField*	text;
	FXList*			list;

protected:
	CConnector m_Connector;
};

#endif // __CNameComboBox_H__
