/**************************************************************************  
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CTrackManager_H__
#define __CTrackManager_H__

#include <new.h>
#include "FXElement.h"
#include "FXArray.h"

class CAppDialog;

class CTrackManager : public FXObject
{
	FXDECLARE(CTrackManager)

	friend CAppDialog;

// Services
public:
	CTrackManager();
	virtual ~CTrackManager();

// Properties
public:
	void DragOutline(FXbool b) {m_bOutline = b;}
	FXbool DragOutline(void) {return m_bOutline;}

	void SnapToGrid(FXbool b) {m_bSnapToGrid = b;}
	FXbool SnapToGrid(void) {return m_bSnapToGrid;}

// Operations
public:
	void alignLeft(void);
	void alignRight(void);
	void alignTop(void);
	void alignBottom(void);

	void sizeWidth(void);
	void sizeHeight(void);
	void sizeBoth(void);

	void centreHorizontal(void);
	void centreVertical(void);

	FXbool isMultipleSelected(void);

	void track(void);

	void setFrames(FXWindow* pAppFrame, FXWindow* pFrame, FXWindow* pPane) 
		{m_pAppFrame = pAppFrame; m_pFrame = pFrame; m_pPane = pPane;}

	void select(CConnector& connector, FXbool bMultiple);

	void clearGrabbers(void);

	FXint getTrackerCount(void);
	CConnector* getConnector(FXint nIndex);

	void recalc();

private:
	void clearTrackers(void);

	void drawGrabbers(void);
	void drawGrabber(FXint index);

//	FXbool isGrabberAtPoint(FXint& nIndex, FXshort& x, FXshort& y);
	FXbool isGrabberAtPoint2(FXWindow* window, FXshort& x, FXshort& y, FXint& nIndex, eGrabber& grabber);

	void setInactive(void);
	FXbool isActive(FXint& nIndex, FXshort& x, FXshort& y);
	FXbool isActive(FXint& nIndex);
	void drawBoundary(FXDCWindow& dc);
	void track(FXint& nIndex);
	void snap(FXshort nSnap);
	void release(void);
	void position(void);
	FXbool isWindowInList(FXint& nIndex, FXWindow* pWnd);

	FXbool trackDialog(void);

	void resetAction(void);

	FXWindow* addWidget(FXWindow* pParent, FXbool bCreate, FXbool bSetCurrent = TRUE, FXshort x = 0, FXshort y = 0, FXshort w = 0, FXshort h = 0);

// Events
public:
	enum
	{
		ID_ADDWIDGET = 0
	};

	long onAddWidget(FXObject* o,FXSelector s,void* ptr);

// Events
protected:
	void onPaint(FXWindow* window);

	FXbool onLButtonDown(FXWindow* window, FXint x, FXint y, FXbool bCtrl);
	FXbool onMouseMove(FXWindow* window, FXint x, FXint y);
	FXbool onLButtonUp(FXWindow* window, FXint x, FXint y); 

	FXbool OnKeyDown(FXWindow* window, FXuint key, FXuint code);
	FXbool OnKeyUp(FXWindow* window, FXuint key, FXuint code);

// Attributes
private:
	FXbool m_bOutline;
	FXbool m_bSnapToGrid;
	FXshort m_nSnap;

	FXWindow* m_pAppFrame;	// This is the main window (the white window the 'dialog' is on)
	FXWindow* m_pFrame;		// This is the 'dialog box'.
	FXWindow* m_pPane;		// This is the contents of the 'dialog box'
	
	FXWindow* m_pTrackingWindow;
	FXWindow* m_pCreateWindow;

	FXArray<CTracker*> m_Trackers;

	FXshort m_nOriginalLeft;
	FXshort m_nOriginalTop;

	FXshort m_nCursorX;
	FXshort m_nCursorY;

	FXbool m_bMouseDown;
	FXbool m_bRubberBand;

	FXint m_nRubberBandIndex;

	FXint m_nPrimaryTracker;

	CConnector* m_pAction;
	CConnector* m_pCurrent;
};

#endif // __CTrackManager_H__