/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#include <fx.h>

#include <ITextStream.h>
#include <IConnector.h>
#include <IProperty.h>
#include <IDispenser.h>

#include <CConnector.h>

#include "CTracker.h"
#include "CTrackManager.h"
#include <CApp.h>
#include "CAppDialog.h"


	#ifndef WIN32 
		#include <stdio.h> 
		#include <stdlib.h> 
		#include <stdarg.h> 
		void DebugPrint(const char *fmt,...) 
		{ 
			char buffer[1024]; 
			va_list arg_ptr; 
			va_start(arg_ptr, fmt); 
			vsprintf(buffer,fmt,arg_ptr); 
			va_end(arg_ptr); 
 
			printf("%s\n", buffer); 
		}
	#else
		#include <windows.h>
		#include <stdio.h>
		#include <stdlib.h>
		#include <stdarg.h>
		void DebugPrint(const char *fmt,...)
		{
			char buffer[1024];
			va_list arg_ptr;
			va_start(arg_ptr, fmt);
			vsprintf(buffer,fmt,arg_ptr);
			va_end(arg_ptr);

			::OutputDebugString(buffer);
		}
	#endif

void CTracker::getPosition(FXWindow* frame, FXWindow* window, FXshort& x, FXshort& y)
{
	if ( frame == 0 )
		return;

	//
	// First, establish the (x,y) position relative to the root
	// window. In other words its absolute position on screen.
	//
	FXWindow* parent = window;

	do
	{
		y += parent->getY();
		x += parent->getX();
		parent = parent->getParent();
	}
	while ( parent );

	//
	// Then calcluate the (x,y) position relative to the requested frame
	//
	parent = frame;

	do
	{
		y -= parent->getY();
		x -= parent->getX();
		parent = parent->getParent();
	}
	while ( parent );
}

void CTracker::getAbsolutePosition(FXWindow* window, FXshort& x, FXshort& y)
{
	if ( window )
	{
		FXWindow* parent = window;

		do
		{
			y += parent->getY();
			x += parent->getX();
			parent = parent->getParent();
		}
		while ( parent );
	}
}

void CTracker::snap(FXshort& value, FXshort nSnap)
{
	FXint r = value % nSnap;
	if (r > 0)
	{
		if (r < 4)
		{
			value -= r;
		}
		else
		{
			value += nSnap - r;
		}
	}
}

CTracker::CTracker()
{
	TRACKRECT r;

	r.bActive = FALSE;
	
	memset(&m_Grabbers[0], 0, sizeof(m_Grabbers));

	m_SelectedGrabber = NONE;

	m_bActive = FALSE;

	m_pWindow = 0;
	m_pFrame = 0;
}

CTracker::CTracker(CTracker& t)
{
	for (FXint i = 0; i < 9; i++)
	{
		m_Grabbers[i] = t.m_Grabbers[i];
	}

	m_rect = t.m_rect;
	m_pWindow = t.m_pWindow;
	m_pFrame = t.m_pFrame;
	m_SelectedGrabber = t.m_SelectedGrabber;

	m_bActive = t.m_bActive;
}

void CTracker::track(FXbool bActive)
{
	m_SelectedGrabber = ALL;
	m_bActive = TRUE;

	m_Grabbers[TOP_LEFT].bActive = bActive;
	m_Grabbers[TOP_CENTRE].bActive = bActive;
	m_Grabbers[TOP_RIGHT].bActive = bActive;
	m_Grabbers[CENTRE_LEFT].bActive = bActive;
	m_Grabbers[CENTRE_RIGHT].bActive = bActive;
	m_Grabbers[BOTTOM_LEFT].bActive = bActive;
	m_Grabbers[BOTTOM_CENTRE].bActive = bActive;
	m_Grabbers[BOTTOM_RIGHT].bActive = bActive;
}

void CTracker::track(FXWindow* frame, FXWindow* window, FXint style)
{
	FXint dialogstyle = GS_DIALOG;
	if ( (style == dialogstyle) )
	{
		m_SelectedGrabber = NONE;
	}
	else
	{
		m_SelectedGrabber = ALL;
		m_Grabbers[ALL].bActive = TRUE;
	}

	m_pFrame  = frame;
	m_pWindow = window;

	m_bActive = TRUE;

	if ( style & GS_TOP_LEFT )
		m_Grabbers[TOP_LEFT].bActive = TRUE;
	if ( style & GS_TOP_CENTRE )  
		m_Grabbers[TOP_CENTRE].bActive = TRUE;
	if ( style & GS_TOP_RIGHT )
		m_Grabbers[TOP_RIGHT].bActive = TRUE;
	if ( style & GS_CENTRE_LEFT )
		m_Grabbers[CENTRE_LEFT].bActive = TRUE;
	if ( style & GS_CENTRE_RIGHT )
		m_Grabbers[CENTRE_RIGHT].bActive = TRUE;
	if ( style & GS_BOTTOM_LEFT )
		m_Grabbers[BOTTOM_LEFT].bActive = TRUE;
	if ( style & GS_BOTTOM_CENTRE )
		m_Grabbers[BOTTOM_CENTRE].bActive = TRUE;
	if ( style & GS_BOTTOM_RIGHT )
		m_Grabbers[BOTTOM_RIGHT].bActive = TRUE;

	//
	// I've adjusted the rectangle here because some windows eg radio
	// and check buttons seem to return a height and width that's one
	// out causing the track rect to display incorrectly.
	//
	// Note that CTracker::position() must reverse these adjustments 
	// to the window's rect otherwise the window gradually increases
	// is size if you repeatedly click on the window!
	//
	m_rect.nTop = /*0*/-1;
	m_rect.nLeft = /*0*/-1;
	m_rect.nHeight = window->getHeight()+2;
	m_rect.nWidth = window->getWidth()+2;

	/* CMW 5.8.99 m_pFrame changed to frame*/
	CTracker::getPosition(frame, window, m_rect.nLeft, m_rect.nTop);
}

//
// This override is used for 'rubber band' mode -- it sets
// a rectangle and activates a single grabber. The c-tor 
// has initialised m_pWindow to NULL so we don't try and
// move a non-existent window!
//
void CTracker::track(FXshort top, FXshort left)
{
	// -2 is used so initial rectangle drawn
	// by drawBoundary is a zero rectangle.
	// This ensures that the rubber band 
	// rectangle is drawn correctly.
	m_rect.nTop = top-2;
	m_rect.nLeft = left-2;

	m_rect.nHeight = 0;
	m_rect.nWidth  = 0;

	m_SelectedGrabber = BOTTOM_RIGHT;
	m_Grabbers[m_SelectedGrabber].bActive = TRUE;
}

void CTracker::move(FXint x, FXint y, bool bNormalise)
{
	if ( m_Grabbers[m_SelectedGrabber].bActive == FALSE )
	{
		//DebugPrint("CTracker::move not active selected = %i\n", m_SelectedGrabber);
		return;
	}

	switch ( m_SelectedGrabber )
	{
	case TOP_LEFT:
		m_rect.nLeft   = m_rect.nLeft    + x;
		m_rect.nTop    = m_rect.nTop     + y;
		m_rect.nWidth  = m_rect.nWidth   - x;
		m_rect.nHeight = m_rect.nHeight  - y;
		break;

	case TOP_CENTRE:
		m_rect.nTop    = m_rect.nTop     + y;
		m_rect.nHeight = m_rect.nHeight  - y;
		break;

	case TOP_RIGHT:
		m_rect.nTop    = m_rect.nTop     + y;
		m_rect.nWidth  = m_rect.nWidth   + x;
		m_rect.nHeight = m_rect.nHeight  - y;
		break;

	case CENTRE_LEFT:
		m_rect.nLeft   = m_rect.nLeft    + x;
		m_rect.nWidth  = m_rect.nWidth   - x;
		break;

	case CENTRE_RIGHT:
		m_rect.nWidth  = m_rect.nWidth   + x;
		break;

	case BOTTOM_LEFT:
		m_rect.nLeft   = m_rect.nLeft    + x;
		m_rect.nWidth  = m_rect.nWidth   - x;
		m_rect.nHeight = m_rect.nHeight  + y;
		break;

	case BOTTOM_CENTRE:
		m_rect.nHeight = m_rect.nHeight  + y;
		break;

	case BOTTOM_RIGHT:
		m_rect.nWidth  = m_rect.nWidth   + x;
		m_rect.nHeight = m_rect.nHeight  + y;
		break;

	case ALL:
		m_rect.nLeft = m_rect.nLeft + x;
		m_rect.nTop  = m_rect.nTop  + y;
		break;

	case NONE:
		break;
	};

	// CMW 28.10.99 If the user has inverted the widget then normalise
	//				the position of the widget and switch trackers
	if ( bNormalise )
	{
		if ( m_rect.nWidth < 0 )
		{
			m_rect.nLeft  += m_rect.nWidth;
			m_rect.nWidth = m_rect.nWidth * -1;

			switch ( m_SelectedGrabber )
			{
			case TOP_LEFT:     m_SelectedGrabber = TOP_RIGHT;    break;
			case TOP_RIGHT:    m_SelectedGrabber = TOP_LEFT;     break;
			case CENTRE_LEFT:  m_SelectedGrabber = CENTRE_RIGHT; break;
			case CENTRE_RIGHT: m_SelectedGrabber = CENTRE_LEFT;  break;
			case BOTTOM_LEFT:  m_SelectedGrabber = BOTTOM_RIGHT; break;
			case BOTTOM_RIGHT: m_SelectedGrabber = BOTTOM_LEFT;  break;
			case TOP_CENTRE:
			case BOTTOM_CENTRE:
			case ALL:
			case NONE:
				break;
			};
		}

		if ( m_rect.nHeight < 0 )
		{
			m_rect.nTop  += m_rect.nHeight;
			m_rect.nHeight = m_rect.nHeight * -1;

			switch ( m_SelectedGrabber )
			{
			case TOP_LEFT:      m_SelectedGrabber = BOTTOM_LEFT;   break;
			case TOP_CENTRE:    m_SelectedGrabber = BOTTOM_CENTRE; break;
			case TOP_RIGHT:     m_SelectedGrabber = BOTTOM_RIGHT;  break;
			case BOTTOM_LEFT:   m_SelectedGrabber = TOP_LEFT;      break;
			case BOTTOM_CENTRE: m_SelectedGrabber = TOP_CENTRE;    break;
			case BOTTOM_RIGHT:  m_SelectedGrabber = TOP_RIGHT;     break;
			case CENTRE_LEFT:
			case CENTRE_RIGHT:
			case ALL:
			case NONE:
				break;
			};
		}
	}
}

void CTracker::snap(FXshort nSnap)
{
	CTracker::snap(m_rect.nTop, nSnap);
	CTracker::snap(m_rect.nLeft, nSnap);
	CTracker::snap(m_rect.nHeight, nSnap);
	CTracker::snap(m_rect.nWidth, nSnap);

	// Adjust slightly so widget fits exactly over grid points
	m_rect.nHeight++;
	m_rect.nWidth++; 
}

void CTracker::release(void)
{
	m_Grabbers[TOP_LEFT].nLeft = m_rect.nLeft-7;
	m_Grabbers[TOP_LEFT].nTop  = m_rect.nTop-7;

	m_Grabbers[TOP_CENTRE].nLeft = m_rect.nLeft + m_rect.nWidth/2/*+3*/;
	m_Grabbers[TOP_CENTRE].nTop  = m_rect.nTop-7;

	m_Grabbers[TOP_RIGHT].nLeft = m_rect.nLeft + m_rect.nWidth;
	m_Grabbers[TOP_RIGHT].nTop  = m_rect.nTop-7;

	m_Grabbers[CENTRE_LEFT].nLeft = m_rect.nLeft-7;
	m_Grabbers[CENTRE_LEFT].nTop  = m_rect.nTop + m_rect.nHeight/2 -4;

	m_Grabbers[CENTRE_RIGHT].nLeft = m_rect.nLeft + m_rect.nWidth;
	m_Grabbers[CENTRE_RIGHT].nTop  = m_rect.nTop + m_rect.nHeight/2 -4;

	m_Grabbers[BOTTOM_LEFT].nLeft = m_rect.nLeft-7;
	m_Grabbers[BOTTOM_LEFT].nTop  = m_rect.nTop + m_rect.nHeight;

	m_Grabbers[BOTTOM_CENTRE].nLeft = m_rect.nLeft + m_rect.nWidth/2/*+3*/;
	m_Grabbers[BOTTOM_CENTRE].nTop  = m_rect.nTop + m_rect.nHeight;

	m_Grabbers[BOTTOM_RIGHT].nLeft = m_rect.nLeft + m_rect.nWidth;
	m_Grabbers[BOTTOM_RIGHT].nTop  = m_rect.nTop + m_rect.nHeight;
}

FXbool CTracker::isGrabberAtPoint(FXWindow* pFrame, FXshort& x, FXshort& y)
{
	m_SelectedGrabber = NONE;

	// Quick check first
	FXshort tlx = m_Grabbers[TOP_LEFT].nLeft;
	FXshort tly = m_Grabbers[TOP_LEFT].nTop;
	FXshort brx = m_Grabbers[BOTTOM_RIGHT].nLeft + 7;
	FXshort bry = m_Grabbers[BOTTOM_RIGHT].nTop + 7;

	CTracker::getPosition(m_pFrame, pFrame, tlx, tly);
	CTracker::getPosition(m_pFrame, pFrame, brx, bry);

	if	(
		x >= tlx && x <= brx &&
		y >= tly && y <= bry
		)
	{
		// Now find specific grabber
		for ( FXint i = 0; i < 8; i++ )
		{
			tlx = m_Grabbers[i].nLeft;
			tly = m_Grabbers[i].nTop;
			CTracker::getPosition(m_pFrame, pFrame, tlx, tly);

			if	(
				x >= tlx && x <= (tlx + 7) &&
				y >= tly && y <= (tly + 7)
				)
			{
				m_SelectedGrabber = (eGrabber)i;
				return TRUE;
			}
		}
	}

	return FALSE;
}

void CTracker::drawBoundary(FXDCWindow& dc)
{
	dc.clipChildren(FALSE);
	dc.setFunction(BLT_NOT_DST);

	// Draw initial bounding box
	dc.fillRectangle(m_rect.nLeft - 2, m_rect.nTop - 2, m_rect.nWidth + 4, 2); // Top
	dc.fillRectangle(m_rect.nLeft + m_rect.nWidth, m_rect.nTop, 2, m_rect.nHeight); // Right
	dc.fillRectangle(m_rect.nLeft, m_rect.nTop + m_rect.nHeight, m_rect.nWidth + 2, 2); // Bottom
	dc.fillRectangle(m_rect.nLeft - 2, m_rect.nTop, 2, m_rect.nHeight + 2); // Left
}

void CTracker::position(FXshort nTop, FXshort nLeft, FXshort nWidth, FXshort nHeight)
{
	if ( m_pWindow )
	{
		m_rect.nLeft   = nLeft;
		m_rect.nTop    = nTop;
		m_rect.nWidth  = nWidth;
		m_rect.nHeight = nHeight;

		release();

		m_pWindow->position(m_rect.nLeft, m_rect.nTop, m_rect.nWidth, m_rect.nHeight);
		m_pWindow->update();
	}
}

//
// The magic numbers here reverse the adjustments made to the
// tracking rectangle in CTracker::track() -- see above for
// details.
//
void CTracker::position(FXbool bInMultiple)
{
	// CMW 19.05.00 Don't move a window if none of the 
	//				grabbers are active. Stops things
	//				like tabitems from jumping when
	//				you click on them. The exception
	//				is when a widget is part of a multiple
	//				selected group.

	FXbool bActive = FALSE;

	bActive |= m_Grabbers[TOP_LEFT].bActive;
	bActive |= m_Grabbers[TOP_CENTRE].bActive;
	bActive |= m_Grabbers[TOP_RIGHT].bActive;
	bActive |= m_Grabbers[CENTRE_LEFT].bActive;
	bActive |= m_Grabbers[CENTRE_RIGHT].bActive;
	bActive |= m_Grabbers[BOTTOM_LEFT].bActive;
	bActive |= m_Grabbers[BOTTOM_CENTRE].bActive;
	bActive |= m_Grabbers[BOTTOM_RIGHT].bActive;

	if ( /*!m_bActive &&*/ !bActive && !bInMultiple )
		return;

	if ( m_pWindow )
	{
		if	(
			m_pWindow->getX()      != m_rect.nLeft+1   ||
			m_pWindow->getY()      != m_rect.nTop+1    ||
			m_pWindow->getWidth()  != m_rect.nWidth-2  ||
			m_pWindow->getHeight() != m_rect.nHeight-2
			)
		{
			m_pWindow->position(m_rect.nLeft+1, m_rect.nTop+1, m_rect.nWidth-2, m_rect.nHeight-2);
			m_pWindow->update();
			application->setDirty();
		}
	}
}

