/**************************************************************************
 *               This file is part of the FOX Dialog Editor               * 
 *                                                                        * 
 *      Copyright (C) 1999-2000 Martin Welch <mwelch@totalise.co.uk>      *
 *                                                                        * 
 *  This program is free software; you can redistribute it and/or modify  *
 *  it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or    *
 *                  (at your option) any later version.                   *
 *                                                                        * 
 *    This program is distributed in the hope that it will be useful,     *
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of      *
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the      *
 *             GNU General Public License for more details.               *
 *                                                                        * 
 *   You should have received a copy of the GNU General Public License    *
 *      along with this program; if not, write to the Free Software       *
 *       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.        *
 **************************************************************************/

#ifndef __CTracker_H__
#define __CTracker_H__

struct TRACKRECT
{
	FXbool bActive;
	FXshort nTop;
	FXshort nLeft;
	FXshort nHeight;
	FXshort nWidth;
};

class CTracker
{
// Services
public:
	CTracker();
	CTracker(CTracker& t);
	TRACKRECT operator [] (FXshort index) {return m_Grabbers[index];}

// Properties
public:
	FXshort top(void) {return m_rect.nTop;}
	FXshort left(void) {return m_rect.nLeft;}
	FXshort height(void) {return m_rect.nHeight;}
	FXshort width(void) {return m_rect.nWidth;}
	FXWindow* window(void) {return m_pWindow;}
	FXWindow* frame(void) {return m_pFrame;}

// Operations
public:
	void track(FXWindow* frame, FXWindow* window, FXint style);
	void track(FXbool bActive);
	void track(FXshort top, FXshort left);
	void move(FXint x, FXint y, bool bNormalise = true);
	void snap(FXshort nSnap);
	void release(void);
	void position(FXshort nTop, FXshort nLeft, FXshort nWidth, FXshort nHeight);
	void position(FXbool bInMultiple);

	FXbool isActive(void) {return m_bActive;}
	void setInactive(void) {m_bActive = false;}

	FXbool isGrabberAtPoint(FXWindow* pFrame, FXshort& x, FXshort& y);
	void setGrabber(eGrabber e) {m_SelectedGrabber = e;}
	TRACKRECT& getGrabber(eGrabber e) {return m_Grabbers[e];}

	void drawBoundary(FXDCWindow& dc);

	static void getPosition(FXWindow* frame, FXWindow* window, FXshort& x, FXshort& y);
	static void getAbsolutePosition(FXWindow* window, FXshort& x, FXshort& y);
	static void snap(FXshort& value, FXshort nSnap);

// Attributes
private:
	TRACKRECT m_Grabbers[9];

	TRACKRECT	m_rect;
	FXWindow*	m_pWindow;
	FXWindow*   m_pFrame;
	eGrabber	m_SelectedGrabber;

	FXbool m_bActive;
};


#endif // __CTracker_H__
